/**
 * Title:	    Traces
 * Description: An analysis tool for traceroutes
 * Copyright:	Copyright (c) Jan-Pascal van Best
 * Email:	    janb@tbm.tudelft.nl
 * @author	    Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.traces;


import java.util.*;
import java.util.logging.Logger;
import java.io.*;
import java.lang.*;
import java.net.*;

import nl.tudelft.tbm.graph.*;


public class AutSystem extends AbstractDatedNode implements Serializable {
    static boolean discrete = false; // true means: don't reveal AS names in XML

    public String name;
    public int number;
    List IPRanges;

    private static Logger logger = Logger.getLogger("nl.tudelft.tbm.traces.AutSystem");

    public AutSystem() {
        name = "";
        number = 0;
        IPRanges = new ArrayList();
    }

    public AutSystem(String name, int number) {
        this.name = name;
        this.number = number;
        IPRanges = new ArrayList();
    }

    public AutSystem(String name, int number, List ranges) {
        this.name = name;
        this.number = number;
        IPRanges = ranges;
    }

    public AutSystem(int number) {
        this.number = number;
        if (number == 0) {
            name = "unknown";
            IPRanges = new ArrayList();
        } else {
			logger.info("Getting whois data for AS "+number);
            IPRanges = Whois.getRangesFromNumberAll(number);
            name = Whois.getNameFromNumberAll(number);
        }
    }
    
    public AutSystem(InetAddress ip) {
        //	  System.out.println("Getting AS for "+ip.toString());
        IPRanges = new ArrayList();
        number = Whois.getNumberFromIPAll(ip, IPRanges);
        if (number == 0) {
            name = "unknown";
            //	      IPRanges = new ArrayList();
        } else {
            IPRanges.addAll(Whois.getRangesFromNumberAll(number));
            name = Whois.getNameFromNumberAll(number);
        }
    }

    public void addRange(IPRange range) {
        IPRanges.add(range);
    }

    public boolean containsIP(InetAddress ip) {
        boolean found = false;

        for (Iterator i = IPRanges.iterator(); i.hasNext() && !found;) {
            IPRange range = (IPRange) i.next();

            found = range.contains(ip);
        }
        return found;
    }

    public String toString() {
        return "" + number;
        //	  return "" + number + "@" + Integer.toHexString(hashCode());
        //	  String s = "AS" + number + " (" + name + ")";
        //	  for (Iterator i=IPRanges.iterator(); i.hasNext();) {
        //	      IPRange range= (IPRange)i.next();
        //	      s = s + " " + range.toString();
        //	  }
        //	  return s;
    }

    public String toLongString() {
        //	  String s = "AS" + number + " (" + name + ")";
        //	  for (Iterator i=IPRanges.iterator(); i.hasNext();) {
        //	      IPRange range= (IPRange)i.next();
        //	      s = s + " " + range.toString();
        //	  }
        return "AS" + number + " (" + name + ")\n" + IPRanges.toString();
    }

    // From interface Node
    public String getName() {
        return "" + number;
    }

    public String getLabel() {
        if (!discrete)
            return name;
        else
            return "" + number;
    }

    public List getPropertyNames() {
        List result = super.getPropertyNames();

        if (!discrete) {
            String[] stringResult = {"ASnumber", "ASname"};

            result.addAll(Arrays.asList(stringResult));
            return result;
        } else {
            String[] stringResult = {"ASnumber"};

            return result;
        }
    }

    public String getProperty(String propName) {
        String result;

        if ("ASnumber".equals(propName)) {
            result = "" + number;
        } else if ("ASname".equals(propName) && !discrete) {
            result = name;
        } else
            result = super.getProperty(propName);
        return result;
    }

    public void writeXML(PrintWriter out, String indent) throws IOException {
        out.println(indent + "<autsystem name=\"" + name.replace('&', '+') + "\" number=\"" + number + "\">");
        for (Iterator i = IPRanges.iterator(); i.hasNext();) {
            IPRange range = (IPRange) i.next();

            out.println(indent + "    <range>" + range.toString() + "</range>");
        }
        out.println(indent + "</autsystem>");
    }
}

