/**
 * Title:	Traces
 * Description: An analysis tool for traceroutes
 * Copyright:	Copyright (c) Jan-Pascal van Best
 * Email:	    janb@tbm.tudelft.nl
 * @author	    Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.traces;


import java.util.*;
import java.io.*;
import java.lang.*;
import java.net.*;
import java.sql.*;
import java.text.*;


public class Cleanup {
    public static String resultSetToString(ResultSet R) throws SQLException {
        String s = "";
	
        ResultSetMetaData M = R.getMetaData();
        int numberOfColumns = M.getColumnCount();

        for (int i = 1; i <= numberOfColumns; i++) {
            s = s + "\"" + M.getColumnName(i) + "\"";
            if (i != numberOfColumns) s = s + ",";
        }
        s = s + "\n";
        while (R.next()) {
            for (int i = 1; i <= numberOfColumns; i++) {
                String value = R.getString(i);

                s = s + "\"" + value + "\"";
                if (i != numberOfColumns) s = s + ",";
            }
            s = s + "\n";
        }
        return s;
    }
     
    public static boolean deleteDelayDuplicatesStep() throws Exception {
        String URL = "jdbc:mysql://130.161.216.187/TTM?user=janpascal&password=?????";

        boolean foundAny = true;

        Class.forName("org.gjt.mm.mysql.Driver").newInstance(); 		
        Connection C = DriverManager.getConnection(URL);
        Statement S = C.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
        Statement S2 = C.createStatement();
        Statement S3 = C.createStatement();

        System.out.println("Looking for up to 2000 non-unique (packetid,src) pairs...");
        String Q = "SELECT packetid,src,COUNT(*) FROM Delays\n" +
            "GROUP BY packetid,src\n" +
            "HAVING COUNT(*)>1\n" +
            "LIMIT 2000";
        ResultSet R = S.executeQuery(Q);

        foundAny = R.isBeforeFirst();
        if (foundAny) {
            System.out.print("Found some, deleting");
        } else {	
            System.out.println("No more duplicates found!");
        }
        String Q2;
        ResultSet R2;
        int deletedDuplicates = 0;

        while (R.next()) {
            long packetid = R.getLong(1);
            int src = R.getInt(2);
            int count = R.getInt(3);

            Q2 = "SELECT id FROM Delays WHERE packetid=" + packetid + " and src=" + src;
            R2 = S2.executeQuery(Q2);
            //	      System.out.print("Deleting "+(count-1)+" of "+count+" items with id,src="+packetid+","+src+": ");
            R2.next(); // We keep the first entry of the duplicates and delete the rest
            //	      long idkeep = R2.getLong(1);
            //	      System.out.println("Keeping id: "+idkeep);
            String Q3 = "DELETE FROM Delays WHERE "; // id="+previd;
            boolean first = true;

            while (R2.next()) {
                long id = R2.getLong(1);

                Q3 = Q3 + (first ? ("id=" + id) : (" or id=" + id));
                //		  System.out.print(",");
                deletedDuplicates++;
                first = false;
            }
            ResultSet R3 = S3.executeQuery(Q3);

            if (deletedDuplicates % 80 == 0) System.out.print(".");
        }
        C.close();
        System.out.println("Got rid of " + deletedDuplicates);
        return foundAny;
    }

    public static boolean deleteDelayDuplicatesStepQuicker() throws Exception {
        String URL = "jdbc:mysql://130.161.216.187/TTM?user=janpascal&password=?????";

        boolean foundAny = true;

        Class.forName("org.gjt.mm.mysql.Driver").newInstance(); 		
        Connection C = DriverManager.getConnection(URL);
        Statement S = C.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
        Statement S2 = C.createStatement();
        Statement S3 = C.createStatement();

        System.out.println("Looking for up to 2000 non-unique (packetid,src) pairs...");
        String Q = "SELECT packetid,src,COUNT(*) FROM Delays\n" +
            "GROUP BY packetid,src\n" +
            "HAVING COUNT(*)>1\n" +
            "LIMIT 2000";
        ResultSet R = S.executeQuery(Q);

        foundAny = R.isBeforeFirst();
        if (!foundAny) {
            System.out.println("No more duplicates found!");
            C.close();
            return false;
        } 
        String Q2;
        ResultSet R2;
        int deletedDuplicates = 0;
        StringWriter Q3 = new StringWriter(100 * 500);

        Q3.write("DELETE FROM Delays WHERE ");
        boolean first = true;

        while (R.next()) {
            long packetid = R.getLong(1);
            int src = R.getInt(2);
            int count = R.getInt(3);

            Q2 = "SELECT id FROM Delays WHERE packetid=" + packetid + " and src=" + src;
            R2 = S2.executeQuery(Q2);
            R2.next(); // We keep the first entry of the duplicates and delete the rest
            long idkeep = R2.getLong(1);

            while (R2.next()) {
                long id = R2.getLong(1);

                Q3.write(first ? ("id=" + id) : (" or id=" + id));
                deletedDuplicates++;
                first = false;
            }
            if (deletedDuplicates % 80 == 0) System.out.print(".");
        }
        System.out.print("Deleting...");
        ResultSet R3 = S3.executeQuery(Q3.toString());

        //	  System.out.println(Q3.toString());
        C.close();
        System.out.println("Got rid of " + deletedDuplicates);
        return foundAny;
    }

    public static void deleteDelayDuplicates() throws Exception {
        System.out.println("Cleaning up duplicates in Delays table");
        boolean goOn = true;
        int count = 0;

        while (goOn) {
            //	      System.out.println("Starting old method: "+(new java.util.Date()).toString());
            //	      goOn = deleteDelayDuplicatesStep();
            //	      count++;
            //	      if (goOn) {
            //		  System.out.println("Starting new method: "+(new java.util.Date()).toString());
            goOn = deleteDelayDuplicatesStepQuicker();
            //		  goOn = false;
            count++;
            //		  System.out.println("Finished new method: "+(new java.util.Date()).toString());
            //	      }
	    
        }
    }
    
    public static void main(String[] args) {
        try {
            deleteDelayDuplicates();
        } catch (Throwable t) {
            t.printStackTrace();
        }
        System.exit(0);
    }
}

