/**
 * Title:	Traces
 * Copyright:	Copyright (c) Jan-Pascal van Best
 * Email:	    janpascal@vanbest.org
 * @author	    Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.traces;


import java.util.*;
import java.io.*;
import java.lang.*;
import org.xml.sax.*;
import nl.tudelft.tbm.graph.*;


public class DelayPair extends Edge {
    public double delay; // in milliseconds

    private Set dates = new HashSet();

    public DelayPair(Object x, Object y) {
        super(x, y);
    }

    public void merge(Edge e) {
        super.merge(e);
        if (e instanceof DelayPair) {
            addDates(((DelayPair)e).getDates());
            // How to merge delays? I would not know
        }
    }
    
    public void addDate(Date date) {
        dates.add(date);
    }

    public void addDates(Set dates) {
        this.dates.addAll(dates);
    }

    public Set getDates() {
        return dates;
    }

    private String datesString() {
        StringBuffer result = new StringBuffer();
        boolean first = true;

        for (Iterator i = dates.iterator(); i.hasNext();) {
            Date date = (Date) i.next();

            result.append(first ? "" + date.getTime() / 1000 : " " + date.getTime() / 1000);
            first = false;
        }
        return result.toString();
    }

    public String getName() {
        return "";
    }

    public String getLabel() {
        return super.getLabel(); // x.toString() + "->" + y.toString();
    }

    public List getPropertyNames() {
        String[] strings = {"dates", "delay"};
        List result = super.getPropertyNames();

        result.addAll(Arrays.asList(strings));
        return result;
    }

    public String getProperty(String name) {
        String result;

        if (name.equals("dates")) {
            result = datesString();
        } else if (name.equals("delay")) {
            result = "" + delay;
        } else
            result = super.getProperty(name);
        return result;
    }

    /*
     static String getElementName() {return "edge";};

     public void writeXML(PrintWriter out, String indent) throws IOException{
     String name = getName();
     if (name!=null) name=name.replace('&','+');
     String label = getLabel();
     if (label!=null) label=label.replace('&','+');
     String from = (x instanceof Attributed) ? ((Attributed)x).getName() : x.toString();
     String to = (y instanceof Attributed) ? ((Attributed)y).getName() : y.toString();
     from = from.replace('&','+'); // XML does not like ampersands
     to = to.replace('&','+');
     if (name!=null && !name.equals("")) {
     out.println(indent+"<edge name=\""+name+"\" source=\""+from+"\" target=\""+to+"\">" );
     } else {
     out.println(indent+"<edge source=\""+from+"\" target=\""+to+"\">" );
     }
     if (label != null && !label.equals("")) {
     out.println(indent+"    <label>"+label+"</label>" );
     }
     List properties = getPropertyNames();
     for (Iterator i=properties.iterator(); i.hasNext(); ) {
     String pname = (String) i.next();
     String pvalue = getProperty(pname);
     if (pvalue!=null) {
     pvalue = pvalue.replace('&','+');
     out.println(indent+"	<property name=\""+pname+"\" content=\""+pvalue+"\"/>" );
     }
     }
     out.println(indent+"</edge>");
     }

     public static XMLSerializable createObject(Attributes attributes, String contents, List subElements) throws Exception {
     String xName = attributes.getValue("source");
     String yName = attributes.getValue("target");
    //	  System.out.println("	  Names: "+xName+"->"+yName);
    Object x = XMLSerializer.getObject(xName);
    Object y = XMLSerializer.getObject(yName);
    DelayPair result = new DelayPair(x,y);
    String dates = attributes.getValue("dates"); // FIXME: WRONG!!! It's a <property>
    String delay = attributes.getValue("delay");
    if (dates!=null) {
    StringTokenizer st = new StringTokenizer(dates);
    while (st.hasMoreTokens()) {
    long date = Long.parseLong(st.nextToken());
    result.addDate(new Date(date));
    }
    }
    if (delay!=null) {
    result.delay = Double.parseDouble(delay);
    }
    // FIXME: handle <label> and <property>
    //	  System.out.println("	  Creating edge: "+x.toString()+"->"+y.toString());
    return result;
    }
    */

    
     public String toString() {
         return super.toString()+"@"+delay;
     }
     
}

