/**
 *    Title:	   Traces
 *    Description: An analysis tool for traceroutes
 *    Copyright:   Copyright (c) Jan-Pascal van Best
 *    Email:	   janb@tbm.tudelft.nl
 *    Author	   Jan-Pascal van Best
 **************************************************************************
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package nl.tudelft.tbm.traces;


import java.util.*;
import java.lang.*;
import java.io.*;
import org.xml.sax.*;

import nl.tudelft.tbm.graph.*;


public class EndToEnd implements Attributed {
    public final static double TIME_UNKNOWN = -1;

    double time = TIME_UNKNOWN; // Time in milliseconds
    Date date;
    public Vector hops;

    public EndToEnd() {
        hops = new Vector();
        date = new Date();
    }

    public EndToEnd(Vector hops, double time) {
        this.hops = hops;
        this.time = time;
        this.date = new Date();
    }

    public EndToEnd(Vector hops, double time, Date date) {
        this.hops = hops;
        this.time = time;
        this.date = date;
    }

    public EndToEnd(EndToEnd d) {
        this.hops = d.hops;
        this.time = d.time;
        this.date = d.date;
    }

    public Vector getHops() {
        return hops;
    }

    public Vector addHop(Object hop) {
        hops.add(hop);
        return hops;
    }

    public void setTime(double time) {
        this.time = time;
    }

    public double getTime() {
        return time;
    }

    public void setDate(Date date) {
        this.date = date;
    }

    public Date getDate() {
        return date;
    }

    public String getName() {
        return null;
    }

    public String getLabel() {
        return null;
    }

    public List getPropertyNames() {
        String[] strings = {"date", "delay"};

        return Arrays.asList(strings);

    }

    public String getProperty(String name) {
        String result;

        if (name.equals("delay")) {
            result = "" + time;
        } else if (name.equals("date")) {
            result = (date == null ? "" : ("" + date.getTime() / 1000));
        } else
            result = null;
        return result;
    }

    public String toString() {
        return hops.toString() + " in " + time + " ms";
    }
}

