/**
 * Title:        Traces
 * Description:  An analysis tool for traceroutes
 * Copyright:    Copyright (c) Jan-Pascal van Best
 * Email:	 janb@sepa.tudelft.nl
 * @author 	 Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.traces;


import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;


public class FileDialog implements WindowListener, ActionListener {
    public int traceFormat;
    public String name;
    public String path;
    public String[] files;

    private Dialog dialog;

    private java.awt.List liFiles;
    private Choice chFormat;

    public FileDialog(String name, String path) {
        this.name = name;
        this.path = path;

        files = null;
        traceFormat = Trace.TRACE_FORMAT_LINUX;
    }

    public String[] getFiles() {
        return files;
    }

    /** Creates the configuration dialog box.
     */
    public int createDialog() {

        Dimension dim;

        Frame f = new Frame();

        // Create a dialog
        dialog = new Dialog(f, name);
        dialog.setResizable(true);
        dialog.setModal(true);
        dialog.setBackground(SystemColor.control);

        dialog.setLayout(new GridBagLayout());

        // Create a constraints object, and specify some default values
        GridBagConstraints c = new GridBagConstraints();

        c.fill = GridBagConstraints.BOTH;   // components grow in both dimensions
        c.insets = new Insets(5, 5, 5, 5); // 5-pixel margins on all sides

        // Create the components
        chFormat = new Choice();
        chFormat.addItem("Linux format");
        chFormat.addItem("Windows NT format");

        File root = new File(path);

        if (!root.isDirectory()) {
            liFiles = new java.awt.List(5, true);
            liFiles.add(root.getPath());
        } else {
            liFiles = new java.awt.List(15, true);
            String[] list = root.list();

            for (int i = 0; i < list.length; i++) {
                liFiles.add(root.getAbsolutePath() + File.separator + list[i]);
            } 
        }

        Button bok = new Button("OK");

        bok.addActionListener(this);
        Button bcancel = new Button("Cancel");

        bcancel.addActionListener(this);

        int line = -1;

        c.gridx = 0;
        c.gridy = ++line;
        c.gridwidth = 2;
        c.gridheight = 1;
        c.weightx = 1.0;
        c.weighty = 0.0;
        dialog.add(liFiles, c);

        c.gridx = 0;
        c.gridy = ++line;
        c.gridwidth = 2;
        c.gridheight = 1;
        c.weightx = 1.0;
        c.weighty = 0.0;
        dialog.add(chFormat, c);

        c.gridx = 0;
        c.gridy = ++line;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.weightx = 1.0;
        c.weighty = 0.0;
        dialog.add(bok, c);

        c.gridx = 1;
        c.gridy = line;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.weightx = 1.0;
        c.weighty = 0.0;
        dialog.add(bcancel, c);

        // Size the dialog
        dim = dialog.getToolkit().getScreenSize();
        dialog.invalidate();
        dialog.setSize(500, 600);
        dialog.validate();

        dialog.setResizable(true);
        dialog.pack();
        dialog.setLocation(dim.width / 2, dim.height / 2);
        dialog.addWindowListener(this);
        dialog.setResizable(true);

        dialog.show();

        return 0;
    }

    public void actionPerformed(ActionEvent e) {

        Button b = (Button) e.getSource();

        if (b.getLabel().equals("Cancel")) {
            dialog.dispose();
            return;
        } else if (b.getLabel().equals("Help")) {
            // if help, pop up text window with help info

        } else if (b.getLabel().equals("OK")) {
            files = liFiles.getSelectedItems();
            switch (chFormat.getSelectedIndex()) {
            case 0:
                traceFormat = Trace.TRACE_FORMAT_LINUX; 
                break;

            case 1:
                traceFormat = Trace.TRACE_FORMAT_WINNT; 
                break;
            }
            dialog.dispose();
            return;
        }
    }

    public void windowOpened(WindowEvent e) {
    }

    public void windowClosing(WindowEvent e) {
        // Dialog dismissed, don't save changes
        dialog.dispose();
    }

    public void windowClosed(WindowEvent e) {
    }

    public void windowIconified(WindowEvent e) {
    }

    public void windowDeiconified(WindowEvent e) {
    }

    public void windowActivated(WindowEvent e) {
    }

    public void windowDeactivated(WindowEvent e) {
    }

}
