/**
 * Title:	 Traces
 * Description:  An analysis tool for traceroutes
 * Copyright:	 Copyright (c) Jan-Pascal van Best
 * Email:	     janb@tbm.tudelft.nl
 * @author	     Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.traces;


import java.util.*;
import java.util.logging.*;
import java.io.*;
import java.lang.*;
import java.net.*;


public class Hop extends Host {
    public final static double MS_UNKNOWN = -1;
    private static Logger logger = Logger.getLogger("nl.tudelft.tbm.traces");
    double ms = MS_UNKNOWN; // milliseconds to here

    public Hop(String ipString, String hostName, double ms) {
        super(ipString, hostName);
        this.ms = ms;
    }
    
    public Hop(String s, int traceFormat) {
        //	  logger.info("in Hop constructor: traceformat: "+traceFormat);
        //	  logger.info(traceFormat == Trace.TRACE_FORMAT_WINNT);
        switch (traceFormat) {
        case Trace.TRACE_FORMAT_LINUX:
            parseLinuxHop(s);
            break;

        case Trace.TRACE_FORMAT_WINNT:
            parseWinntHop(s);
            break;
        }
    }

    void parseWinntHop(String s) {
        // fetch milliseconds
        double temp = 0;
        int N = 0;

        int pos3 = 0;
        int pos1;

        while (N < 3 && pos3 >= 0) {
            pos3 = s.indexOf(" ms", pos3 + 1);
            if (pos3 >= 0) {
                pos1 = s.lastIndexOf(" ", pos3 - 1);
                int pos1a = s.lastIndexOf("<", pos3 - 1);

                if (pos1a > pos1) pos1 = pos1a;
                try {
                    logger.info("+" + s.substring(pos1 + 1, pos3) + "+");
                    temp = temp + Double.parseDouble(s.substring(pos1 + 1, pos3));
                    N++;
                } catch (NumberFormatException e) {
                }
            }
        }
        if (N > 0)
            ms = temp / N;
        else
            ms = (double) 0.0;

        logger.info(ms + " ms");
	
        pos3 = s.indexOf("]");
        int pos2 = s.indexOf("[");

        if (pos2 >= 0) {
            pos1 = s.lastIndexOf(" ", pos2 - 2);
            set(s.substring(pos2 + 1, pos3), s.substring(pos1 + 1, pos2 - 1));
        } else {
            pos1 = s.lastIndexOf(" ", s.length() - 2);
            logger.info("Weird address: *" + s + "* (" + pos1 + "," + pos2 + "), using \"" + s.substring(pos1 + 1, s.length() - 1) + "\"");
            set(s.substring(pos1 + 1, s.length() - 1), null);
        }

    }

    void parseLinuxHop(String s) {
        //	  logger.info(s);
        // fetch ip address and host name
        int pos3 = s.indexOf(")");
        int pos2 = s.indexOf("(");
        int pos1;

        if (pos2 >= 0) {
            pos1 = s.lastIndexOf(" ", pos2 - 2);
            // host = new Host(s.substring(pos2+1,pos3), s.substring(pos1+1, pos2-1));
            set(s.substring(pos2 + 1, pos3), s.substring(pos1 + 1, pos2 - 1));
        } else { 
            // host = new Host("0.0.0.0", "unknown");
            set("0.0.0.0", "unknown");
        }

        // fetch milliseconds
        double temp = 0;
        int N = 0;

        while (pos3 >= 0) {
            pos3 = s.indexOf(" ms", pos3 + 1);
            if (pos3 >= 0) {
                pos1 = s.lastIndexOf(" ", pos3 - 1);
                try {
                    //		      logger.info(s.substring(pos1+1, pos3));
                    temp = temp + Double.parseDouble(s.substring(pos1 + 1, pos3));
                    N++;
                } catch (NumberFormatException e) {
                }
            }
        }
        if (N > 0)
            ms = temp / N;
        else
            ms = (double) 0.0;

        //	  logger.info(ms + " ms");
    }

    public String toString() {
        return super.toString(); //  + "("+ms+" ms)";
    }
}

