/**
 * Title:	 Traces
 * Description:  An analysis tool for traceroutes
 * Copyright:	 Copyright (c) Jan-Pascal van Best
 * Email:	    janb@sepa.tudelft.nl
 * @author	    Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.traces;


import java.util.*;
import java.io.*;
import java.lang.*;
import java.net.*;
import nl.tudelft.tbm.graph.*;


public class Host extends AbstractDatedNode implements Serializable {
    public InetAddress ip;
    public String hostName;
    public AutSystem AS = null;

    public Host() {
        ip = null;
        hostName = null;
    }

    public Host(InetAddress ip, String hostName) {
        this.ip = ip;
        if (hostName == null) {
            hostName = ip.getHostName();
            if (hostName == null) hostName = ip.getHostAddress();
        }
        this.hostName = hostName;
    }

    public Host(Host host) {
        this.ip = host.ip;
        this.hostName = host.hostName;
        this.AS = host.AS;
    }

    public Host(String ipString, String hostName) {
        set(ipString, hostName);
    }

    public void set(String ipString, String hostName) {
        ip = null;
        try {
            ip = InetAddress.getByName(ipString);
        } catch (UnknownHostException e) {
            try {
                ip = InetAddress.getByName("0.0.0.0");
            } catch (UnknownHostException f) {
            }
        }
        if (hostName != null)
            this.hostName = hostName;
        else {
            //	      this.hostName = ipString;
            this.hostName = ip.getHostName();
            if (this.hostName == null) this.hostName = ipString;
        }
    }

    public static Host getLocalHost() {
        InetAddress ip = null;

        try {
            ip = InetAddress.getLocalHost();
        } catch (UnknownHostException e) {
            try {
                ip = InetAddress.getByName("127.0.0.1");
            } catch (UnknownHostException f) {
            }
        }

        return new Host(ip, ip.getHostName());
    }

    public InetAddress getIp() {
        return ip;
    }

    public String getHostName() {
        return hostName;
    }

    public boolean equals(Object other) {
        boolean result = false;

        if (other instanceof Host) {
            result = ip.equals(((Host) other).ip);
        }
        //	  System.out.println("Comparing Hosts: "+this+" to "+other+"; result = "+result);
        return result;
    }

    public int hashCode() {
        return ip.hashCode();
    }

    //    public boolean equals(Host other) { return ip.equals(other.ip); }

    public String toString() {
        //	  return hostName + " (" + ip.getHostAddress() + ")";
        return ip.getHostAddress();
        //	  return hostName+"@"+Integer.toHexString(System.identityHashCode(this));
    }

    public String getName() {
        return ip.getHostAddress();
    }

    public String getLabel() {
        return hostName;
    }

    public List getPropertyNames() {
        String[] strings = {"IPaddress", "hostname", "ASnumber", "ASname"};
        List result = super.getPropertyNames();

        result.addAll(Arrays.asList(strings));
        return result;
    }

    public String getProperty(String name) {
        String result;

        if (name.equals("IPaddress")) {
            result = ip.getHostAddress();
        } else if (name.equals("hostname")) {
            result = hostName == null ? "unknown" : hostName;
        } else if (name.equals("ASnumber")) {
            result = AS == null ? "-1" : "" + AS.number;
        } else if (name.equals("ASname")) {
            result = AS == null ? "unknown" : AS.name;
        } else
            result = super.getProperty(name);
        return result;
    }

    /* NOT FINISHED
     static String getElementName() {return "node";};
     public void writeXML(PrintWriter out, String indent) throws IOException{
     String name = getName().replace('&','+');
     String label = getLabel().replace('&','+');
     out.println(indent+"<node name=\""+name+"\">\n";
     if (label != null && !label.equals("")) {
     s = s + "		 <label>"+label+"</label>\n";
     }
     if ((n.getX() != Node.POSITION_UNKNOWN) && (n.getY() != Node.POSITION_UNKNOWN)) {
     s = s + "		 <position x=\""+n.getX()+"\" y=\""+n.getY()+"\"/>\n";
     }
     List properties = n.getPropertyNames();
     for (Iterator i=properties.iterator(); i.hasNext(); ) {
     String pname = (String) i.next();
     String pvalue = n.getProperty(pname);
     if (pvalue!=null) {
     pvalue = pvalue.replace('&','+');
     s = s + "	     <property name=\""+pname+"\" content=\""+pvalue+"\"/>\n";
     }
     }
     s = s + "	     <size width=\"10.0\" height=\"1.0\"/>\n";
     s = s + "	 </node>\n";
     out.println(indent+"<node name=\""+getName()+"\"/>");
     }
     public static XMLSerializable createObject(Attributes attributes, String contents, List subElements) throws Exception {
     return new NodeImpl(attributes.getValue("name"));
     }
     */
}

