/**
 * Title:	 Traces
 * Description:  An analysis tool for traceroutes
 * Copyright:	 Copyright (c) Jan-Pascal van Best
 * Email:	     janb@tbm.tudelft.nl
 * @author	     Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.traces;


import java.util.*;
import java.io.*;
import java.lang.*;
import java.net.*;


public class IPRange implements Serializable {
    public InetAddress network;
    public int prefix;

    public IPRange(InetAddress network, int prefix) {
        this.network = network;
        this.prefix = prefix;
    }

    public IPRange(String range) throws NumberFormatException, UnknownHostException {
        int slash = range.indexOf("/");

        //	  logger.info("1. "+range + " " + slash);
        this.network = InetAddress.getByName(range.substring(0, slash));
        this.prefix = Integer.parseInt(range.substring(slash + 1, range.length()));
        //	  logger.info(toString());
    }

    public boolean contains(InetAddress ip) {
        byte[] net = network.getAddress();
        byte[] addr = ip.getAddress();

        boolean ok = true;
        boolean finished = false;

        //	  logger.info(toString());
        //	  logger.info(ip.getHostAddress());
        for (int i = 0; i < 4 && ok && !finished; i++) {
            finished = (prefix <= i * 8);
            if (!finished) {
                if (prefix >= i * 8 + 8) {
                    ok = (net[i] == addr[i]);
                    //		      System.out.print("255.");
                } else {
                    ok = (net[i] == (addr[i] & prefixToMask(prefix - i * 8)));
                    //		      System.out.print(prefixToMask(prefix-i*8));
                }
            }
        }
        //	  logger.info("");
        return ok;
    }

    public InetAddress getNetwork() {
        return network;
    }

    public int getPrefix() {
        return prefix;
    }

    byte prefixToMask(int prefix) {
        return (byte) (255 - ((1 << (8 - prefix)) - 1));
    }

    public String toString() {
        return network.getHostAddress() + "/" + prefix;
    }

    public boolean equals(Object o) {
        if (o instanceof IPRange) {
            IPRange r = (IPRange) o;

            return network.equals(r.network) && (prefix == r.prefix);
        } else {
            return false;
        }
    }

    /**
     * Returns true for
     * 0.0.0.0
     * 255.255.255.255
     * 127.x.x.x
     * 10.x.x.x
     * 172.0001xxxx.x.x
     * 192.168.x.x
     */
    public static boolean isBogusIPAddress(InetAddress ip) {
        byte[] bytes = ip.getAddress();

        return (((bytes[0] == 0) && (bytes[1] == 0) && (bytes[2] == 0) && (bytes[3] == 0)) ||
                ((bytes[0] == -1) && (bytes[1] == -1) && (bytes[2] == -1) && (bytes[3] == -1)) ||
                (bytes[0] == 127) ||
                (bytes[0] == 10) ||
                ((bytes[0] == 172 - 256) && ((bytes[1] & (240 - 256)) == 16)) ||
                ((bytes[0] == 192 - 256) && (bytes[1] == 168 - 256)));
    }

}

