/**
 * Title:	Traces
 * Description: An analysis tool for traceroutes
 * Copyright:	Copyright (c) Jan-Pascal van Best
 * Email:	    janb@tbm.tudelft.nl
 * @author	    Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.traces;


import java.util.*;
import java.io.*;
import java.lang.*;
import java.sql.*;
import java.text.*;
import java.awt.*;


public class ImportDelays {

    public static String resultSetToString(ResultSet R) throws SQLException {
        String s = "";
	
        ResultSetMetaData M = R.getMetaData();
        int numberOfColumns = M.getColumnCount();

        for (int i = 1; i <= numberOfColumns; i++) {
            s = s + "\"" + M.getColumnName(i) + "\"";
            if (i != numberOfColumns) s = s + ",";
        }
        s = s + "\n";
        while (R.next()) {
            for (int i = 1; i <= numberOfColumns; i++) {
                String value = R.getString(i);

                s = s + "\"" + value + "\"";
                if (i != numberOfColumns) s = s + ",";
            }
            s = s + "\n";
        }
        return s;
    }

    public static void importDelay(String fileName) throws Exception {
        String URL = "jdbc:mysql://130.161.216.187/TTM?user=janpascal&password=????";

        System.out.println("Adding delay data from " + fileName + " to MySQL database");
        Connection C = null;

        Class.forName("org.gjt.mm.mysql.Driver").newInstance();
        C = DriverManager.getConnection(URL);
        Statement S = C.createStatement();

        BufferedReader reader = new BufferedReader(new FileReader(fileName));

        reader.readLine(); // Skip header line
        int count = 0;

        while (reader.ready()) {
            StringWriter query = new StringWriter(100 * 500);

            query.write("INSERT INTO Delays (packetid,src,srcport,dst,dstport,packetsize,arrivaltime,packetdelay,srcclock,dstclock,hops,routeid,srcntp,dstntp) VALUES ");
            for (int i = 0; i < 10 && reader.ready(); i++) {
                StringTokenizer tok = new StringTokenizer(reader.readLine());

                tok.nextToken(); // skip "i"-column
                if (i == 0) {
                    query.write("(");
                } else {
                    query.write(",(");
                }
                while (tok.hasMoreTokens()) {
                    query.write(tok.nextToken());
                    if (tok.hasMoreTokens()) query.write(",");
                }
                query.write(")");
            }
            //	      System.out.print("Executing..");
            int result = S.executeUpdate(query.toString());

            //	      System.out.println(" Finished!");
            count++;
            if (count % 50 == 0) {
                System.out.print("."); // every 500 updates
                if (count % 2500 == 0) System.out.println((10 * count) + ""); // every 25000=500*50 updates
            }
        }
        System.out.println();
        reader.close();
        C.close();
    }

    public static void importDelay(int box) throws Exception {
        String boxname = (box < 10) ? "tt0" + box + ".ripe.net" : "tt" + box + ".ripe.net";

        importDelay("e:\\src\\root\\20010625\\" + boxname + ".20010625.txt");
    }

    public static void importDelays() throws Exception {
        for (int i = 1; i < 100; i++) {
            importDelay(i);
            Toolkit.getDefaultToolkit().beep();
            Thread.sleep(5000); // wait 5 seconds
        }
    }

    public static void main(String[] args) {
        try {
            importDelays();
        } catch (Throwable t) {
            t.printStackTrace();
        }
        System.exit(0);
    }
}

