// This progarm is intended to query and display  from remote MySql database server
// that holds the TTM database file.

package nl.tudelft.tbm.traces;


import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.URL.*;
import java.sql.*;
import java.text.*;
import java.util.*;
import java.util.prefs.*;
import javax.swing.*;
import java.util.logging.Logger;

import nl.tudelft.tbm.graph.*;


public class ImportDialog extends JDialog {
    static public final int RESULT_CANCEL = 0;
    static public final int RESULT_OK = 1;

    static public final int QUERY_FULL = 1;
    static public final int QUERY_PARTIAL = 2;

    static final int MAX_NUMBERS_PER_COLUMN = 15;
    //    static public final int QUERY_SINGLE = 3;

    public int returnValue;

    public Set dates; // contains java.util.Date
	public int interval = 12*60; // interval width in seconds around dates

    public int queryType;
    public boolean useDNS;
    public boolean readDelays;

    public SortedSet from = new TreeSet();  // contains Integer
    public SortedSet to = new TreeSet();    // contains Integer

    OptionsDialog options;

    ButtonGroup bgQueryType;
    JRadioButton rbQFull;
    JRadioButton rbQPartial;

    JButton bSelectFromTo;
    JLabel lSelected;
    //    JRadioButton rbQSingle;

    JSpinner spStartDate;
    JSpinner spEndDate;
    JSpinner spIntervalNumber;
    JSpinner spIntervalUnit;
    JSpinner spMaxDates;
	JSpinner spQueryWidth;

    //    JSpinner spTestboxFrom;
    //    JSpinner spTestboxTo;

    JCheckBox cbDoNoDull;
    JCheckBox cbUseDNS;
    JCheckBox cbReadDelays;

    JButton bOK;
    JButton bCancel;

    boolean busy = false;

    static String[] intervalStrings = {"minutes", "hours", "days", "weeks", "months", "years"};

    ImportDialog self = this;

    private static Logger logger = Logger.getLogger("nl.tudelft.tbm.traces");

	public ImportDialog(Frame owner, OptionsDialog options) {
        // Create a modal dialog
        super(owner, "Import data from test-traffic database", true);
        this.options = options;

        Container c = getContentPane();

        c.setLayout(new GridBagLayout());

        // Create a constraints object, and specify some default values
        GridBagConstraints g = new GridBagConstraints();

        g.fill = GridBagConstraints.BOTH;   // components grow in both dimensions
        g.insets = new Insets(5, 5, 5, 5); // 5-pixel margins on all sides

        bgQueryType = new ButtonGroup();
        rbQFull = new JRadioButton("All test boxes", true);
        rbQPartial = new JRadioButton("Subset of boxes", false);
        //	  rbQSingle = new JRadioButton("Single trace", true);
        bgQueryType.add(rbQFull);
        bgQueryType.add(rbQPartial);
        //	  bgQueryType.add(rbQSingle);

        spStartDate = new JSpinner(new SpinnerDateModel(new java.util.Date(99, 8, 1, 20, 0),
                        new java.util.Date(99, 0, 1, 0, 0), new java.util.Date(), Calendar.DAY_OF_MONTH));
        spEndDate = new JSpinner(new SpinnerDateModel(new java.util.Date(99, 8, 1, 20, 0),
                        new java.util.Date(99, 0, 1, 0, 0), new java.util.Date(), Calendar.DAY_OF_MONTH));

        spIntervalNumber = new JSpinner(new SpinnerNumberModel(1, 1, 1000, 1));
        spIntervalUnit = new JSpinner(new SpinnerListModel(intervalStrings));
        spMaxDates = new JSpinner(new SpinnerNumberModel(100, 1, Integer.MAX_VALUE, 1));
		spQueryWidth = new JSpinner(new SpinnerNumberModel(interval, 0, Integer.MAX_VALUE, 100));

        bSelectFromTo = new JButton("Select...");
        bSelectFromTo.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bSelectFromTo) {
                        rbQPartial.setSelected(true);
                        SelectFromTo();
                        lSelected.setText("Src: " + calculateBoxesString(from) + "; Dst: " + calculateBoxesString(to));
                    }
                }
            }
        );

        lSelected = new JLabel("Src: Dst: ");

        cbUseDNS = new JCheckBox("Use DNS (may be slow)", false);
        cbReadDelays = new JCheckBox("Read Delays (EXPERIMENTAL)", false);

    //	  spTestboxFrom = new JSpinner(new SpinnerNumberModel(1, 1, 99, 1));
        //	  spTestboxTo = new JSpinner(new SpinnerNumberModel(1, 1, 99, 1));

        bOK = new JButton("OK");
        bOK.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bOK) {
                        calculateInput();
                        self.returnValue = RESULT_OK;
                        hide();
                    }
                }
            }
        );
        bCancel = new JButton("Cancel");
        bCancel.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bCancel) {
                        self.returnValue = RESULT_CANCEL;
                        hide();
                    }
                }
            }
        );

        int line = -1;

        JLabel lDate2 = new JLabel("Start date: ");

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(lDate2, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spStartDate, g);

        JLabel lDate3 = new JLabel("End date: ");

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(lDate3, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spEndDate, g);

        JLabel lInterval = new JLabel("Interval: ");

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(lInterval, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spIntervalNumber, g);

        g.gridx = 2;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spIntervalUnit, g);

        g.gridx = 0; g.gridy = ++line; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 0.0;
        c.add(new JLabel("Time width (seconds)"), g);
        g.gridx = 1; g.gridy = line; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 0.0;
        c.add(spQueryWidth, g);
		
		JLabel lmaxdates = new JLabel("Max number of dates: ");

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(lmaxdates, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spMaxDates, g);

        JLabel lQType = new JLabel("Query type: ");

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(lQType, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(rbQFull, g);

        g.gridx = 1;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(rbQPartial, g);

        g.gridx = 2;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(bSelectFromTo, g);

        g.gridx = 1;
        g.gridy = ++line;
        g.gridwidth = 2;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(lSelected, g);

        g.gridx = 1; g.gridy = ++line; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 0.0;
        c.add(cbUseDNS, g);

        g.gridx = 1; g.gridy = ++line; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 0.0;
        c.add(cbReadDelays, g);
        g.gridx = 0; g.gridy = ++line; g.gridwidth = 2; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 0.0;
        c.add(new JLabel("Only delays from 25th June, 2001 are in database!"), g);

        //	  g.gridx = 1; g.gridy = ++line; g.gridwidth = 1; g.gridheight = 1;
        //	  g.weightx = 1.0; g.weighty = 0.0;
        //	  c.add(rbQSingle, g);

        //	   JLabel ltbFrom = new JLabel("From Testbox #: ");
        //	   g.gridx = 0; g.gridy = ++line; g.gridwidth = 1; g.gridheight = 1;
        //	   g.weightx = 1.0; g.weighty = 0.0;
        //	   c.add(ltbFrom, g);
        //
        //	   g.gridx = 1; g.gridy = line; g.gridwidth = 1; g.gridheight = 1;
        //	   g.weightx = 1.0; g.weighty = 0.0;
        //	   c.add(spTestboxFrom, g);
        //
        //	   JLabel ltbTo = new JLabel("To Testbox #: ");
        //	   g.gridx = 0; g.gridy = ++line; g.gridwidth = 1; g.gridheight = 1;
        //	   g.weightx = 1.0; g.weighty = 0.0;
        //	   c.add(ltbTo, g);
        //
        //	   g.gridx = 1; g.gridy = line; g.gridwidth = 1; g.gridheight = 1;
        //	   g.weightx = 1.0; g.weighty = 0.0;
        //	   c.add(spTestboxTo, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 3.0;
        g.weighty = 0.0;
        c.add(bOK, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 3.0;
        g.weighty = 0.0;
        c.add(bCancel, g);

        pack();
        spIntervalUnit.setValue("days");
    }

    void calculateInput() {
        java.util.Date startDate = (java.util.Date) spStartDate.getValue();
        java.util.Date endDate = (java.util.Date) spEndDate.getValue();
        String intervalString = (String) spIntervalUnit.getValue();
        int intervalNumber = ((Number) spIntervalNumber.getValue()).intValue();
		interval = ((Number) spQueryWidth.getValue()).intValue();
		int maxDates = ((Number) spMaxDates.getValue()).intValue();

        logger.info("Start, end, interval: " + startDate + ", " + endDate + ", " + intervalNumber + " " + intervalString);

        dates = new HashSet();
        Calendar cal = new GregorianCalendar();

        cal.setTime(startDate);
        Calendar finish = new GregorianCalendar();

        finish.setTime(endDate);
        int i = 0;

        while (!cal.after(finish) && i < maxDates) {
            dates.add(cal.getTime());
            if (intervalString.equals("minutes")) {
                cal.add(Calendar.MINUTE, intervalNumber);
            } else if (intervalString.equals("hours")) {
                cal.add(Calendar.HOUR_OF_DAY, intervalNumber);
            } else if (intervalString.equals("days")) {
                cal.add(Calendar.DATE, intervalNumber);
            } else if (intervalString.equals("weeks")) {
                cal.add(Calendar.DATE, 7 * intervalNumber);
            } else if (intervalString.equals("months")) {
                cal.add(Calendar.MONTH, intervalNumber);
            } else if (intervalString.equals("years")) {
                cal.add(Calendar.YEAR, intervalNumber);
            } else
                logger.info("OOPS: unknown interval unit: " + intervalString);
            i++;
        }

        if (i >= maxDates)
            logger.info("Cutting off date intervals after " + maxDates + " points.");

            //	  from = ((Number)spTestboxFrom.getValue()).intValue();
            //	  to = ((Number)spTestboxTo.getValue()).intValue();

        if (rbQFull.isSelected()) {
            queryType = QUERY_FULL;
        } else if (rbQPartial.isSelected()) {
            queryType = QUERY_PARTIAL;
            //	  } else if (rbQSingle.isSelected()) {
            //	      queryType = QUERY_SINGLE;
        } else {
            queryType = QUERY_FULL;
            logger.info("WARNING: no query type selected, this should be impossible...");
        }

        useDNS = cbUseDNS.isSelected();
        readDelays = cbReadDelays.isSelected();
    }

    String calculateBoxesString(SortedSet numbers) {
        StringBuffer s = new StringBuffer();
        Iterator i = numbers.iterator();

        if (!i.hasNext()) return "";
        int prev = ((Integer) i.next()).intValue();
        int start = prev;
        int box;

        while (i.hasNext()) {
            box = ((Integer) i.next()).intValue();
            if (box != prev + 1) {
                if (s.length() > 0) s.append(",");
                if (start == prev) s.append("" + start);
                else s.append(start + "-" + prev);
                start = box;
            }
            prev = box;
        }
        if (s.length() > 0) s.append(",");
        if (start == prev) s.append("" + start);
        else s.append(start + "-" + prev);

        return s.toString();
    }

    void SelectFromTo() {
        final SortedSet numbers = TestBox.getTestBoxNumbers();
        int numColumns = (int) Math.round(Math.ceil(((double) numbers.size()) / MAX_NUMBERS_PER_COLUMN));

        final JDialog d = new JDialog(this, "Select source and destination testboxes", true);

        Container c = d.getContentPane();

        c.setLayout(new GridBagLayout());

        GridBagConstraints g = new GridBagConstraints();

        g.fill = GridBagConstraints.BOTH;   // components grow in both dimensions
        g.insets = new Insets(2, 0, 2, 0); // 5-pixel margins on all sides

        final JCheckBox[] fromChecks = new JCheckBox[numbers.size()];
        final JCheckBox[] toChecks = new JCheckBox[numbers.size()];

        int count = 0;

        for (Iterator i = numbers.iterator(); i.hasNext(); count++) {
            Integer num = (Integer) i.next();
            int number = num.intValue();

            fromChecks[count] = new JCheckBox("" + number, from.contains(num));
            toChecks[count] = new JCheckBox("" + number, to.contains(num));
        }

        final JButton bSelectOK = new JButton("OK");

        bSelectOK.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bSelectOK) {
                        self.from = new TreeSet();
                        self.to = new TreeSet();
                        for (int i = 0; i < numbers.size(); i++) {
                            if (fromChecks[i].isSelected()) {
                                self.from.add(new Integer(fromChecks[i].getText()));
                            }
                            if (toChecks[i].isSelected()) {
                                self.to.add(new Integer(toChecks[i].getText()));
                            }
                        }
                        d.hide();
                    }
                }
            }
        );
        final JButton bSelectCancel = new JButton("Cancel");

        bSelectCancel.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bSelectCancel) {
                        d.hide();
                    }
                }
            }
        );

        int line = -1;

        JLabel lSource = new JLabel("Source testboxes");

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = numColumns;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(lSource, g);

        JLabel lDestination = new JLabel("Destination testboxes");

        g.gridx = numColumns;
        g.gridy = line;
        g.gridwidth = numColumns;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(lDestination, g);

        int column = 0;
        int row = 0;

        for (int i = 0; i < numbers.size(); i++) {
            g.gridx = column;
            g.gridy = 1 + row;
            g.gridwidth = 1;
            g.gridheight = 1;
            g.weightx = 1.0;
            g.weighty = 0.0;
            c.add(fromChecks[i], g);

            g.gridx = column + numColumns;
            g.gridy = 1 + row;
            g.gridwidth = 1;
            g.gridheight = 1;
            g.weightx = 1.0;
            g.weighty = 0.0;
            c.add(toChecks[i], g);
            row++;
            if (row == MAX_NUMBERS_PER_COLUMN) {
                row = 0;
                column++;
            }
        }

        if (numbers.size() > MAX_NUMBERS_PER_COLUMN)
            line = 1 + MAX_NUMBERS_PER_COLUMN;
        else
            line = 1 + row;

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = numColumns;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(bSelectOK, g);

        g.gridx = numColumns;
        g.gridy = line;
        g.gridwidth = numColumns;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(bSelectCancel, g);

        d.pack();
        d.show();
    }
}

