// This progarm is intended to query and display  from remote MySql database server
// that holds the TTM database file.

package nl.tudelft.tbm.traces;


import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.URL.*;
import java.security.*;
import java.sql.*;
import java.text.*;
import java.util.*;
import java.util.prefs.*;
import javax.swing.*;

import nl.tudelft.tbm.graph.*;


public class LayoutDialog extends JDialog {
    private static final String PREFS_VERSION = "prefs-version";
    private static final String PREFS_GAMMA = "gamma";
    private static final String PREFS_START_T = "startT";
    private static final String PREFS_END_T = "endT";
    private static final String PREFS_NODE_REPULSION = "nodeRepulsion";
    private static final String PREFS_BORDER_REPULSION = "borderRepulsion";
    private static final String PREFS_EDGE_STRETCH = "edgeStretch";
    private static final String PREFS_EDGE_CROSS_CUTOFF_T = "edgeCrossCutoff";
    private static final String PREFS_EDGE_CROSSING = "edgeCrossing";
    private static final String PREFS_NODE_EDGE_REPULSION = "nodeEdgeRepulsion";
    private static final String PREFS_USE_PARTIAL = "usePartial";
    private static final String PREFS_PARTIAL = "partial";
    private static final String PREFS_MERGE_BIDIR = "merge-bidir-edges";

    private static final int PREFS_CURRENT_VERSION = 2;
    private static final double PREFS_DEFAULT_GAMMA = 0.85;
    private static final double PREFS_DEFAULT_START_T = 20;
    private static final double PREFS_DEFAULT_END_T = 0.000000005;
    private static final double PREFS_DEFAULT_NODE_REPULSION = 1.0;
    private static final double PREFS_DEFAULT_BORDER_REPULSION = 1.0;
    private static final double PREFS_DEFAULT_EDGE_STRETCH = 0.00000001;
    private static final double PREFS_DEFAULT_EDGE_CROSS_CUTOFF_T = 0.000001;
    private static final double PREFS_DEFAULT_EDGE_CROSSING = 10.0;
    private static final double PREFS_DEFAULT_NODE_EDGE_REPULSION = 0.1;
    private static final int PREFS_DEFAULT_USE_PARTIAL = PartialLayout.GET_ALL_NODES;
    private static final double PREFS_DEFAULT_PARTIAL = 0.85;
    private static final boolean PREFS_DEFAULT_MERGE_BIDIR = true;

    OptionsDialog options;

    boolean returnValue;

    JFormattedTextField spGamma;
    JFormattedTextField spStartT;
    JFormattedTextField spEndT;
    JFormattedTextField spNodeRepulsion;
    JFormattedTextField spBorderRepulsion;
    JFormattedTextField spEdgeStretch;
    JFormattedTextField spEdgeCrossing;
    JFormattedTextField spNodeEdgeRepulsion;
    JFormattedTextField spFinetuneCutoff;
    JComboBox cbPartial;
    JFormattedTextField spPartial;
    JCheckBox cbMergeBidir;

    JButton bOK;
    JButton bCancel;

    public LayoutDialog(Frame owner, OptionsDialog options) {
        // Create a modal dialog
        super(owner, "Layout network", true);
        this.options = options;
        returnValue = false;

        Format format = new DecimalFormat("#########.##################");

        spGamma = new JFormattedTextField(format);
        spGamma.setValue(new Double(0.75));

        spStartT = new JFormattedTextField(format);
        spStartT.setValue(new Double(10.0));

        spEndT = new JFormattedTextField(format);
        spEndT.setValue(new Double(0.0000005));

        spNodeRepulsion = new JFormattedTextField(format);
        spNodeRepulsion.setValue(new Double(1.0));

        spBorderRepulsion = new JFormattedTextField(format);
        spBorderRepulsion.setValue(new Double(1.0));

        spEdgeStretch = new JFormattedTextField(format);
        spEdgeStretch.setValue(new Double(0.0001));

        spEdgeCrossing = new JFormattedTextField(format);
        spEdgeCrossing.setValue(new Double(1.0));

        spNodeEdgeRepulsion = new JFormattedTextField(format);
        spNodeEdgeRepulsion.setValue(new Double(1.0));

        spFinetuneCutoff = new JFormattedTextField(format);
        spFinetuneCutoff.setValue(new Double(0.0001));

        cbPartial = new JComboBox(PartialLayout.PARTIAL_STRINGS);
//        cbPartial = new JCheckBox("Only layout low-degree nodes", false);
        cbMergeBidir = new JCheckBox("Count bidir edges as one for annealing", true);

        spPartial = new JFormattedTextField(format);
//        spPartial.setValue(new Double(0.0001));

        bOK = new JButton("OK");
        bOK.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bOK) {
                        updatePreferences();
                        returnValue = true;
                        hide();
                    }
                }
            }
        );

        bCancel = new JButton("Cancel");
        bCancel.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bCancel) {
                        returnValue = false;
                        hide();
                    }
                }
            }
        );

        Container c = getContentPane();

        c.setLayout(new GridBagLayout());

        // Create a constraints object, and specify some default values
        GridBagConstraints g = new GridBagConstraints();

        g.fill = GridBagConstraints.BOTH;   // components grow in both dimensions
        g.insets = new Insets(5, 5, 5, 5); // 5-pixel margins on all sides

        int line = -1;

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(new JLabel("Cooling factor"), g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spGamma, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(new JLabel("Start Temperature"), g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spStartT, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(new JLabel("End Temperature"), g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spEndT, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(new JLabel("Node Repulsion factor"), g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spNodeRepulsion, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(new JLabel("Border Repulsion factor"), g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spBorderRepulsion, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(new JLabel("Edge Stretch factor"), g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spEdgeStretch, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(new JLabel("Edge Crossing cutoff temperature Tc"), g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spFinetuneCutoff, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(new JLabel("Edge Crossing factor (for T>Tc)"), g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spEdgeCrossing, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(new JLabel("Node-Edge Repulsion factor (for T<Tc)"), g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spNodeEdgeRepulsion, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(cbPartial, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(new JLabel("Part of nodes to keep in partial graph"), g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spPartial, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(cbMergeBidir, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(bOK, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(bCancel, g);

        retrievePreferences();
        pack();
    }

    public Layout createLayout(Graph g, GraphComponent c) {
        double gamma = ((Number) spGamma.getValue()).doubleValue();
        double startT = ((Number) spStartT.getValue()).doubleValue();
        double endT = ((Number) spEndT.getValue()).doubleValue();
        double nodeRepulsion = ((Number) spNodeRepulsion.getValue()).doubleValue();
        double borderRepulsion = ((Number) spBorderRepulsion.getValue()).doubleValue();
        double edgeStretch = ((Number) spEdgeStretch.getValue()).doubleValue();
        double edgeCrossing = ((Number) spEdgeCrossing.getValue()).doubleValue();
        double nodeEdgeRepulsion = ((Number) spNodeEdgeRepulsion.getValue()).doubleValue();
        double edgeCrossCutoffT = ((Number) spFinetuneCutoff.getValue()).doubleValue();
        double partial = ((Number) spPartial.getValue()).doubleValue();
        boolean mergeBidir = cbMergeBidir.isSelected();

        AnnealLayout layout = new AnnealLayout(g, c);

        layout.setGamma(gamma);
        layout.setStartT(startT);
        layout.setEndT(endT);
        layout.setNodeRepulsion(nodeRepulsion);
        layout.setBorderRepulsion(borderRepulsion);
        layout.setEdgeStretch(edgeStretch);
        layout.setEdgeCrossing(edgeCrossing);
        layout.setNodeEdgeRepulsion(nodeEdgeRepulsion);
        layout.setEdgeCrossCutoffT(edgeCrossCutoffT);

        return new PartialLayout(g, c, layout, partial, cbPartial.getSelectedIndex(), mergeBidir);
    }

    void retrievePreferences() {
        int version;
        try {
            Preferences prefs = Preferences.userNodeForPackage(LayoutDialog.class);
            version = prefs.getInt(PREFS_VERSION, 0);
            spGamma.setValue(new Double(prefs.getDouble(PREFS_GAMMA, PREFS_DEFAULT_GAMMA)));
            spStartT.setValue(new Double(prefs.getDouble(PREFS_START_T, PREFS_DEFAULT_START_T)));
            spEndT.setValue(new Double(prefs.getDouble(PREFS_END_T, PREFS_DEFAULT_END_T)));
            spNodeRepulsion.setValue(new Double(prefs.getDouble(PREFS_NODE_REPULSION, PREFS_DEFAULT_NODE_REPULSION)));
            spBorderRepulsion.setValue(new Double(prefs.getDouble(PREFS_BORDER_REPULSION, PREFS_DEFAULT_BORDER_REPULSION)));
            spEdgeStretch.setValue(new Double(prefs.getDouble(PREFS_EDGE_STRETCH, PREFS_DEFAULT_EDGE_STRETCH)));
            spEdgeCrossing.setValue(new Double(prefs.getDouble(PREFS_EDGE_CROSSING, PREFS_DEFAULT_EDGE_CROSSING)));
            spNodeEdgeRepulsion.setValue(new Double(prefs.getDouble(PREFS_NODE_EDGE_REPULSION, PREFS_DEFAULT_NODE_EDGE_REPULSION)));
            spFinetuneCutoff.setValue(new Double(prefs.getDouble(PREFS_EDGE_CROSS_CUTOFF_T, PREFS_DEFAULT_EDGE_CROSS_CUTOFF_T)));
            spPartial.setValue(new Double(prefs.getDouble(PREFS_PARTIAL, PREFS_DEFAULT_PARTIAL)));
            cbMergeBidir.setSelected(prefs.getBoolean(PREFS_MERGE_BIDIR, PREFS_DEFAULT_MERGE_BIDIR));
            if(version<2) {
                boolean usePartial = prefs.getBoolean(PREFS_USE_PARTIAL, false);
                cbPartial.setSelectedIndex(usePartial?PartialLayout.GET_LOW_DEGREE_NODES:PartialLayout.GET_ALL_NODES);
            } else {
                cbPartial.setSelectedIndex(prefs.getInt(PREFS_USE_PARTIAL, PREFS_DEFAULT_USE_PARTIAL));
            }
        } catch (AccessControlException e) {
            spGamma.setValue(new Double(PREFS_DEFAULT_GAMMA));
            spStartT.setValue(new Double(PREFS_DEFAULT_START_T));
            spEndT.setValue(new Double(PREFS_DEFAULT_END_T));
            spNodeRepulsion.setValue(new Double(PREFS_DEFAULT_NODE_REPULSION));
            spBorderRepulsion.setValue(new Double(PREFS_DEFAULT_BORDER_REPULSION));
            spEdgeStretch.setValue(new Double(PREFS_DEFAULT_EDGE_STRETCH));
            spEdgeCrossing.setValue(new Double(PREFS_DEFAULT_EDGE_CROSSING));
            spNodeEdgeRepulsion.setValue(new Double(PREFS_DEFAULT_NODE_EDGE_REPULSION));
            spFinetuneCutoff.setValue(new Double(PREFS_DEFAULT_EDGE_CROSS_CUTOFF_T));
            cbPartial.setSelectedIndex(PREFS_DEFAULT_USE_PARTIAL);
            spPartial.setValue(new Double(PREFS_DEFAULT_PARTIAL));
            cbMergeBidir.setSelected(PREFS_DEFAULT_MERGE_BIDIR);
        }
    }

    void updatePreferences() {
        try {
            Preferences prefs = Preferences.userNodeForPackage(LayoutDialog.class);

            prefs.putInt(PREFS_VERSION, PREFS_CURRENT_VERSION);
            prefs.putDouble(PREFS_GAMMA, ((Number) spGamma.getValue()).doubleValue());
            prefs.putDouble(PREFS_START_T, ((Number) spStartT.getValue()).doubleValue());
            prefs.putDouble(PREFS_END_T, ((Number) spEndT.getValue()).doubleValue());
            prefs.putDouble(PREFS_NODE_REPULSION, ((Number) spNodeRepulsion.getValue()).doubleValue());
            prefs.putDouble(PREFS_BORDER_REPULSION, ((Number) spBorderRepulsion.getValue()).doubleValue());
            prefs.putDouble(PREFS_EDGE_STRETCH, ((Number) spEdgeStretch.getValue()).doubleValue());
            prefs.putDouble(PREFS_EDGE_CROSSING, ((Number) spEdgeCrossing.getValue()).doubleValue());
            prefs.putDouble(PREFS_NODE_EDGE_REPULSION, ((Number) spNodeEdgeRepulsion.getValue()).doubleValue());
            prefs.putDouble(PREFS_EDGE_CROSS_CUTOFF_T, ((Number) spFinetuneCutoff.getValue()).doubleValue());
            prefs.putInt(PREFS_USE_PARTIAL, cbPartial.getSelectedIndex());
            prefs.putDouble(PREFS_PARTIAL, ((Number) spPartial.getValue()).doubleValue());
            prefs.putBoolean(PREFS_MERGE_BIDIR, cbMergeBidir.isSelected());
        } catch (AccessControlException e) {
        }
    }
}

