// This progarm is intended to query and display  from remote MySql database server
// that holds the TTM database file.

package nl.tudelft.tbm.traces;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.text.*;
import java.util.*;
import javax.print.*;
import javax.print.attribute.*;
import javax.swing.*;
import javax.swing.text.*;


public class Log extends JDialog implements ActionListener {
    private JTextArea taLog;
    private JButton bDismiss;
    private JButton bPrint;
    private JScrollBar sbVBar;

    public static int MAX_LINES = 10000;

    static Log log = null;

    public Log() {
        super();

        setTitle("Traces Log");

        Container c = getContentPane();

        c.setLayout(new GridBagLayout());

        // Create a constraints object, and specify some default values
        GridBagConstraints g = new GridBagConstraints();

        g.fill = GridBagConstraints.BOTH;   // components grow in both dimensions
        g.insets = new Insets(5, 5, 5, 5); // 5-pixel margins on all sides

        taLog = new JTextArea(25, 50);
		taLog.setFont(new Font("Monospaced", Font.PLAIN, 12));
        bDismiss = new JButton("Hide Window");
        bDismiss.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bDismiss) hide();
                }
            }
        );

        bPrint = new JButton("Print (EXPERIMENTAL!)");
        bPrint.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bPrint) print();
                }
            }
        );

        int line = -1;

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 2;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 50.0;
        JScrollPane pane = new JScrollPane(taLog);

        sbVBar = pane.getVerticalScrollBar();
        c.add(new JScrollPane(taLog), g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(bDismiss, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(bPrint, g);

        pack();
    }

	public static void setColumns(int columns) {
		log.taLog.setColumns(columns);
		log.pack();
	}

	public static void setRows(int rows) {
		log.taLog.setRows(rows);
		log.pack();
	}

    public void actionPerformed(ActionEvent e) {
        //...Get information from the action event...
        String command = e.getActionCommand();

        // Process command
    }

    public void truncate() {
        int lineCount = taLog.getLineCount();
        if (lineCount > MAX_LINES) {
            try {
                int pos = taLog.getLineStartOffset(lineCount - MAX_LINES);
                taLog.replaceRange(null, 0, pos);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public void scrollToEnd() {
        try {
            taLog.setCaretPosition(taLog.getLineStartOffset(taLog.getLineCount() - 1));
        } catch (BadLocationException e) {
            e.printStackTrace();
        }
    }

    public static synchronized void log(String s) {
        if (log == null) log = new Log();
            //	  boolean isAtMaximum = log.sbVBar.getValue()==log.sbVBar.getMaximum();
        log.taLog.append(s + "\n");

        log.truncate();
        log.scrollToEnd();
    }

    public static void logException(Throwable t) {
        StringWriter s = new StringWriter();
        t.printStackTrace(new PrintWriter(s));
        log(s.toString());
    }

    public static void log(Throwable t) {
        logException(t);
    }

    public static synchronized void logChar(String s) {
        if (log == null) log = new Log();
        log.taLog.append(s);
    }

    public static synchronized void showLog() {
        if (log == null) log = new Log();
        log.show();
    }

    public static synchronized void hideLog() {
        if (log != null) {
            log.hide();
        }
    }

    void print() {
        DocFlavor flavor = DocFlavor.INPUT_STREAM.POSTSCRIPT;

        /*
         PrintService[] services = PrintServiceLookup.lookupPrintServices(flavor, null);
         javax.print.attribute.PrintRequestAttributeSet attributes = new HashPrintRequestAttributeSet();
         //	  GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
         //	  GrahpicsDevice gd = ge.getScreenDevices()[0];
         //	  GraphicsConfiguration gc = gd.getDefaultConfiguration();
         PrintService service =	ServiceUI.printDialog(null, 50, 50, services, services[0], flavor, attributes);
         */
        PrintService service = PrintServiceLookup.lookupDefaultPrintService();

        if (service != null) {
            log(service.getName());
            DocFlavor[] df = service.getSupportedDocFlavors();

            for (int i = 0; i < df.length; i++) {
                log(df[i].toString());
            }
            PipedOutputStream os = new PipedOutputStream();
            PrintWriter out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(os)));
            InputStream is = null;

            try {
                is = new PipedInputStream(os);
                BufferedReader in = new BufferedReader(new StringReader(taLog.getText()));
                // InputStreamDoc is an implementation of the Doc interface //
                final DocPrintJob pj = service.createPrintJob();
                final Doc doc = new SimpleDoc(is, flavor, null);
                Thread t = new Thread() {
                        public void run() {
                            try {
                                pj.print(doc, null);
                            } catch (PrintException e) {
                                log("Error printing: " + e.getMessage());
                            }
                        }
                    };

                t.start();
                String s = "";

                while (s != null) {
                    s = in.readLine();
                    out.println(s);
                }
            } catch (IOException e) {
                log(e.getMessage());
            }
            out.close();
        }
    }

    public static void redirectOut() {
        if (log == null) log = new Log();
        final Log myLog = log;
        OutputStream os = new OutputStream() {
                public void write(int b) throws IOException {
                    myLog.logChar(Character.toString((char) b));
                }
            };
        
        PrintStream ps = new PrintStream(os, true) {; // auto-flush
                public void print(String x) {
                    myLog.logChar(x);
                }

                public void println(String x) {
                    myLog.log(x);
                }
            };

        System.setOut(ps);
    }
}

