// This progarm is intended to query and display  from remote MySql database server
// that holds the TTM database file.

package nl.tudelft.tbm.traces;


import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.URL.*;
import java.sql.*;
import java.text.*;
import java.util.*;
import java.util.prefs.*;
import javax.swing.*;

import nl.tudelft.tbm.graph.*;


public class MainApp extends JFrame implements ActionListener {
    final static String PREFS_OUTPUT_PATH = "output_path";
    static boolean isApplet = false;

    OptionsDialog options;

    JMenuBar menuBar;

    ButtonGroup bgQueryType;
    JRadioButton rbQFull;
    JRadioButton rbQPartial;
    JRadioButton rbQSingle;

    JSpinner spStartDate;
    JSpinner spEndDate;
    JSpinner spIntervalNumber;
    JSpinner spIntervalUnit;
    JSpinner spMaxDates;

    JSpinner spTestboxFrom;
    JSpinner spTestboxTo;

    JCheckBox cbDoAS;
    JCheckBox cbCalculateIP;
    JCheckBox cbCalculateAS;
    JCheckBox cbShowGraph;

    JCheckBox cbOutputXML;
    JCheckBox cbOutputOtter;
    JCheckBox cbOutputDaVinci;

    JCheckBox cbOutputTraceData;
    JCheckBox cbDoNoDull;

    JButton bOptions;
    JButton bDoIt;
    JButton bQuit;

    JButton bShowLog;
    JButton bShowSuspiciousAS;
    JButton bRemoveAS0;

    boolean busy = false;

    static String[] intervalStrings = {"minutes", "hours", "days", "weeks", "months", "years"};

    MainApp self = this;

    public static void main(String args[]) {
        final MainApp app = new MainApp();

        app.addWindowListener(
            new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    app.quit();
                }
            }
        );
    }

    public MainApp() {
        JMenu menu, submenu;
        JMenuItem menuItem;

        menuBar = new JMenuBar();
        setJMenuBar(menuBar);

        setTitle("Traces v" + Version.VERSION);

        menu = new JMenu("File");
        menu.setMnemonic(KeyEvent.VK_F);
        menu.getAccessibleContext().setAccessibleDescription(
            "The only menu in this program that has menu items");
        menuBar.add(menu);

        menuItem = new JMenuItem("Open");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Options");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Exit");
        menuItem.addActionListener(this);
        menu.add(menuItem);

        options = new OptionsDialog(this);

        Container c = getContentPane();

        c.setLayout(new GridBagLayout());

        // Create a constraints object, and specify some default values
        GridBagConstraints g = new GridBagConstraints();

        g.fill = GridBagConstraints.BOTH;   // components grow in both dimensions
        g.insets = new Insets(5, 5, 5, 5); // 5-pixel margins on all sides

        bgQueryType = new ButtonGroup();
        rbQFull = new JRadioButton("All test boxes", false);
        rbQPartial = new JRadioButton("Subset of boxes", false);
        rbQSingle = new JRadioButton("Single trace", true);
        bgQueryType.add(rbQFull);
        bgQueryType.add(rbQPartial);
        bgQueryType.add(rbQSingle);

        spStartDate = new JSpinner(new SpinnerDateModel(new java.util.Date(99, 8, 1, 20, 0),
                        new java.util.Date(99, 0, 1, 0, 0), new java.util.Date(), Calendar.DAY_OF_MONTH));
        spEndDate = new JSpinner(new SpinnerDateModel(new java.util.Date(99, 8, 1, 20, 0),
                        new java.util.Date(99, 0, 1, 0, 0), new java.util.Date(), Calendar.DAY_OF_MONTH));

        spIntervalNumber = new JSpinner(new SpinnerNumberModel(1, 1, 1000, 1));
        spIntervalUnit = new JSpinner(new SpinnerListModel(intervalStrings));
        spMaxDates = new JSpinner(new SpinnerNumberModel(100, 1, Integer.MAX_VALUE, 1));

        spTestboxFrom = new JSpinner(new SpinnerNumberModel(1, 1, 99, 1));
        spTestboxTo = new JSpinner(new SpinnerNumberModel(1, 1, 99, 1));

        cbDoAS = new JCheckBox("Use AS data", true);
        cbCalculateIP = new JCheckBox("Perform delay calculations (IP) EXPERIMENTAL!", false);
        cbCalculateAS = new JCheckBox("Perform delay calculations (AS) EXPERIMENTAL!", false);
        cbShowGraph = new JCheckBox("Show graph EXPERIMENTAL!", false);

        cbOutputXML = new JCheckBox("Output XML format", true);
        cbOutputOtter = new JCheckBox("Output Otter (odf) format", false);
        cbOutputDaVinci = new JCheckBox("Output daVinci format", false);

        cbOutputTraceData = new JCheckBox("Include trace data in XML file", false);
        cbDoNoDull = new JCheckBox("Also produce -nodull files");

        bShowSuspiciousAS = new JButton("Show suspicious Autonomous Systems");
        bShowSuspiciousAS.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bShowSuspiciousAS)
                        showSuspiciousAS();
                }
            }
        );

        bOptions = new JButton("Options");
        bOptions.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bOptions) {
                        options.setLocationRelativeTo(self);
                        options.setVisible(true);
                        Log.MAX_LINES = options.maxLogLines;
                    }
                }
            }
        );

        bDoIt = new JButton("Do it!");
        bDoIt.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bDoIt) {
                        doIt();
                    }
                }
            }
        );

        bShowLog = new JButton("Show Log");
        bShowLog.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bShowLog)
                        Log.showLog();
                }
            }
        );

        bQuit = new JButton("Quit");
        bQuit.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bQuit) {
                        Log.log("Exit");
                        quit();
                    }
                }
            }
        );

        bRemoveAS0 = new JButton("Clean up AS hash");
        bRemoveAS0.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bRemoveAS0) {
                        retryAS0();
                    }
                }
            }
        );

        int line = -1;

        JLabel lDate2 = new JLabel("Start date: ");

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(lDate2, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spStartDate, g);

        JLabel lDate3 = new JLabel("End date: ");

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(lDate3, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spEndDate, g);

        JLabel lInterval = new JLabel("Interval: ");

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(lInterval, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spIntervalNumber, g);

        g.gridx = 2;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spIntervalUnit, g);

        JLabel lmaxdates = new JLabel("Max number of dates: ");

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(lmaxdates, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spMaxDates, g);

        JLabel lQType = new JLabel("Query type: ");

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(lQType, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(rbQFull, g);

        g.gridx = 1;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(rbQPartial, g);

        g.gridx = 1;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(rbQSingle, g);

        JLabel ltbFrom = new JLabel("From Testbox #: ");

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(ltbFrom, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spTestboxFrom, g);

        JLabel ltbTo = new JLabel("To Testbox #: ");

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(ltbTo, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(spTestboxTo, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 2;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(cbDoAS, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 2;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(cbCalculateIP, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 2;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(cbCalculateAS, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 2;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(cbShowGraph, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 2;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(cbOutputXML, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 2;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(cbOutputOtter, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 2;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(cbOutputDaVinci, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 2;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(cbOutputTraceData, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 2;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        c.add(cbDoNoDull, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 3.0;
        g.weighty = 0.0;
        c.add(bOptions, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 3.0;
        g.weighty = 0.0;
        c.add(bDoIt, g);

        g.gridx = 2;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 3.0;
        g.weighty = 0.0;
        c.add(bQuit, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 3.0;
        g.weighty = 0.0;
        c.add(bShowLog, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 3.0;
        g.weighty = 0.0;
        c.add(bShowSuspiciousAS, g);

        g.gridx = 2;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 3.0;
        g.weighty = 0.0;
        c.add(bRemoveAS0, g);

        pack();
        spIntervalUnit.setValue("days");
        Log.showLog();
        show();
    }

    public static DelayGraph tracesToDelayGraph(Vector traces) throws IOException, Exception {
        DelayGraph g = new DelayGraph();

        g.addNodes(TestBox.getTestBoxSet());
        for (Enumeration e1 = traces.elements(); e1.hasMoreElements();) {
            Trace trace = (Trace) e1.nextElement();

            g.addEndToEnd(trace);
        }

        return g;
    }

    public void analyse() {
        Vector traces = new Vector();

        TestBox.clearCache();
        Preferences prefs = null;

        if (!isApplet) {
            prefs = Preferences.userNodeForPackage(MainApp.class);
        }
        try {
            String password;
            if (options.DBStorePassword) {
                password = options.DBPassword;
            } else {
                password = JOptionPane.showInputDialog(null, "Password: ", 
                    "Database password", JOptionPane.QUESTION_MESSAGE);
            }
            String URL = "jdbc:" + options.DBType + "://" + options.DBHost + 
                "/" + options.DBDatabase +
                "?user=" + options.DBUser + "&password=" + password;

            TTMData ttm = new TTMData(URL);

            if (options.useBackup) {
                ttm.setTables("Backup_Records", "Backup_Routes", "Delays");
            } else {
                ttm.setTables("Records", "Routes", "Delays");
            }
            java.util.Date startDate = (java.util.Date) spStartDate.getValue();
            java.util.Date endDate = (java.util.Date) spEndDate.getValue();
            String intervalString = (String) spIntervalUnit.getValue();
            int intervalNumber = ((Number) spIntervalNumber.getValue()).intValue();
            int maxDates = ((Number) spMaxDates.getValue()).intValue();
            boolean doNoDull = cbDoNoDull.isSelected();

            Log.log("Start, end, interval: " + startDate + ", " + endDate + ", " + intervalNumber + " " + intervalString);

            Set dates = new HashSet();
            Calendar cal = new GregorianCalendar();

            cal.setTime(startDate);
            Calendar finish = new GregorianCalendar();

            finish.setTime(endDate);
            int i = 0;

            while (!cal.after(finish) && i < maxDates) {
                dates.add(cal.getTime());
                if (intervalString.equals("minutes")) {
                    cal.add(Calendar.MINUTE, intervalNumber);
                } else if (intervalString.equals("hours")) {
                    cal.add(Calendar.HOUR_OF_DAY, intervalNumber);
                } else if (intervalString.equals("days")) {
                    cal.add(Calendar.DATE, intervalNumber);
                } else if (intervalString.equals("weeks")) {
                    cal.add(Calendar.DATE, 7 * intervalNumber);
                } else if (intervalString.equals("months")) {
                    cal.add(Calendar.MONTH, intervalNumber);
                } else if (intervalString.equals("years")) {
                    cal.add(Calendar.YEAR, intervalNumber);
                } else
                    Log.log("OOPS: unknown interval unit: " + intervalString);
                i++;
            }

            if (i >= maxDates)
                Log.log("Cutting off date intervals after " + maxDates + " points.");

            int from = ((Number) spTestboxFrom.getValue()).intValue();
            int to = ((Number) spTestboxTo.getValue()).intValue();

            if (rbQFull.isSelected()) {
                traces = ttm.getTracesSnapshot(dates);
            } else if (rbQPartial.isSelected()) {
                traces = ttm.getPartialTracesSnapshot(from, to, dates);
            } else if (rbQSingle.isSelected()) {
                traces = ttm.getTrace(from, to, dates);
            }
            ttm.close();
            Log.log("Finished reading, now converting to graph");

            // Convert traces to Graph
            DelayGraph graph = tracesToDelayGraph(traces);

            graph.removeUnconnectedNodes();
            Log.log("Number of nodes in IP graph: " + graph.nodes.size() + "; number of edges: " + graph.edges.size());

            if (cbCalculateIP.isSelected()) {
                graph.buildEquations();
            }

            DelayGraph graph2 = null;

            if (cbDoAS.isSelected()) {
                // Make AS graph; also needed to fill in the AS fields in the Hosts.
                Log.log("Reducing graph using autonomous systems");
                ASHash hash;

                if (new File(options.ASCacheLocation).canRead()) {
                    try {
                        hash = ASHash.readASHash(options.ASCacheLocation);
                        //		  Log.log("Read hash: " + hash.toString());
                    } catch (Exception e) {
                        Log.log("Failed reading AS data, starting with empty cache");
                        //		  e.printStackTrace();
                        hash = new ASHash(true);
                    }
                } else {
                    Log.log("No AS data, starting with empty cache");
                    hash = new ASHash(true);
                }
                graph2 = graph.reduceUsingAutonomousSystems(hash);
                try {
                    hash.consolidate();
                    ASHash.writeASHash(hash, options.ASCacheLocation);
                } catch (Exception e) {
                    Log.log("Failed writing AS data");
                    e.printStackTrace();
                }
                Log.log("Number of nodes in AS graph: " + graph2.nodes.size() + "; number of edges: " + graph2.edges.size());

                if (cbShowGraph.isSelected()) {
                    Frame f = new Frame("Testing GraphComponent");
                    GraphComponent c = new GraphComponent(graph2);

                    f.add(c);
                    f.setSize(600, 500);
                    //	      f.addActionListener(c);
                    f.show();
                }

                if (cbCalculateAS.isSelected()) {
                    graph2.buildEquations();
                }
            }

            //	      Log.log("Graph data: ");
            //	      Log.log(graph.toString());

            boolean doXML = cbOutputXML.isSelected();
            boolean doOtter = cbOutputOtter.isSelected();
            boolean doDaVinci = cbOutputDaVinci.isSelected();

            if (doXML || doOtter || doDaVinci) {
                Log.log("Querying user for filename");
                JFileChooser chooser = new JFileChooser();

                if (!isApplet) {
                    chooser.setCurrentDirectory(new File(prefs.get(PREFS_OUTPUT_PATH, System.getProperty("user.dir"))));
                }
                chooser.setDialogTitle("Choose output base name");
                int returnVal = chooser.showOpenDialog(self);

                if (returnVal == JFileChooser.APPROVE_OPTION) {
                    if (!isApplet)
                        prefs.put(PREFS_OUTPUT_PATH, chooser.getCurrentDirectory().getPath());
                        //		      String dateString = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT).format(date).replace(':','.');
                    String basename = chooser.getSelectedFile().getAbsolutePath(); // + "-" + dateString;

                    Log.log("Base name: " + basename);
                    graph.includeTracesInXML = cbOutputTraceData.isSelected();
                    graph2.includeTracesInXML = cbOutputTraceData.isSelected();

                    //		      graph.addEdge(new Host("42.42.42.42", "DUMMY"), TestBox.getTestBox(1));

                    Log.log("Exporting IP graph");
                    if (doXML) graph.exportXML(basename + "-IP.xml");
                    if (doOtter) graph.exportOtter(basename + "-IP.odf");
                    if (doDaVinci) graph.exportdaVinci(basename + "-IP.daVinci");

                    if (doNoDull) {
                        Log.log("Removing dull nodes from IP graph");
                        graph.removeDullNodes();
                        Log.log("Number of nodes in IP graph without dull nodes: " + graph.nodes.size() + "; number of edges: " + graph.edges.size());
                        Log.log("Exporting IP-nodull graph");
                        if (doXML) graph.exportXML(basename + "-IP-nodull.xml");
                        if (doOtter) graph.exportOtter(basename + "-IP-nodull.odf");
                        if (doDaVinci) graph.exportdaVinci(basename + "-IP-nodull.daVinci");
                    }

                    if (cbDoAS.isSelected()) {
                        AutSystem.discrete = true;
                        //		  if (graph2.nodes.size()>0) {
                        //		      graph2.addEdge(new AutSystem("DUMMY", 42), graph2.nodes.iterator().next());
                        //		  }
                        Log.log("Exporting ASnumber graph");
                        if (doXML) graph2.exportXML(basename + "-ASnumber.xml");
                        if (doOtter) graph2.exportOtter(basename + "-ASnumber.odf");
                        if (doDaVinci) graph2.exportdaVinci(basename + "-ASnumber.daVinci");
                        AutSystem.discrete = false;
                        Log.log("Exporting ASname graph");
                        if (doXML) graph2.exportXML(basename + "-ASname.xml");
                        if (doOtter) graph2.exportOtter(basename + "-ASname.odf");
                        if (doDaVinci) graph2.exportdaVinci(basename + "-ASname.daVinci");
                        if (doNoDull) {
                            Log.log("Removing dull nodes from AS graph");
                            graph2.removeDullNodes();
                            Log.log("Number of nodes in AS graph without dull nodes: " + graph2.nodes.size() + "; number of edges: " + graph2.edges.size());
                            AutSystem.discrete = true;
                            Log.log("Exporting ASnumber-nodull graph");
                            if (doXML) graph2.exportXML(basename + "-ASnumber-nodull.xml");
                            if (doOtter) graph2.exportOtter(basename + "-ASnumber-nodull.odf");
                            if (doDaVinci) graph2.exportdaVinci(basename + "-ASnumber-nodull.daVinci");
                            AutSystem.discrete = false;
                            Log.log("Exporting ASname-nodull graph");
                            if (doXML) graph2.exportXML(basename + "-ASname-nodull.xml");
                            if (doOtter) graph2.exportOtter(basename + "-ASname-nodull.odf");
                            if (doDaVinci) graph2.exportdaVinci(basename + "-ASname-nodull.daVinci");
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void doIt() {
        if (busy) {
            Log.log("Already doing something!");
        } else {
            busy = true;
            Thread t = new Thread() {
                    public void run() {
                        Log.log("Starting...");
                        analyse();
                        Log.log("Finished.");
                        busy = false;
                    }
                };

            t.start();
        }
    }

    public void showSuspiciousAS() {
        if (busy) {
            Log.log("Already doing something!");
        } else {
            busy = true;
            Thread t = new Thread() {
                    public void run() {
                        ASHash hash;

                        try {
                            hash = ASHash.readASHash(options.ASCacheLocation);
                        } catch (Exception e) {
                            Log.log("Error reading AS hash from " + options.ASCacheLocation + ", creating new");
                            hash = new ASHash(true);
                        }
                        hash.printSuspiciousEntries();
                        busy = false;
                    }
                };

            t.start();
        }
    }

    public void retryAS0() {
        if (busy) {
            Log.log("Already doing something!");
        } else {
            busy = true;
            Thread t = new Thread() {
                public void run() {
                    ASHash hash;

                    try {
                        hash = ASHash.readASHash(options.ASCacheLocation);
                    } catch (Exception e) {
                        Log.log("Error reading AS hash from " + options.ASCacheLocation + ", creating new");
                        hash = new ASHash(true);
                    }
                    hash.retryAS0();
                    try {
                        hash.writeASHash(hash, options.ASCacheLocation);
                    } catch (Exception e) {
                        Log.log("Error writing AS hash to " + options.ASCacheLocation);
                    }
                    Log.log("Finished cleanup");
                    busy = false;
                }
            };

            t.start();
        }
    }

    public void actionPerformed(ActionEvent e) {
        //...Get information from the action event...
        String command = e.getActionCommand();

        // Process command
        if (command.equals("Open")) {
            Log.log("Open doesn't do anything");
        } else if (command.equals("Options")) {
            options.setLocationRelativeTo(this);
            options.setVisible(true);
            Log.MAX_LINES = options.maxLogLines;
            Log.log("Done options");
        } else if (command.equals("Exit")) {
            Log.log("Exit");
            quit();
        }
    }

    public void quit() {
        if (busy) {
            Log.log("Busy! There should be a message box here, but exiting anyway....");
            try {
                Thread.sleep(500);
                Thread.yield();
                Thread.sleep(500);
                Thread.yield();
                Thread.sleep(500);
                Thread.yield();
                Thread.sleep(500);
                Thread.yield();
                Thread.sleep(500);
                Thread.yield();
                Thread.sleep(500);
                Thread.yield();
            } catch (InterruptedException e) {
            }
            System.exit(0);
        } else {
            Log.log("Exiting....");
            try {
                Thread.sleep(000);
            } catch (InterruptedException e) {
            }
            System.exit(0);
        }
    }

}

