/* NNLS
 * Non-Negative Least Squares
 * (C) 2001 Samson Habte Tesfazgi
 * All rights reserved
 */

package nl.tudelft.tbm.traces;


import java.util.*;
import java.util.logging.*;
import Jama.*;


public class NNLS {
    static Logger logger = Logger.getLogger("nl.tudelft.tbm.traces");
    public static Matrix solve(Matrix A, Matrix b) {
        // get the dimensions of Matrix A
        int m, n, t, q, index1, index2, colindex, iter = 0;
        double min, result1, result2, alpha;
        boolean isremoved;
        Integer in1, in2;

        //System.out.print("A is: ");
        //A.print(1, 5);
        //System.out.print("b is: ");
        //b.print(1, 5);
        m = A.getRowDimension();
        n = A.getColumnDimension();
        LinkedList dumpP = new LinkedList();
		
        // Declare two variables for the two sets: P and Z that hold indexes
        // and initialize set Z to hold all available indexes.
        HashSet P = new HashSet();
        HashSet Z = new HashSet();

        for (int i = 0; i < n; i++) {
            Z.add(new Integer(i));
        }

        // declare and initialize the unknown matrix X
        Matrix x = new Matrix(n, 1);
				
        // declare and initialize two n-vector variables: z and w to serve as working space
        // and intermidiate matrixes.
        Matrix w, zz, tranA, Ax;
						
        do {
            logger.info("Iteration: "+iter);
            //compute w
            tranA = A.transpose();
            Ax = A.times(x);
            w = tranA.times(b.minus(Ax));
            if ((Z.isEmpty()) || (isElementsNegative(w, Z))) break;
                //if((Z.isEmpty())) break;
                //get the index of the maximum element of w matrix
            t = getMaxIndex(w, Z);
            isremoved = Z.remove(new Integer(t));
            P.add(new Integer(t));
            //System.out.println("P:" + P);
            //System.out.println("Z:" + Z);	
            //construct a matrix from A to solve part of the solution
            do {
                Matrix Ap = new Matrix(m, P.size());
                LinkedList orderedP = new LinkedList();

                colindex = 0;
                for (int j = 0; j < n; j++) {
                    if (P.contains(new Integer(j))) {
                        orderedP.add(new Integer(j));
                        for (int i = 0; i < m; i++) {
                            Ap.set(i, colindex, A.get(i, j));
                        }
                        colindex++;
                    }
                }
			
                // compute the solution of the least square problem: Ap*zz= b 
                // zz is computed using QRDecompostion method
                // write exception class or use assert to give error
                // message when the isFullRank method returns false.the method is not reliable use rank and check your self
                // rank is deficient if Rank(A) < min(m,n) and full rank if Rank(A) = min (m,n)
                QRDecomposition qrAp = new QRDecomposition(Ap);
                zz = qrAp.solve(b);

                Matrix z = new Matrix(n, 1);
                for (int k = 0; k < zz.getRowDimension(); k++) {
                    z.set(((Integer) orderedP.get(k)).intValue(), 0, zz.get(k, 0));
                }
                //System.out.print("z: ");
                //z.print(1, 2);			
                if (!(isMatrixPositive(zz))) {
                    // find an index to compute alpha
                    min = 0;
                    //System.out.println("alpha:");
                    Iterator itP1 = P.iterator();

                    while (itP1.hasNext()) {
                        in1 = (Integer) itP1.next();
                        index1 = in1.intValue();
                        if (z.get(index1, 0) <= 0) {
                            result1 = (x.get(index1, 0)) / (x.get(index1, 0) - z.get(index1, 0));
                            if (min > result1) {
                                min = result1;
                                q = index1;
                            }
                        }
                    }
                    alpha = min;
                    for (int ii = 0; ii < x.getRowDimension(); ii++) {
                        result2 = x.get(ii, 0) + 0.2 * (z.get(ii, 0) - x.get(ii, 0));
                        x.set(ii, 0, result2);
                    }
                    System.out.println("test" + alpha);
                    Iterator itP2 = P.iterator();

                    while (itP2.hasNext()) {
                        in2 = (Integer) itP2.next();
                        index2 = in2.intValue();
                        if (x.get(index2, 0) == 0)
                        {
//                            System.out.println("testA"); 
                            dumpP.add(new Integer(index2));
                            Z.add(new Integer(index2));
                            //System.out.println("testB");
                        }
                    }
                    //System.out.println("testC");
                    for (int kk = 0; kk < dumpP.size(); kk++) {
                        P.remove((Integer) dumpP.get(kk));
                    }
                } else {
                    x = z;
                }

            }
            while (!(isMatrixPositive(zz))); 
            iter++;

        }
        while (iter <= 3 * n);
        //Ap.print(1,2);
        //z.print(1,2);
        //w.print(1,2);
        //System.out.print("x: ");
        //x.print(1, 4);
//      //System.out.println("alpha:" + alpha);
        return x;
    }				 											

    public  static boolean isElementsNegative(Matrix a, HashSet h) 
    //post: returns true if all elements of a matrix a with one column dimension
    //      are not positive for all indexes in the h HashSet.
    {
        int i;
        Iterator it = h.iterator();

        while (it.hasNext()) {
            i = ((Integer) it.next()).intValue();
            if (a.get(i, 0) > 0) return false; 
        }
        return true;
    }	
	
    public  static int getMaxIndex(Matrix a, HashSet h) //post: returns the index of the maximum element
    {
        int index, i;
        Iterator it = h.iterator();
        double max;

        index = ((Integer) it.next()).intValue();
        max = a.get(index, 0);
        while (it.hasNext()) {
            i = ((Integer) it.next()).intValue();
            if (max < a.get(i, 0)) {
                max = a.get(i, 0);   
                index = i;
            }
        }
        return index;
    }

    public  static boolean isMatrixPositive(Matrix a) // post: returns true if all elements of a sigle dimension matrix are positive 
    {
        for (int i = 0; i < a.getRowDimension(); i++) {
            if ((a.get(i, 0)) < 0) return false;
        }
        return true;
    }

}
