// This program is intended to query and display  from remote MySql database server
// that holds the TTM database file.

package nl.tudelft.tbm.traces;


import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.net.URL.*;
// import java.sql.*;
import java.text.*;
import java.util.*;
import java.util.logging.*;
import java.util.prefs.*;
import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.HyperlinkEvent;
import javax.swing.filechooser.*;
import javax.swing.text.html.HTMLDocument;
import javax.swing.tree.*;

import java.awt.print.*;
import javax.print.*;
import javax.print.attribute.*;
import javax.print.attribute.standard.*;

import nl.tudelft.tbm.graph.*;


public class NewApp extends JFrame implements ActionListener {
    final static String PREFS_OUTPUT_PATH = "output_path";
    final static String BASE_TITLE = "Traces v" + Version.VERSION;
	final static String README_LOCATION = "nl/tudelft/tbm/traces/README.html";

    final static javax.swing.filechooser.FileFilter FILE_FILTER_XML = new ExampleFileFilter("xml", "GraphXML Files");
    final static javax.swing.filechooser.FileFilter FILE_FILTER_OTTER = new ExampleFileFilter("odf", "Otter Data Files");
    final static javax.swing.filechooser.FileFilter FILE_FILTER_DAVINCI = new ExampleFileFilter("daVinci", "daVinci Data Files");
    final static javax.swing.filechooser.FileFilter FILE_FILTER_POSTSCRIPT = new ExampleFileFilter("ps", "PostScript Files");
    final static javax.swing.filechooser.FileFilter FILE_FILTER_TEXT = new ExampleFileFilter("txt", "Text Files");

    static boolean isApplet = false;

    boolean busy = false;
    static String preloadFilename = null;

    NewApp self = this;
    OptionsDialog options;
    String currentFilename = null;
    Preferences prefs = null;
    DelayGraph graph = null;
    Box graphBox;
    JScrollPane scroll = null;
    GraphComponent view = null;
    Layout currentLayout = null;

	final static NumberFormat delayFormat = new DecimalFormat("#####.###");

	ArrayList filters = new ArrayList();

	ASHash hash = null;

    int showLabels = NodeComponent.SHOW_LABEL_NONE;
    boolean autoZoom = false;
    boolean showEdges = true;
    boolean showGraph = true;
	String edgeColorProperty = null;
	int edgeStyle = EdgeComponent.STYLE_CURVED;

    private FileHandler fileHandler = null;
    private Logger rootLogger = Logger.getLogger("");
    private static Logger logger = Logger.getLogger("nl.tudelft.tbm.traces");

    public static void main(String args[]) {
        if (args.length == 1) 
            preloadFilename = args[0];

        final NewApp app = new NewApp();

        app.addWindowListener(
            new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    app.quit();
                }
            }
        );
        app.show();
    }

    public NewApp() {
        System.setProperty("line.separator", "\n");
		prepareMenus();
        setTitle(BASE_TITLE);

        options = new OptionsDialog(this);

        initLogging();

        Log.MAX_LINES = options.maxLogLines;
        TestBox.dataFilename = options.testboxLocation;
        if (!isApplet) {
            prefs = Preferences.userNodeForPackage(NewApp.class);
        }
        try {
            logger.info("Trying to read whois server list from "+options.whoisServersFilename);
            Whois.readServersXML(options.whoisServersFilename);
        } catch (Exception e) {
            logger.log(Level.INFO, "Failed, using default whois servers", e);
            Whois.useDefaultServers();
        }

		prepareScreen();

        setSize(600, 500);
        //	  pack();
        //	  Log.showLog();

        if (preloadFilename != null) {
            openFilename(preloadFilename);
        }

    }

	void prepareMenus() {
	    JMenuBar menuBar;
        JMenu menu, submenu;
        JMenuItem menuItem;

        menuBar = new JMenuBar();
        setJMenuBar(menuBar);

        menu = new JMenu("File");
        menu.setMnemonic(KeyEvent.VK_F);
        menu.getAccessibleContext().setAccessibleDescription(
            "The only menu in this program that has menu items");
        menuBar.add(menu);

        menuItem = new JMenuItem("Open");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Save");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Save as");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Query Database");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        submenu = new JMenu("Export...");
        menu.add(submenu);
        menuItem = new JMenuItem("Export GraphML (EXPERIMENTAL)");
        menuItem.addActionListener(this);
        submenu.add(menuItem);
        menuItem = new JMenuItem("Export PostScript");
        menuItem.addActionListener(this);
        submenu.add(menuItem);
        menuItem = new JMenuItem("Export Otter");
        menuItem.addActionListener(this);
        submenu.add(menuItem);
        menuItem = new JMenuItem("Export daVinci");
        menuItem.addActionListener(this);
        submenu.add(menuItem);
        menuItem = new JMenuItem("Print");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Exit");
        menuItem.addActionListener(this);
        menu.add(menuItem);

        menu = new JMenu("View");
        menu.setMnemonic(KeyEvent.VK_V);
        menuBar.add(menu);

        menuItem = new JCheckBoxMenuItem("Show graph", true);
        menuItem.addActionListener(this);
        menu.add(menuItem);

        menuItem = new JCheckBoxMenuItem("Show Edges", showEdges);
        menuItem.addActionListener(this);
        menu.add(menuItem);

        submenu = new JMenu("Labels...");
        menu.add(submenu);
        ButtonGroup group = new ButtonGroup();

        menuItem = new JRadioButtonMenuItem("Show none", true);
        menuItem.addActionListener(this);
        group.add(menuItem);
        submenu.add(menuItem);

        menuItem = new JRadioButtonMenuItem("Show labels", false);
        menuItem.addActionListener(this);
        group.add(menuItem);
        submenu.add(menuItem);

        menuItem = new JRadioButtonMenuItem("Show names", false);
        menuItem.addActionListener(this);
        group.add(menuItem);
        submenu.add(menuItem);

        submenu = new JMenu("Zoom...");
        menu.add(submenu);

        menuItem = new JRadioButtonMenuItem("Auto Zoom", autoZoom);
        menuItem.addActionListener(this);
        submenu.add(menuItem);

        menuItem = new JMenuItem("Fit");
        menuItem.addActionListener(this);
        submenu.add(menuItem);

        menuItem = new JMenuItem("Zoom In");
        menuItem.addActionListener(this);
        submenu.add(menuItem);

        menuItem = new JMenuItem("Zoom Out");
        menuItem.addActionListener(this);
        submenu.add(menuItem);

        menuItem = new JMenuItem("Layout...");
        menuItem.addActionListener(this);
        menu.add(menuItem);

        submenu = new JMenu("Color Edges...");
        menu.add(submenu);
        group = new ButtonGroup();

        menuItem = new JRadioButtonMenuItem("don't color", (edgeColorProperty==null));
        menuItem.addActionListener(this);
        group.add(menuItem);
        submenu.add(menuItem);

        menuItem = new JRadioButtonMenuItem("color by delay", ("delay".equals(edgeColorProperty)));
        menuItem.addActionListener(this);
        group.add(menuItem);
        submenu.add(menuItem);

        submenu = new JMenu("Edge Style...");
        menu.add(submenu);
        group = new ButtonGroup();

        menuItem = new JRadioButtonMenuItem("straight edges", (edgeStyle==EdgeComponent.STYLE_STRAIGHT));
        menuItem.addActionListener(this);
        group.add(menuItem);
        submenu.add(menuItem);

        menuItem = new JRadioButtonMenuItem("curved edges", (edgeStyle==EdgeComponent.STYLE_CURVED));
        menuItem.addActionListener(this);
        group.add(menuItem);
        submenu.add(menuItem);

		menuItem = new JMenuItem("Show Log window");
        menuItem.addActionListener(this);
        menu.add(menuItem);

        menu = new JMenu("Tools");
        menu.setMnemonic(KeyEvent.VK_T);
        menuBar.add(menu);
        menuItem = new JMenuItem("Convert to AS graph");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Delay calculations");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Remove dull nodes");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Show suspicious ASs");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Cleanup cache");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Show database freshness");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Abort current layout");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Options");
        menuItem.addActionListener(this);
        menu.add(menuItem);

        menu = new JMenu("Experimental");
        menu.setMnemonic(KeyEvent.VK_E);
        menuBar.add(menu);
        menuItem = new JMenuItem("Segment Graph");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Convert to router graph");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Merge bidirectional edges");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Clear busy flag");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Dump IP addresses");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Show memory");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Test");
        menuItem.addActionListener(this);
        menu.add(menuItem);

        menu = new JMenu("Help");
        menu.setMnemonic(KeyEvent.VK_H);
        menuBar.add(menu);
	    menuItem = new JMenuItem("View README");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Check for new version");
        menuItem.addActionListener(this);
        menu.add(menuItem);
	    menuItem = new JMenuItem("About...");
        menuItem.addActionListener(this);
        menu.add(menuItem);
	}

	void prepareScreen() {
		Container c = getContentPane();

        graphBox = Box.createHorizontalBox();
		graphBox.setBackground(Color.WHITE);
		JPanel controls = new JPanel();

		Dimension valueDim = new Dimension(70,30); // new JLabel("99999,999").getSize();

        final EdgePropertyFilter delayFilterLeft = new EdgePropertyFilter("delay", PropertyFilter.FILTER_TYPE_DOUBLE, "0", PropertyFilter.ALLOW_HIGHER_EQUAL, true, 100);
		filters.add(delayFilterLeft);
		final JLabel delayValueLeft = new JLabel("0");
		delayValueLeft.setMinimumSize(valueDim);
		delayValueLeft.setMaximumSize(valueDim);
        final JSlider delaySliderLeft = new JSlider(JSlider.HORIZONTAL, 0, 100, 0);
		delaySliderLeft.setMaximumSize(new Dimension(100,75));
        delaySliderLeft.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                delayFilterLeft.setThreshold(""+delaySliderLeft.getValue());
				delayValueLeft.setText(delayFormat.format(delayFilterLeft.getThresholdValue()));
            }
        });

        final EdgePropertyFilter delayFilterRight = new EdgePropertyFilter("delay", PropertyFilter.FILTER_TYPE_DOUBLE, "100", PropertyFilter.ALLOW_LOWER_EQUAL, true, 100);
		filters.add(delayFilterRight);
		final JLabel delayValueRight = new JLabel("100.000");
		delayValueRight.setMinimumSize(valueDim);
		delayValueRight.setMaximumSize(valueDim);
        final JSlider delaySliderRight = new JSlider(JSlider.HORIZONTAL, 0, 100, 100);
		delaySliderRight.setMaximumSize(new Dimension(100,75));
        delaySliderRight.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                delayFilterRight.setThreshold(""+delaySliderRight.getValue());
				delayValueRight.setText(delayFormat.format(delayFilterRight.getThresholdValue()));
            }
        });

        final EdgeCountFilter degreeFilterLeft = new EdgeCountFilter(EdgeCountFilter.COUNT_BIDIRECTIONAL_ONCE, 0, EdgeCountFilter.ALLOW_HIGHER_EQUAL, true, 100);
		filters.add(degreeFilterLeft);
		final JLabel degreeValueLeft = new JLabel("0");
		degreeValueLeft.setMinimumSize(valueDim);
		degreeValueLeft.setMaximumSize(valueDim);
        final JSlider degreeSliderLeft = new JSlider(JSlider.HORIZONTAL, 0, 100, 0);
		degreeSliderLeft.setMaximumSize(new Dimension(100,75));
        degreeSliderLeft.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                degreeFilterLeft.setThreshold(degreeSliderLeft.getValue());
				degreeValueLeft.setText(delayFormat.format(degreeFilterLeft.getThresholdValue()));
            }
        });

        final EdgeCountFilter degreeFilterRight = new EdgeCountFilter(EdgeCountFilter.COUNT_BIDIRECTIONAL_ONCE, 100, EdgeCountFilter.ALLOW_LOWER_EQUAL, true, 100);
		filters.add(degreeFilterRight);
		final JLabel degreeValueRight = new JLabel("100");
		degreeValueRight.setMinimumSize(valueDim);
		degreeValueRight.setMaximumSize(valueDim);
        final JSlider degreeSliderRight = new JSlider(JSlider.HORIZONTAL, 0, 100, 100);
		degreeSliderRight.setMaximumSize(new Dimension(100,75));
        degreeSliderRight.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                degreeFilterRight.setThreshold(degreeSliderRight.getValue());
				degreeValueRight.setText(delayFormat.format(degreeFilterRight.getThresholdValue()));
            }
        });
	
        final DatesGraphFilter dateFilter = new DatesGraphFilter(true, 100);
		dateFilter.setEnabled(false);
		dateFilter.setResolution(DatesGraphFilter.RESOLUTION_DAY);
		filters.add(dateFilter);
		final JLabel dateValue = new JLabel("0");
        final JSlider dateSlider = new JSlider(JSlider.HORIZONTAL, 0, 100, 100);
		dateSlider.setMaximumSize(new Dimension(100,75));
        dateSlider.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                dateFilter.setThreshold(dateSlider.getValue());
				dateValue.setText(""+dateFilter.getThresholdValue());
            }
        });
		dateSlider.setEnabled(false);
		final JCheckBox dateCheckBox = new JCheckBox("Date Filter", false);
		dateCheckBox.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
				boolean active = dateCheckBox.isSelected();
                dateSlider.setEnabled(active);
                dateFilter.setEnabled(active);
            }
        });
		final JComboBox dateResolution = new JComboBox(DatesGraphFilter.RESOLUTION_STRINGS);
		dateResolution.setSelectedIndex(DatesGraphFilter.RESOLUTION_HOUR);
		dateResolution.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
				logger.info("Resolution changed to "+DatesGraphFilter.RESOLUTION_STRINGS[dateResolution.getSelectedIndex()]);
                dateFilter.setResolution(dateResolution.getSelectedIndex());
            }
        });
	
        c.setLayout(new GridBagLayout());
        GridBagConstraints g = new GridBagConstraints();
        g.fill = GridBagConstraints.BOTH;   // components grow in both dimensions
        g.insets = new Insets(5, 5, 5, 5); // 5-pixel margins on all sides

		g.gridx = 0; g.gridy = 0; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 50.0; g.weighty = 1.0;
		c.add(graphBox, g);
		g.gridx = 1; g.gridy = 0; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 1.0;
		c.add(controls, g);

		controls.setLayout(new GridBagLayout());
		int line = -1;
		g.gridx = 0; g.gridy = ++line; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 10.0;
		controls.add(new JPanel(), g);

		g.gridx = 0; g.gridy = ++line; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 0.0; g.weighty = 0.0;
		controls.add(new JLabel("Delay filter"), g);

		g.gridx = 0; g.gridy = ++line; 
		controls.add(delaySliderLeft, g);

		g.gridx = 1; g.gridy = line; 
		controls.add(delayValueLeft, g);

		g.gridx = 0; g.gridy = ++line; 
        controls.add(delaySliderRight, g);

		g.gridx = 1; g.gridy = line; 
		controls.add(delayValueRight, g);

		g.gridx = 0; g.gridy = ++line; 
		controls.add(new JLabel("Degree filter"), g);

		g.gridx = 0; g.gridy = ++line; 
        controls.add(degreeSliderLeft, g);
			
		g.gridx = 1; g.gridy = line; 
		controls.add(degreeValueLeft, g);

		g.gridx = 0; g.gridy = ++line; 
        controls.add(degreeSliderRight, g);

		g.gridx = 1; g.gridy = line; 
		controls.add(degreeValueRight, g);

		g.gridx = 0; g.gridy = ++line; 
		controls.add(dateCheckBox, g);

		g.gridx = 1; g.gridy = line; 
		controls.add(dateResolution, g);

		g.gridwidth = 2;
		g.gridx = 0; g.gridy = ++line; 
        controls.add(dateSlider, g);
			
		g.gridx = 0; g.gridy = ++line; 
		controls.add(dateValue, g);

		g.gridx = 0; g.gridy = ++line; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 10.0;
		controls.add(new JPanel(), g);

		pack();
	}

    void initLogging() {
//        Log.redirectOut();

        // Remove default handlers
        Handler[] old = rootLogger.getHandlers();
        for(int i=0; i<old.length; i++) {
            rootLogger.removeHandler(old[i]);
        }
        // Log all messages
        rootLogger.setLevel(Level.ALL);

        // Add Swing handler
        Handler handler = new SwingHandler();
        handler.setFormatter(new TrivialFormatter());
        rootLogger.addHandler(handler);

        // Start logging to file, if set in options
        updateFileLogging();

    }
        
    void updateFileLogging() {
        if (fileHandler!=null) {
            rootLogger.removeHandler(fileHandler);
            fileHandler.close();
            fileHandler = null;
        }
        // Add file handler
        if (options.logToFile) {
            try {
                fileHandler = new FileHandler(options.logFilename, options.logLimit, options.logRotate, options.logAppend);
                fileHandler.setFormatter(new TrivialFormatter());
                rootLogger.addHandler(fileHandler);
            } catch (IOException e) {
                e.printStackTrace();
                logger.info("Unable to write to log file");
            }
        }
    }

    
    public void actionPerformed(ActionEvent e) {
        //...Get information from the action event...
        String command = e.getActionCommand();

        // Process command
        if (command.equals("Open")) {
            open();
        } else if (command.equals("Save")) {
            Save();
        } else if (command.equals("Save as")) {
            SaveAs();
        } else if (command.equals("Query Database")) {
            Import();
        } else if (command.equals("Export GraphML")) {
            ExportGraphML();
        } else if (command.equals("Export Otter")) {
            ExportOtter();
        } else if (command.equals("Export daVinci")) {
            ExportDaVinci();
        } else if (command.equals("Export PostScript")) {
            ExportPostScript();
        } else if (command.equals("Print")) {
            Print();
        } else if (command.equals("Exit")) {
            logger.info("Exit");
            quit();
        } else if (command.equals("Show graph")) {
            showGraph = ((JCheckBoxMenuItem) e.getSource()).isSelected();
            if (scroll == null && showGraph) {
                updateView();
            }
            if (scroll != null && !showGraph) {
				updateView();
            }
        } else if (command.equals("Show none")) {
            if (view != null) {
                if (((JRadioButtonMenuItem) e.getSource()).isSelected()) showLabels = NodeComponent.SHOW_LABEL_NONE;
                view.setShowLabels(showLabels);
            }
        } else if (command.equals("Show labels")) {
            if (view != null) {
                if (((JRadioButtonMenuItem) e.getSource()).isSelected()) showLabels = NodeComponent.SHOW_LABEL_LABEL;
                view.setShowLabels(showLabels);
            }
        } else if (command.equals("Show names")) {
            if (view != null) {
                if (((JRadioButtonMenuItem) e.getSource()).isSelected()) showLabels = NodeComponent.SHOW_LABEL_NAME;
                view.setShowLabels(showLabels);
            }
        } else if (command.equals("Auto Zoom")) {
            autoZoom = ((JRadioButtonMenuItem) e.getSource()).isSelected();
            if (view != null) {
                if (autoZoom) {
                    view.setSize(scroll.getSize());
                    view.zoomFit();
                    //		      scroll.validate();
                }
                view.setZoomFixed(!autoZoom);
            }
        } else if (command.equals("Show Edges")) {
            showEdges = ((JCheckBoxMenuItem) e.getSource()).isSelected();
            if (view != null) {
                view.setShowEdges(showEdges);
            }
        } else if (command.equals("Fit")) {
            if (view != null) {
                view.setSize(scroll.getSize());
                view.zoomFit();
            }
        } else if (command.equals("Zoom In")) {
            if (view != null) {
                view.zoomIn();
                scroll.validate();
            }
        } else if (command.equals("Zoom Out")) {
            if (view != null) {
                view.zoomOut();
                scroll.validate();
            }
        } else if (command.equals("Show Log window")) {
            Log.showLog();
        } else if (command.equals("don't color")) {
            JRadioButtonMenuItem item = (JRadioButtonMenuItem) e.getSource();
            if (item.isSelected()) ColorEdges(null);
        } else if (command.equals("color by delay")) {
            JRadioButtonMenuItem item = (JRadioButtonMenuItem) e.getSource();
            if (item.isSelected()) ColorEdges("delay");
        } else if (command.equals("straight edges")) {
            JRadioButtonMenuItem item = (JRadioButtonMenuItem) e.getSource();
            if (item.isSelected()) {
				edgeStyle = EdgeComponent.STYLE_STRAIGHT;
				if(view!=null) view.setEdgeStyle(edgeStyle);
			}
        } else if (command.equals("curved edges")) {
            JRadioButtonMenuItem item = (JRadioButtonMenuItem) e.getSource();
            if (item.isSelected()) {
				edgeStyle = EdgeComponent.STYLE_CURVED;
				if(view!=null) view.setEdgeStyle(edgeStyle);
			}
		} else if (command.equals("Layout...")) {
            Layout();
        } else if (command.equals("Convert to AS graph")) {
            ConvertToAS();
        } else if (command.equals("Convert to router graph")) {
            ConvertToRouter();
        } else if (command.equals("Remove dull nodes")) {
            RemoveDullNodes();
        } else if (command.equals("Merge bidirectional edges")) {
            MergeBidirEdges();
        } else if (command.equals("Show database freshness")) {
            ShowDatabaseFreshness();
        } else if (command.equals("Abort current layout")) {
            AbortLayout();
        } else if (command.equals("Options")) {
            Options();
        } else if (command.equals("Show suspicious ASs")) {
            showSuspiciousAS();
        } else if (command.equals("Cleanup cache")) {
            retryAS0();
        } else if (command.equals("Delay calculations")) {
            DelayCalculations();
        } else if (command.equals("Segment Graph")) {
            GetSegmentGraph();
        } else if (command.equals("Clear busy flag")) {
            busy = false;
        } else if (command.equals("Dump IP addresses")) {
            DumpAddresses();
        } else if (command.equals("Show memory")) {
            ShowMemory();
        } else if (command.equals("Test")) {
            Test();
	    } else if (command.equals("View README")) {
            ViewReadme();
        } else if (command.equals("Check for new version")) {
            testNewVersion();
	    } else if (command.equals("About...")) {
            About();
        } else
            logger.info(command + " doesn't do anything");
    }

    void open() {
        if (!checkSaveFirst())
            return;
        JFileChooser chooser = new JFileChooser();

        if (!isApplet) {
            chooser.setCurrentDirectory(new File(prefs.get(PREFS_OUTPUT_PATH, System.getProperty("user.dir"))));
        }
        chooser.setDialogTitle("Open");
        chooser.setFileFilter(FILE_FILTER_XML);
        int returnVal = chooser.showOpenDialog(self);

        if (returnVal == JFileChooser.APPROVE_OPTION) {
            if (!isApplet)
                prefs.put(PREFS_OUTPUT_PATH, chooser.getCurrentDirectory().getPath());
            final String filename = chooser.getSelectedFile().getAbsolutePath();

            if (busy) {
                logger.info("Already doing something!");
            } else {
                openFilename(filename);
            }
        }
    }

    void openFilename(final String filename) {
        busy = true;
        Thread t = new Thread() {
                public void run() {
                    logger.info("Starting open...");

                    if (hash == null) readASHash();
                    logger.info("Opening " + filename);
                    try {
                        TestBox.clearCache();
                        graph = DelayGraph.readXML(hash, filename);
                        currentFilename = filename;
                        setTitle(BASE_TITLE + " " + filename);
                    } catch (Exception e) {
                        logger.log(Level.INFO, "Error reading graph from " + filename, e);
                        busy = false;
                        return;
                    }
                    if (graph != null) {
                        graph.attributeASData(hash);
                        if (hash.isModified()) writeASHash();
                        updateView();
                        if (view != null) view.setModified(false);
                    }
                    logger.info("Finished.");
                    busy = false;
                }
            };

        t.start();
    }

    boolean Save() {
        if (currentFilename == null)
            return SaveAs();
        else if (graph != null) {
            return doSave(currentFilename);
        }
        return false;
    }

    boolean SaveAs() {
        if (graph != null) {
            JFileChooser chooser = new JFileChooser();

            if (!isApplet) {
                chooser.setCurrentDirectory(new File(prefs.get(PREFS_OUTPUT_PATH, System.getProperty("user.dir"))));
            }
            //	      chooser.setDialogTitle("Save as");
            //	      chooser.setApproveButtonText("Save");
            chooser.setFileFilter(FILE_FILTER_XML);
            int returnVal = chooser.showSaveDialog(self);

            if (returnVal == JFileChooser.APPROVE_OPTION) {
                if (!isApplet)
                    prefs.put(PREFS_OUTPUT_PATH, chooser.getCurrentDirectory().getPath());
                String filename = chooser.getSelectedFile().getAbsolutePath();

                return doSave(filename);
            }
        }
        return false;
    }

    boolean doSave(String filename) {
        try {
            logger.info("Saving graph to " + filename);
            graph.includeTracesInXML = options.includeTracesXML;
            graph.exportXML(filename);
            currentFilename = filename;
            if (view != null) view.setModified(false);
            setTitle(BASE_TITLE + " " + filename);
            return true;
        } catch (IOException e) {
            logger.log(Level.INFO, "Error saving to " + filename + ": " + e.getMessage(), e);
            return false;
        }
    }

    void Import() {
        if (!checkSaveFirst())
            return;
        final ImportDialog dialog = new ImportDialog(this, options);

        dialog.setLocationRelativeTo(this);
        dialog.show();
        int result = dialog.returnValue;
        dialog.dispose();
		logger.info("Dialog: interval="+dialog.interval);
        logger.info("Dialog result is " + result);
        if (result == dialog.RESULT_OK) {
            if (busy) {
                logger.info("Already doing something!");
            } else {
                busy = true;
                Thread t = new Thread() {
                        public void run() {
                            logger.info("Starting...");
                            doIt(dialog);
                            logger.info("Finished.");
                            busy = false;
                        }
                    };

                t.start();
            }

        }
    }

    String getTTMURL() {
        String password;
        if (options.DBStorePassword) {
            password = options.DBPassword;
        } else {
            password = JOptionPane.showInputDialog(null, "Password: ", 
                "Database password", JOptionPane.QUESTION_MESSAGE);
        }
        return "jdbc:" + options.DBType + "://" + options.DBHost + 
            "/" + options.DBDatabase +
            "?user=" + options.DBUser + "&password=" + password;
    }

    void doIt(ImportDialog dialog) {
        Vector traces = new Vector();

        try {
            TTMData ttm = new TTMData(getTTMURL());

            if (options.useBackup) {
                ttm.setTables("Backup_Records", "Backup_Routes", "Delays");
            } else {
                ttm.setTables("Records", "Routes", "Delays");
            }

            ttm.setUseDNS(dialog.useDNS);

            logger.info("Reading traces...");
            if (dialog.queryType == dialog.QUERY_FULL) {
                if (dialog.readDelays) {
                    Date date = (Date) dialog.dates.iterator().next();
                    logger.info("Only traces at one time read! "+date.toString());
                    traces = ttm.getDelaySnapshot(date);
                } else {
                    traces = ttm.getTracesSnapshot(dialog.dates, dialog.interval);
                }
            } else if (dialog.queryType == dialog.QUERY_PARTIAL) {
                if (dialog.readDelays) {
                    Date date = (Date) dialog.dates.iterator().next();
                    logger.info("Only traces at one time read! "+date.toString());
                    traces = ttm.getPartialDelaySnapshot(dialog.from, dialog.to, date);
                } else {
                    traces = ttm.getPartialTracesSnapshot(dialog.from, dialog.to, dialog.dates, dialog.interval);
                }
                //	      } else if (dialog.queryType == dialog.QUERY_SINGLE) {
                //		  traces=ttm.getTrace(dialog.from,dialog.to, dialog.dates);
            }
            ttm.close();
            logger.info("Finished reading, now converting to graph");

            // Convert traces to Graph
            graph = new DelayGraph();
            currentFilename = null;
            setTitle(BASE_TITLE);

            TestBox.clearCache();
            graph.addNodes(TestBox.getTestBoxSet());
            for (Enumeration e1 = traces.elements(); e1.hasMoreElements();) {
                Trace trace = (Trace) e1.nextElement();
                graph.addEndToEnd(trace);
				logger.info("Trace from "+((TestBox)trace.from).number+" to "+((TestBox)trace.to).number);
            }
            graph.removeUnconnectedNodes();
            if (hash == null) readASHash();
            graph.attributeASData(hash);
            if (hash.isModified()) writeASHash();
            logger.info("Number of nodes in IP graph: " + graph.nodes.size() + "; number of edges: " + graph.edges.size());
        } catch (Exception e) {
            logger.log(Level.INFO, "Error reading AS data: " + e.getMessage(), e);
        }
        if (graph != null && showGraph) {
            updateView();
            if (view != null) view.setModified(true);
        }
    }

    void ShowDatabaseFreshness() {
        try {
			logger.info("Opening TTM database");
            TTMData ttm = new TTMData(getTTMURL());

            if (options.useBackup) {
                ttm.setTables("Backup_Records", "Backup_Routes", "Delays");
            } else {
                ttm.setTables("Records", "Routes", "Delays");
            }

			logger.info("Getting data of last entry");
            Date date = ttm.getLastEntry();
            ttm.close();

            JOptionPane.showMessageDialog(null, 
                "Date of last record in database: " +
                date.toString());
        } catch (Exception e) {
            logger.log(Level.INFO, "Error in reading database freshness", e);
        }
    }

    void ExportGraphML() {
        if (graph != null) {
            JFileChooser chooser = new JFileChooser();

            if (!isApplet) {
                chooser.setCurrentDirectory(new File(prefs.get(PREFS_OUTPUT_PATH, System.getProperty("user.dir"))));
            }
            chooser.setDialogTitle("Export to GraphML");
            chooser.setFileFilter(FILE_FILTER_XML);
            int returnVal = chooser.showSaveDialog(self);

            if (returnVal == JFileChooser.APPROVE_OPTION) {
                if (!isApplet)
                    prefs.put(PREFS_OUTPUT_PATH, chooser.getCurrentDirectory().getPath());
                String filename = chooser.getSelectedFile().getAbsolutePath();

                try {
                    logger.info("Exporting graph to " + filename);
                    graph.exportGraphML(filename);
                } catch (IOException e) {
                    logger.log(Level.INFO, "Error exporting GraphML to " + filename + ": " + e.getMessage(),e);
                }
            }
        }
    }

    void ExportOtter() {
        if (graph != null) {
            JFileChooser chooser = new JFileChooser();

            if (!isApplet) {
                chooser.setCurrentDirectory(new File(prefs.get(PREFS_OUTPUT_PATH, System.getProperty("user.dir"))));
            }
            chooser.setDialogTitle("Export to Otter");
            chooser.setFileFilter(FILE_FILTER_OTTER);
            int returnVal = chooser.showSaveDialog(self);

            if (returnVal == JFileChooser.APPROVE_OPTION) {
                if (!isApplet)
                    prefs.put(PREFS_OUTPUT_PATH, chooser.getCurrentDirectory().getPath());
                String filename = chooser.getSelectedFile().getAbsolutePath();

                try {
                    logger.info("Exporting graph to " + filename);
                    graph.exportOtter(filename);
                } catch (IOException e) {
                    logger.log(Level.INFO, "Error exporting to " + filename + ": " + e.getMessage(),e);
                }
            }
        }
    }

    void ExportDaVinci() {
        if (graph != null) {
            JFileChooser chooser = new JFileChooser();

            if (!isApplet) {
                chooser.setCurrentDirectory(new File(prefs.get(PREFS_OUTPUT_PATH, System.getProperty("user.dir"))));
            }
            chooser.setDialogTitle("Export to daVinci");
            chooser.setFileFilter(FILE_FILTER_DAVINCI);
            int returnVal = chooser.showSaveDialog(self);

            if (returnVal == JFileChooser.APPROVE_OPTION) {
                if (!isApplet)
                    prefs.put(PREFS_OUTPUT_PATH, chooser.getCurrentDirectory().getPath());
                String filename = chooser.getSelectedFile().getAbsolutePath();

                try {
                    logger.info("Exporting graph to " + filename);
                    graph.exportdaVinci(filename);
                } catch (IOException e) {
                    logger.log(Level.INFO, "Error exporting to " + filename + ": " + e.getMessage(),e);
                }
            }
        }
    }

    void ExportPostScript() {
        if (view == null) {
            logger.info("Nothing to export!");
            return;
        }

        JFileChooser chooser = new JFileChooser();

        if (!isApplet) {
            chooser.setCurrentDirectory(new File(prefs.get(PREFS_OUTPUT_PATH, System.getProperty("user.dir"))));
        }
        chooser.setDialogTitle("Export Postscript");
        chooser.setFileFilter(FILE_FILTER_POSTSCRIPT);
        int returnVal = chooser.showSaveDialog(self);

        if (returnVal == JFileChooser.APPROVE_OPTION) {
            if (!isApplet)
                prefs.put(PREFS_OUTPUT_PATH, chooser.getCurrentDirectory().getPath());
            String filename = chooser.getSelectedFile().getAbsolutePath();

            try {
                logger.info("Exporting graph to " + filename);
                DocFlavor flavor = DocFlavor.SERVICE_FORMATTED.PRINTABLE;
                String psMimeType = DocFlavor.BYTE_ARRAY.POSTSCRIPT.getMimeType();
                FileOutputStream fos;
                StreamPrintService psPrinter;
                StreamPrintServiceFactory[]spsFactories = StreamPrintServiceFactory.lookupStreamPrintServiceFactories(flavor, psMimeType);

                if (spsFactories.length == 0) {
                    logger.info("No PostScript converter available in Java VM");
                    return;
                }
                fos = new FileOutputStream(filename);
                psPrinter = spsFactories[0].getPrintService(fos);
                DocPrintJob job = psPrinter.createPrintJob();
                PrintRequestAttributeSet aset = new HashPrintRequestAttributeSet();
                Doc doc = new SimpleDoc(view, flavor, null);

                job.print(doc, aset);
                fos.close();
            } catch (Exception e) {
                    logger.log(Level.INFO, "Error exporting to " + filename + ": " + e.getMessage(),e);
            }
        }
    }

    void DumpAddresses() {
        for (Iterator i = graph.nodes.iterator(); i.hasNext();) {
            Node node = (Node) i.next();

            logger.info(node.getName());
        }
    }

    void Print() {
        if (view == null) {
            logger.info("Nothing to print!");
            return;
        }

        try {
            logger.info("Printing");
            PrintRequestAttributeSet aset = new HashPrintRequestAttributeSet();

            aset.add(new JobName("Traces " + currentFilename, null));
            PrinterJob pj = PrinterJob.getPrinterJob();

            pj.setPrintable(view);

            PrintService[] services = PrinterJob.lookupPrintServices();

            if (services.length == 0) {
                logger.info("No printers available!");
            } else {
                pj.setPrintService(services[0]);
                //		  pj.pageDialog(aset);
                if (pj.printDialog(aset)) {
                    pj.print(aset);
                }
            }
            logger.info("Finished Printing");
        } catch (Exception e) {
            logger.log(Level.INFO, "Error printing!",e);
        }
    }

    public void About() {
        JOptionPane.showMessageDialog(null, 
            "Traces v"+Version.VERSION+", built "+Version.VERSION_DATE+".\n" +
            "Uses graph v"+nl.tudelft.tbm.graph.Version.VERSION+".\n"+
            "Uses matrix v"+nl.tudelft.tbm.matrix.Version.VERSION+".\n"+
            "Written by Jan-Pascal van Best, janpascal@vanbest.org.\n\n"+
            "Developed at the ICT group of the Technology, Policy and Management\n"+
            "department of Delft University of Technology. See for more information\n"+
            "about the group and its research http://www.ict.tbm.tudelft.nl/\n\n"+
            "See web site at http://www.tbm.tudelft.nl/webstaf/janb/software/index.html \n"+
            "for updates, source code and more information.\n\n" +
            "Licensed under the GNU General Public License, see\nCOPYING file "+
            "for details.");
    }

    public void Options() {
        logger.info("Doing options...");
        String saveASCache = options.ASCacheLocation;
        String saveTestbox = options.testboxLocation;

        options.setLocationRelativeTo(this);
        options.setVisible(true);
        Log.MAX_LINES = options.maxLogLines;
        if (!saveASCache.equals(options.ASCacheLocation)) hash = null;
        if (!saveTestbox.equals(options.testboxLocation)) {
            TestBox.dataFilename = options.testboxLocation;
            TestBox.clearCache();
        }
        if(options.logFileSettingsChanged()) {
            updateFileLogging();
        }
        logger.info("Done options");
    }

    public void ConvertToAS() {
        if (!checkSaveFirst()) return;

        if (busy) {
            logger.info("Already doing something!");
        } else {
            busy = true;
            Thread t = new Thread() {
                    public void run() {
                        logger.info("Reducing graph using autonomous systems");
                        FilterDialog dialog = new FilterDialog(null);
                        dialog.show();
                        int filter = dialog.filter;
                        logger.info("Selected filter mask: "+filter+" means "+DelayGraph.filterToString(filter));
                        dialog.dispose();
                        if(filter<0) {
                            busy=false;
                            return;
                        }

                        if (hash == null) readASHash();
                        DelayGraph newGraph = graph.reduceUsingAutonomousSystems(hash, filter, false);

                        if (hash.isModified()) writeASHash();
                        logger.info("Number of nodes in AS graph: " + newGraph.nodes.size() + "; number of edges: " + newGraph.edges.size());

                        graph = newGraph;
                        currentFilename = null;
                        setTitle(BASE_TITLE);

                        if (graph != null) {
                            updateView();
                            if (view != null) view.setModified(true);
                        }
                        logger.info("Finished.");
                        busy = false;
                    }
                };

            t.start();
        }
    }

    public void Layout() {
        if (busy) {
            logger.info("Already doing something!");
        } else {
            if (currentLayout != null) {
                if (!currentLayout.isLayouting()) {
                    currentLayout = null;
                } else {
                    int option = JOptionPane.showConfirmDialog(null, "Busy layouting! Abort running layout?", "Already layouting", JOptionPane.YES_NO_OPTION);

                    if (option == JOptionPane.NO_OPTION) {
                        return;
                    } else {
                        logger.info("Interrupting layout, stand by");
                        currentLayout.interruptLayout();
                        while (currentLayout.isLayouting()) {
                            try {
                                Thread.sleep(10);
                            } catch (InterruptedException e) {
                            }
                        }
                        logger.info("Layout interrupted successfully");
                        currentLayout = null;
                    }
                }
            }

            LayoutDialog dialog = new LayoutDialog(this, options);

            dialog.setLocationRelativeTo(this);
            dialog.show();
            if (dialog.returnValue) {
                logger.info("Layouting...");
                currentLayout = dialog.createLayout(graph, view);
                currentLayout.doLayout();
                java.util.Timer timer = new java.util.Timer(true); // daemon timer
                TimerTask task = new TimerTask() {
                    public void run() {
                        if(currentLayout!=null && !currentLayout.isLayouting()) {
                            currentLayout = null;
                            cancel();
                        }
                    }
                };
                timer.schedule(task, 100, 100); // Check each 100 ms
            }
        }
    }

    public void AbortLayout() {
        // This is not completely thread safe: the thread could end
        // in the middle of this statement, nulling currentLayout!
        if (currentLayout != null && currentLayout.isLayouting()) {
            logger.info("Interrupting layout, stand by");
            currentLayout.interruptLayout();
            while (currentLayout.isLayouting()) {
                try {
                    Thread.sleep(10);
                } catch (InterruptedException e) {
                    break;
                }
            }
            logger.info("Layout interrupted successfully");
        }
        currentLayout = null;
    }

    public void ColorEdges(String property) {
		edgeColorProperty = property;
        if(view!=null) {
            logger.info("Colouring edges by "+property+"...");
            view.setEdgeColorProperty(property);
            logger.info("Finished coloring edges");
        }
    }

    public void ConvertToRouter() {
        if (!checkSaveFirst()) return;

        JFileChooser chooser = new JFileChooser();

        if (!isApplet) {
            chooser.setCurrentDirectory(new File(prefs.get(PREFS_OUTPUT_PATH, System.getProperty("user.dir"))));
        }
        chooser.setDialogTitle("Locate IP->Router map");
        chooser.setFileFilter(FILE_FILTER_TEXT);
        int returnVal = chooser.showOpenDialog(self);

        if (returnVal != JFileChooser.APPROVE_OPTION) {
            return;
        }

        if (!isApplet) {
            prefs.put(PREFS_OUTPUT_PATH, chooser.getCurrentDirectory().getPath());
        }
        String filename = chooser.getSelectedFile().getAbsolutePath();

        final Map map = new HashMap();

        try {
            BufferedReader in = new BufferedReader(new FileReader(filename));
            String s;

            do {
                s = in.readLine();
                if (s != null) {
                    StringTokenizer tok = new StringTokenizer(s);
                    int count = tok.countTokens();

                    if (count == 0 || count > 2) {
                        logger.info("Strange line: " + s);
                    }
                    if (count == 2) {
                        String hostString = tok.nextToken();
                        String routerString = tok.nextToken();

                        try {
                            InetAddress host = InetAddress.getByName(hostString);
                            InetAddress router = InetAddress.getByName(routerString);

                            map.put(host, router);
                        } catch (UnknownHostException e) {
                            logger.info("Error parsing IP addresses in line " + s);
                        }
                    }
                }
            }
            while (s != null);
            in.close();
        } catch (IOException e) {
            logger.log(Level.INFO, "Error reading router mapping from " + filename,e);
            return;
        }

        if (busy) {
            logger.info("Already doing something!");
        } else {
            busy = true;
            Thread t = new Thread() {
                    public void run() {
                        logger.info("Reducing graph using router data");
                        DelayGraph newGraph = graph.reduceUsingRouters(map);

                        logger.info("Number of nodes in router graph: " + newGraph.nodes.size() + "; number of edges: " + newGraph.edges.size());

                        graph = newGraph;
                        currentFilename = null;
                        setTitle(BASE_TITLE);

                        if (graph != null) {
                            updateView();
                            if (view != null) view.setModified(true);
                        }
                        logger.info("Finished.");
                        busy = false;
                    }
                };

            t.start();
        }
    }

    public void RemoveDullNodes() {
        if (graph == null) return;
        if (busy) {
            logger.info("Already doing something!");
        } else {
            busy = true;
            Thread t = new Thread() {
                    public void run() {
                        graph.removeDullNodes();
                        updateView();
                        busy = false;
                    }
                };

            t.start();
        }
    }

    public void MergeBidirEdges() {
        if (graph == null) return;
        if (busy) {
            logger.info("Already doing something!");
        } else {
            graph.mergeBidirEdges();
            if (view!=null) view.repaint();
        }
    }


    public void GetSegmentGraph() {
        if (graph == null) return;
        if (busy) {
            logger.info("Already doing something!");
        } else {
            busy = true;
            Thread t = new Thread() {
                    public void run() {
                        graph = graph.getSegmentGraph();
                        updateView();
                        busy = false;
                    }
                };

            t.start();
        }
    }

    public void showSuspiciousAS() {
        if (busy) {
            logger.info("Already doing something!");
        } else {
            busy = true;
            Thread t = new Thread() {
                    public void run() {
                        if (hash == null) readASHash();
                        hash.printSuspiciousEntries();
                        busy = false;
                    }
                };

            t.start();
        }
    }

    public void retryAS0() {
        if (busy) {
            logger.info("Already doing something!");
        } else {
            busy = true;
            Thread t = new Thread() {
                    public void run() {
                        if (hash == null) readASHash();
                        hash.retryAS0();
                        if (hash.isModified()) writeASHash();
                        logger.info("Finished cleanup");
                        busy = false;
                    }
                };

            t.start();
        }
    }

	void notifyFilters(Graph graph) {
		for(Iterator i=filters.iterator(); i.hasNext(); ) {
			((GraphFilter) i.next()).graphChanged(graph);
		}
	}

    void DelayCalculations() {
        if (graph == null) return;
        if (busy) {
            logger.info("Already doing something!");
            return;
        }
        busy = true;
        Thread t = new Thread() {
                public void run() {
                    graph.buildEquations();
                    if(view!=null) view.setEdgeColorProperty("delay");
                    logger.info("Finished delay calculations");
					notifyFilters(graph);
                    busy = false;
                }
            };

        t.start();
    }

    void ShowMemory() {
        Runtime runtime = Runtime.getRuntime();
        runtime.gc();
//        logger.info("Max Memory:   "+runtime.maxMemory());
//        logger.info("Total Memory: "+runtime.totalMemory());
//        System.out.println("Free Memory:  "+runtime.freeMemory());
        System.out.println("Used Memory:  "+(runtime.totalMemory()-runtime.freeMemory()));
    }

    void showThreads() {
        ThreadGroup group = Thread.currentThread().getThreadGroup();
        int count = group.activeCount();
        System.out.println("Threads:      "+count);
        Thread[] threads = new Thread[2*count];
        group.enumerate(threads);
        for(int i=0; i<threads.length; i++) {
            Thread thread = threads[i];
            if(thread != null) {
                System.out.println("Thread "+i+":      "+thread.getName());
            }
        }
    }

    void Test() {
        Frame f = new Frame("Object browser");
        TreeNode root = new ReflectTreeNode("root", this, null);
        JTree tree = new JTree(root);
        JScrollPane scroll = new JScrollPane(tree);
        f.add(scroll);
        f.setSize(1024, 900);
        f.show();
        
//        Test.showFields(this);
//        AutSystem AS32 = hash.getAS(32);
        //logger.info("Out edges of AS32: \n"+graph.outEdges(AS32));
        //logger.info("In edges of AS32: \n"+graph.inEdges(AS32));
    }

    void updateView() {
        if (scroll != null) {
            graphBox.remove(scroll);
        }
        if (showGraph) {
            logger.info("Updating view with new graph, stand by...");
            view = new GraphComponent(graph, edgeColorProperty, edgeStyle);

			for(Iterator i=filters.iterator(); i.hasNext(); ) {
				((GraphFilter) i.next()).clearListeners();
			}
            notifyFilters(graph);
            view.addGraphFilters(filters);

            scroll = new JScrollPane(view);
            graphBox.add(scroll);
            validate();
            view.setShowLabels(showLabels);
            view.setZoomFixed(!autoZoom);
            view.setShowEdges(showEdges);
            if (autoZoom) view.setSize(scroll.getSize());
        } else {
            scroll = null;
            view = null;
        }
    }

    void readASHash() {
        if (new File(options.ASCacheLocation).canRead()) {
            try {
                hash = ASHash.readASHash(options.ASCacheLocation);
            } catch (Exception e) {
                logger.info("Failed reading AS data, starting with empty cache");
                hash = new ASHash(true);
            }
        } else {
            logger.info("No AS data, starting with empty cache");
            hash = new ASHash(true);
        }
    }

    void writeASHash() {
        try {
            hash.consolidate();
            hash.writeASHash(hash, options.ASCacheLocation);
        } catch (Exception e) {
            logger.info("Error writing AS hash to " + options.ASCacheLocation);
        }
    }

    public void quit() {
        if (busy) {
            try {
                if (JOptionPane.showConfirmDialog(this, "Still busy! Really quit?", "Warning", JOptionPane.OK_CANCEL_OPTION) == JOptionPane.CANCEL_OPTION) {
                    return;
                }
            } catch (HeadlessException e) {
                logger.log(Level.INFO, "Can't give a warning that I'm still busy", e);
                return;
            }
        }

        if (view != null && view.isModified()) {
            try {
                int option = JOptionPane.showConfirmDialog(this, "File '" + currentFilename + "' is modified. Save changes?", "Warning", JOptionPane.YES_NO_CANCEL_OPTION);

                switch (option) {
                case JOptionPane.YES_OPTION:
                    Save();
                    break;

                case JOptionPane.NO_OPTION:
                    break;

                case JOptionPane.CANCEL_OPTION:
                    return;
                }
            } catch (HeadlessException e) {
                logger.log(Level.INFO, "Cannot display confirm dialog", e);
                return;
            }
        }

        if (hash != null && hash.isModified()) writeASHash();
        try {
            Whois.exportServersXML(options.whoisServersFilename);
        } catch (IOException e) {
            logger.warning("Error writing whois server list");
        }

        // Shut down logging
        if (fileHandler!=null) {
            rootLogger.removeHandler(fileHandler);
            fileHandler.close();
        }
        Handler[] old = rootLogger.getHandlers();
        for(int i=0; i<old.length; i++) {
            rootLogger.removeHandler(old[i]);
        }

//        showThreads();
        logger.info("Exiting!");
        System.exit(0);
    }

	void testNewVersion() {
		Properties properties = new Properties();
		try {
			properties.load(new URL(Version.CURRENT_VERSION_URL).openStream());
		} catch (IOException e) {
			logger.info("Error reading version from "+Version.CURRENT_VERSION_URL);
		}
		String newest = properties.getProperty("release");
		String beta = properties.getProperty("beta");
		if (newest==null) newest="Unknown (see Log window)";
		if (beta==null) beta="Unknown (see Log window)";
		String message = "This is Traces v"+Version.VERSION+"\n" +
			"Latest release: "+newest+"\n" +
			"Latest beta: "+beta + "\n\n" +
			"See http://www.tbm.tudelft.nl/webstaf/janb/software/index.html\n"+
			"for downloads and CVS access (for betas)";

		JOptionPane.showMessageDialog(null, message);
	}

	void ViewReadme() {
		URL url = ClassLoader.getSystemResource(README_LOCATION);
		logger.info("Using README from "+url.toString());
		try {
			final JEditorPane pane = new JEditorPane(url);
			pane.setEditable(false);
			pane.addHyperlinkListener(new HyperlinkListener() {
				public void hyperlinkUpdate(HyperlinkEvent e) {
					if (e.getEventType()==HyperlinkEvent.EventType.ACTIVATED) {
						try {
							pane.setPage(e.getURL());
						}
						catch (IOException ex) {
							logger.info("Failed to load linked document "+e.getURL());
						}
					}
				}
			});
			HTMLDocument HTML = (HTMLDocument) pane.getDocument();
			HTML.setBase(ClassLoader.getSystemResource(""));
			JFrame README = new JFrame("README for Traces v"+Version.VERSION);
			README.getContentPane().add(new JScrollPane(pane));
			README.setResizable(true);
			README.setSize(800, 600);
//			README.pack();
			README.show();
//			JOptionPane.showMessageDialog(this, pane, "README for Traces v"+Version.VERSION, JOptionPane.PLAIN_MESSAGE);
		} catch (IOException e) {
			logger.info("Unable to read README from "+url.toString());
		}
	}

    boolean checkSaveFirst() {
        if (view == null || !view.isModified())
            return true;

        try {
            int option = JOptionPane.showConfirmDialog(this, "File '" + currentFilename + "' is modified. Save changes?", "Warning", JOptionPane.YES_NO_CANCEL_OPTION);

            switch (option) {
            case JOptionPane.YES_OPTION:
                return Save();

            case JOptionPane.NO_OPTION:
                return true;

            case JOptionPane.CANCEL_OPTION:
                return false;

            default:
                return false;
            }
        } catch (HeadlessException e) {
            logger.log(Level.INFO, "Cannot display modified dialog", e);
            return false;
        }
    }

}

class FilterDialog extends JDialog {
    public int filter = -1; // -1 if cancel was pressed

    String[] strings;
    JCheckBox[] cb;
    JButton bOK;
    JButton bCancel;

    private static Logger logger = Logger.getLogger("nl.tudelft.tbm.traces");

    FilterDialog(Frame owner) {
        super(owner, "Select filter for traces", true);

        Container c = getContentPane();
        c.setLayout(new GridBagLayout());

        // Create a constraints object, and specify some default values
        GridBagConstraints g = new GridBagConstraints();
        g.fill = GridBagConstraints.BOTH;   // components grow in both dimensions
        g.insets = new Insets(5, 5, 5, 5); // 5-pixel margins on all sides

        strings  = DelayGraph.AS_FILTER_STRINGS;
        // First string is None; last is All
        cb = new JCheckBox[strings.length-2];
        for(int i=0; i<cb.length; i++) {
            cb[i] = new JCheckBox(strings[i+1], false);
        }

        bOK = new JButton("OK");
        bOK.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bOK) {
                        logger.info("Ok Pressed!");
                        calculateInput();
                        hide();
                    }
                }
            }
        );

        bCancel = new JButton("Cancel");
        bCancel.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bCancel) {
                        hide();
                    }
                }
            }
        );

        int line=-1;

        g.gridx = 0; g.gridy = ++line; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 0.0;
        c.add(new JLabel("Filter traces:"), g);
        line--;
        for(int i=0; i<cb.length; i++) {
            g.gridx = 1; g.gridy = ++line; g.gridwidth = 1; g.gridheight = 1;
            g.weightx = 1.0; g.weighty = 0.0;
            c.add(cb[i], g);
        }

        g.gridx = 0; g.gridy = ++line; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 0.0;
        c.add(bOK, g);

        g.gridx = 1; g.gridy = line; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 0.0;
        c.add(bCancel, g);

        pack();
    }

    void calculateInput() {
        filter = 0;
        logger.info("Calculating!");
        for(int i=0; i<cb.length; i++) {
            if(cb[i].isSelected()) {
                filter |= 1<<i;
            }
        }
    }
}


