// This progarm is intended to query and display  from remote MySql database server
// that holds the TTM database file.

package nl.tudelft.tbm.traces;


import java.net.URL.*;
import java.sql.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.text.*;
import java.util.prefs.*;
import java.security.*;


public class OptionsDialog extends JDialog {
    private static final String PREFS_DB_TYPE = "database_type";
    private static final String PREFS_DB_LOCATION = "database_location";
    private static final String PREFS_DB_NAME = "database_name";
    private static final String PREFS_DB_BACKUP = "database_use_backup";
    private static final String PREFS_DB_USER = "database_user";
    private static final String PREFS_DB_PASSWORD = "database_password";
    private static final String PREFS_DB_STORE_PASSWORD = "store_database_password";
    private static final String PREFS_AS_CACHE_LOCATION = "ascache_location";
    private static final String PREFS_TESTBOX_LOCATION = "testbox_datafile_location";
    private static final String PREFS_MAX_LOG_LINES = "lines_in_log";
    private static final String PREFS_LOG_TO_FILE = "log_to_file";
    private static final String PREFS_LOG_FILENAME = "log_filename";
    private static final String PREFS_LOG_LIMIT = "log_limit";
    private static final String PREFS_LOG_USE_ROTATE = "log_use_rotate";
    private static final String PREFS_LOG_ROTATE = "log_rotate";
    private static final String PREFS_LOG_APPEND = "log_append";
    private static final String PREFS_XML_INCLUDE_TRACES = "output_traces";
    private static final String PREFS_WHOIS_SERVERS_FILENAME = "whois_servers";

//    String DBURL;

    String DBType;
    String DBHost;
    String DBDatabase;
    String DBUser;
    String DBPassword;
    boolean DBStorePassword;
    
    String ASCacheLocation;
    String testboxLocation;
    boolean useBackup;
    boolean includeTracesXML;
    String whoisServersFilename;

    // Logging
    int maxLogLines;

    boolean logToFile;
    String logFilename; 
    int logRotate;      // 1 if no rotation
    int logLimit;
    boolean logAppend;  // true if append to existing log file

    private JRadioButton rbDBTypeMySQL;
    private JRadioButton rbDBTypeUnknown;
    private ButtonGroup bgDBType;

    private JTextField tfDBLocation;
    private JTextField tfDBName;
    private JCheckBox cbDBBackup;
    private JTextField tfDBUser;
    private JTextField tfDBPassword;
    private JCheckBox cbDBStorePassword;

    private JTextField tfASCacheLocation;
    private JButton bASCacheBrowse;
    private JButton bTestboxBrowse;
    private JTextField tfTestboxLocation;
    private JTextField tfWhoisServersFilename;
    private JButton bWhoisBrowse;

    private JSpinner spLogLines;
    private JCheckBox cbLogToFile;
    private JTextField tfLogFilename;
    private JButton bLogFilenameBrowse;
    private JCheckBox cbLogUseRotation;
    private JSpinner spLogRotation;
    private JCheckBox cbLogAppend;
    private JSpinner spLogLimit;

    private JCheckBox cbIncludeTraces;

    private JButton bOK;
    private JButton bCancel;

    private JDialog self = this;

    private boolean logFileIsChanged = false;

    public OptionsDialog(Frame owner) {
        super(owner, "Options", true);
        //	  super(owner, "Options", true);

        bgDBType = new ButtonGroup();
        rbDBTypeMySQL = new JRadioButton("MySQL", true);
        rbDBTypeUnknown = new JRadioButton("unknown", false);
        bgDBType.add(rbDBTypeMySQL);
        bgDBType.add(rbDBTypeUnknown);

        tfDBLocation = new JTextField("255.255.255.255");
        tfDBName = new JTextField("DATABASE_NAME");
        cbDBBackup = new JCheckBox("Use Backup", false);
        tfDBUser = new JTextField("USER_NAME_LONG");
        tfDBPassword = new JPasswordField("LONG_PASSWORD");
        cbDBStorePassword = new JCheckBox("Remember password", false);

        tfASCacheLocation = new JTextField("/home/user_name_long/path/AS.dat");
        tfTestboxLocation = new JTextField("/home/user_name_long/path/testboxes.txt");
        tfWhoisServersFilename = new JTextField("/home/user_name_long/path/testboxes.txt");

        spLogLines = new JSpinner(new SpinnerNumberModel(10000, 10, 9999999, 100));
        cbLogToFile = new JCheckBox("Dump log to disk", false);
        tfLogFilename = new JTextField("/home/user_name_long/path/traces.log");
        bLogFilenameBrowse = new JButton("Browse");
        bLogFilenameBrowse.addActionListener(new BrowseListener(this, bLogFilenameBrowse, tfLogFilename));
        cbLogUseRotation = new JCheckBox("Rotate log", false);
        spLogRotation = new JSpinner(new SpinnerNumberModel(3, 1, 999999, 1));
        cbLogAppend = new JCheckBox("Append to log file", false);
        spLogLimit = new JSpinner(new SpinnerNumberModel(1024*1024, 100, 1024*1024*1024, 10240));

        cbIncludeTraces = new JCheckBox("Include trace data in XML output", false);
        bASCacheBrowse = new JButton("Browse");
        bASCacheBrowse.addActionListener(new BrowseListener(this, bASCacheBrowse, tfASCacheLocation));
/*
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bASCacheBrowse) {
                        JFileChooser chooser = new JFileChooser();
                        File f = new File(tfASCacheLocation.getText());

                        chooser.setSelectedFile(f);
                        int returnVal = chooser.showOpenDialog(self);

                        if (returnVal == JFileChooser.APPROVE_OPTION) {
                            tfASCacheLocation.setText(chooser.getSelectedFile().getAbsolutePath());
                        }
                    }
                }
            }
        );
*/
        bTestboxBrowse = new JButton("Browse");
        bTestboxBrowse.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bTestboxBrowse) {
                        JFileChooser chooser = new JFileChooser();
                        File f = new File(tfTestboxLocation.getText());

                        chooser.setSelectedFile(f);
                        int returnVal = chooser.showOpenDialog(self);

                        if (returnVal == JFileChooser.APPROVE_OPTION) {
                            tfTestboxLocation.setText(chooser.getSelectedFile().getAbsolutePath());
                        }
                    }
                }
            }
        );
        bWhoisBrowse = new JButton("Browse");
        bWhoisBrowse.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bWhoisBrowse) {
                        JFileChooser chooser = new JFileChooser();
                        File f = new File(tfWhoisServersFilename.getText());

                        chooser.setSelectedFile(f);
                        int returnVal = chooser.showOpenDialog(self);

                        if (returnVal == JFileChooser.APPROVE_OPTION) {
                            tfWhoisServersFilename.setText(chooser.getSelectedFile().getAbsolutePath());
                        }
                    }
                }
            }
        );
        bOK = new JButton("OK");
        bOK.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bOK) {
                        update();
                        updatePreferences();
                        setVisible(false);
                    }
                }
            }
        );
                
        bCancel = new JButton("Cancel");
        bCancel.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (e.getSource() == bCancel) {
                        setVisible(false);
                    }
                }
            }
        );


        Container c = getContentPane();
        c.setLayout(new GridBagLayout());

        // Create a constraints object, and specify some default values
        GridBagConstraints g = new GridBagConstraints();

        g.fill = GridBagConstraints.BOTH;   // components grow in both dimensions
        g.insets = new Insets(5, 5, 5, 5); // 5-pixel margins on all sides

        JTabbedPane tabs = new JTabbedPane();

        JPanel dbPanel = new JPanel();
        dbPanel.setLayout(new GridBagLayout());

        int line = -1;

        JLabel lDBType = new JLabel("Database type: ");

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        dbPanel.add(lDBType, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        dbPanel.add(rbDBTypeMySQL, g);

        g.gridx = 1;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        dbPanel.add(rbDBTypeUnknown, g);

        JLabel lDBLoc = new JLabel("Database location: ");

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        dbPanel.add(lDBLoc, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        dbPanel.add(tfDBLocation, g);

        JLabel lDBName = new JLabel("Database name: ");

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        dbPanel.add(lDBName, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        dbPanel.add(tfDBName, g);

        g.gridx = 2;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        dbPanel.add(cbDBBackup, g);

        JLabel lDBUser = new JLabel("Database user: ");

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        dbPanel.add(lDBUser, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        dbPanel.add(tfDBUser, g);

        JLabel lDBPassword = new JLabel("Database password: ");

        g.gridx = 0; g.gridy = ++line; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 0.0;
        dbPanel.add(lDBPassword, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        dbPanel.add(tfDBPassword, g);

        g.gridx = 1; g.gridy = ++line; g.gridwidth = 2; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 0.0;
        dbPanel.add(cbDBStorePassword, g);

        tabs.addTab("Database settings", dbPanel);

        JPanel fileLocationsPanel = new JPanel();
        fileLocationsPanel.setLayout(new GridBagLayout());

        line = -1;

        JLabel lASCache = new JLabel("Autonomous System Cache: ");

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        fileLocationsPanel.add(lASCache, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        fileLocationsPanel.add(tfASCacheLocation, g);

        g.gridx = 2;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        fileLocationsPanel.add(bASCacheBrowse, g);

        JLabel lTestbox = new JLabel("Testbox data file: ");

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        fileLocationsPanel.add(lTestbox, g);

        g.gridx = 1;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        fileLocationsPanel.add(tfTestboxLocation, g);

        g.gridx = 2;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        fileLocationsPanel.add(bTestboxBrowse, g);

        g.gridx = 0;
        g.gridy = ++line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        fileLocationsPanel.add(new JLabel("Whois server data file: "), g);

        g.gridx = 1; g.gridy = line; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 0.0;
        fileLocationsPanel.add(tfWhoisServersFilename, g);

        g.gridx = 2;
        g.gridy = line;
        g.gridwidth = 1;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        fileLocationsPanel.add(bWhoisBrowse, g);

        tabs.add("File locations", fileLocationsPanel);

        JPanel logPanel = new JPanel();
        logPanel.setLayout(new GridBagLayout());
        
        line = -1;

        JLabel lLogLines = new JLabel("Max lines in log window:");
        g.gridx = 0; g.gridy = ++line; g.gridwidth = 1;
        g.gridheight = 1; g.weightx = 1.0; g.weighty = 0.0;
        logPanel.add(lLogLines, g);

        g.gridx = 1; g.gridy = line; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 0.0;
        logPanel.add(spLogLines, g);

        g.gridx = 1; g.gridy = ++line; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 0.0;
        logPanel.add(cbLogToFile, g);

        g.gridx = 0; g.gridy = ++line; g.gridwidth = 1;
        g.gridheight = 1; g.weightx = 1.0; g.weighty = 0.0;
        logPanel.add(new JLabel("Logfile location"), g);

        g.gridx = 1; g.gridy = line; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 0.0;
        logPanel.add(tfLogFilename, g);

        g.gridx = 2; g.gridy = line; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 0.0;
        logPanel.add(bLogFilenameBrowse, g);

        g.gridx = 1; g.gridy = ++line; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 0.0;
        logPanel.add(cbLogUseRotation, g);

        g.gridx = 2; g.gridy = line; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 0.0;
        logPanel.add(spLogRotation, g);

        g.gridx = 1; g.gridy = ++line; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 0.0;
        logPanel.add(cbLogAppend, g);

        g.gridx = 0; g.gridy = ++line; g.gridwidth = 1;
        g.gridheight = 1; g.weightx = 1.0; g.weighty = 0.0;
        logPanel.add(new JLabel("Logfile size limit"), g);

        g.gridx = 1; g.gridy = line; g.gridwidth = 1; g.gridheight = 1;
        g.weightx = 1.0; g.weighty = 0.0;
        logPanel.add(spLogLimit, g);
        
        tabs.add("Logging", logPanel);

        JPanel otherPanel = new JPanel();
        otherPanel.setLayout(new GridBagLayout());

        line = -1;

        g.gridx = 1;
        g.gridy = ++line;
        g.gridwidth = 2;
        g.gridheight = 1;
        g.weightx = 1.0;
        g.weighty = 0.0;
        otherPanel.add(cbIncludeTraces, g);

        tabs.add("Miscellaneous", otherPanel);

        line = -1;

        g.gridx = 0; g.gridy = ++line; g.gridwidth = 2; g.gridheight = 1; 
        g.weightx = 1.0; g.weighty = 0.0; 
        c.add(tabs, g);

        g.gridx = 0; g.gridy = ++line; g.gridwidth = 1; g.gridheight = 1; 
        g.weightx = 1.0; g.weighty = 0.0;
        c.add(bOK, g);

        g.gridx = 1; g.gridy = line; g.gridwidth = 1; g.gridheight = 1; 
        g.weightx = 1.0; g.weighty = 0.0;
        c.add(bCancel, g);

        //	      c.setLocation(0,0);
        // c.setSize(500, 500);
        c.invalidate();
        c.validate();
        pack();

        retrievePreferences();
    }

    private void update() {
        /*
        DBURL = "jdbc:";
        if (rbDBTypeMySQL.isSelected()) DBURL = DBURL + "mysql:";
        DBURL = DBURL + "//" + tfDBLocation.getText() + "/" + tfDBName.getText() +
                "?user=" + tfDBUser.getText() + "&password=" + tfDBPassword.getText();
        */

        boolean bTemp;
        int iTemp;
        String sTemp;

        if (rbDBTypeMySQL.isSelected()) DBType = "mysql";
        else DBType = "";
        DBHost = tfDBLocation.getText();
        DBDatabase = tfDBName.getText();
        DBUser = tfDBUser.getText();
        DBPassword = tfDBPassword.getText();
        DBStorePassword = cbDBStorePassword.isSelected();

        ASCacheLocation = tfASCacheLocation.getText();
        testboxLocation = tfTestboxLocation.getText();
        whoisServersFilename = tfWhoisServersFilename.getText();
        useBackup = cbDBBackup.isSelected();

        // Log options
        maxLogLines = ((Number) spLogLines.getValue()).intValue();
        bTemp = cbLogToFile.isSelected();
        if (bTemp != logToFile) {
            logToFile = bTemp;
            logFileIsChanged = true;
        }
        sTemp = tfLogFilename.getText(); 
        if (!sTemp.equals(logFilename)) {
            logFilename = sTemp;
            logFileIsChanged = true;
        }
        iTemp = cbLogUseRotation.isSelected() ?
            ((Number) spLogRotation.getValue()).intValue()
            : 1;
        if (iTemp != logRotate) {
            logRotate = iTemp;
            logFileIsChanged = true;
        }
        iTemp =  ((Number) spLogLimit.getValue()).intValue();
        if (logLimit != iTemp) {
            logLimit = iTemp;
            logFileIsChanged = true;
        }
        bTemp = cbLogAppend.isSelected();
        if (logAppend != bTemp) {
            logAppend = bTemp;
            logFileIsChanged = true;
        }

        // Misc options
        includeTracesXML = cbIncludeTraces.isSelected();
    //	    logger.info("New options: "+DBURL+"; "+ASCacheLocation);
    }

    boolean logFileSettingsChanged() {
        boolean result = logFileIsChanged;
        logFileIsChanged = false;
        return result;
    }

    void retrievePreferences() {
        String defaultDir = System.getProperty("user.dir") + System.getProperty("file.separator");
        try {
            Preferences prefs = Preferences.userNodeForPackage(OptionsDialog.class);
            String dbType = prefs.get(PREFS_DB_TYPE, "MySQL");

            for (Enumeration e = bgDBType.getElements(); e.hasMoreElements();) {
                JRadioButton button = (JRadioButton) e.nextElement();

                button.setSelected(button.getText().equals(dbType));
            }
            tfDBLocation.setText(prefs.get(PREFS_DB_LOCATION, "tux.ict.tbm.tudelft.nl"));
            tfDBName.setText(prefs.get(PREFS_DB_NAME, "TTM"));
            cbDBBackup.setSelected(prefs.getBoolean(PREFS_DB_BACKUP, false));
            tfDBUser.setText(prefs.get(PREFS_DB_USER, ""));
            tfDBPassword.setText(prefs.get(PREFS_DB_PASSWORD, ""));
            cbDBStorePassword.setSelected(prefs.getBoolean(PREFS_DB_STORE_PASSWORD, false));
            tfASCacheLocation.setText(prefs.get(PREFS_AS_CACHE_LOCATION, defaultDir+"AS-cache.xml"));
            tfTestboxLocation.setText(prefs.get(PREFS_TESTBOX_LOCATION, defaultDir+"testboxes.txt"));
            tfWhoisServersFilename.setText(prefs.get(PREFS_WHOIS_SERVERS_FILENAME, defaultDir+"whois.xml"));
            // Log
            try {
                int maxLines = Integer.parseInt(prefs.get(PREFS_MAX_LOG_LINES, "10000"));
                spLogLines.setValue(new Integer(maxLines));
            } catch (Exception e) {
                spLogLines.setValue(new Integer(10000));
            }
            cbLogToFile.setSelected(prefs.getBoolean(PREFS_LOG_TO_FILE, false));
            tfLogFilename.setText(prefs.get(PREFS_LOG_FILENAME, defaultDir+"traces.log"));
            cbLogUseRotation.setSelected(prefs.getBoolean(PREFS_LOG_USE_ROTATE, false));
            try {
                int logRotate = Integer.parseInt(prefs.get(PREFS_LOG_ROTATE, "5"));
                spLogRotation.setValue(new Integer(logRotate));
            } catch (Exception e) {
                spLogRotation.setValue(new Integer(5));
            }
            cbLogAppend.setSelected(prefs.getBoolean(PREFS_LOG_APPEND, false));
            try {
                int logLimit = Integer.parseInt(prefs.get(PREFS_LOG_LIMIT, "1000000"));
                spLogLimit.setValue(new Integer(logLimit));
            } catch (Exception e) {
                spLogLimit.setValue(new Integer(1000000));
            }

            // Misc
            cbIncludeTraces.setSelected(prefs.getBoolean(PREFS_XML_INCLUDE_TRACES, false));
        } catch (AccessControlException e) {
            String dbType = "MySQL";

            for (Enumeration e1 = bgDBType.getElements(); e1.hasMoreElements();) {
                JRadioButton button = (JRadioButton) e1.nextElement();

                button.setSelected(button.getText().equals(dbType));
            }
            tfDBLocation.setText("130.161.216.187");
            tfDBName.setText("TTM");
            cbDBBackup.setSelected(false);
            tfDBUser.setText("");
            tfDBPassword.setText("");
            cbDBStorePassword.setSelected(false);
            tfASCacheLocation.setText(defaultDir+"AS-cache.xml");
            tfTestboxLocation.setText(defaultDir+"testboxes.txt");
            tfWhoisServersFilename.setText(defaultDir+"whois.xml");
            spLogLines.setValue(new Integer(10000));
            cbLogToFile.setSelected(false);
            tfLogFilename.setText(defaultDir+"traces.log");
            cbLogUseRotation.setSelected(false);
            spLogRotation.setValue(new Integer(5));
            cbLogAppend.setSelected(false);
            spLogLimit.setValue(new Integer(1000000));
            cbIncludeTraces.setSelected(false);
        }
        update();
    }

    void updatePreferences() {
        try {
            Preferences prefs = Preferences.userNodeForPackage(OptionsDialog.class);
            String dbType = "";

            for (Enumeration e = bgDBType.getElements(); e.hasMoreElements();) {
                JRadioButton button = (JRadioButton) e.nextElement();

                if (button.isSelected()) dbType = button.getText();
            }
            prefs.put(PREFS_DB_TYPE, dbType);
            prefs.put(PREFS_DB_LOCATION, tfDBLocation.getText());
            prefs.put(PREFS_DB_NAME, tfDBName.getText());
            prefs.putBoolean(PREFS_DB_BACKUP, cbDBBackup.isSelected());
            prefs.put(PREFS_DB_USER, tfDBUser.getText());
            if (cbDBStorePassword.isSelected()) {
                prefs.put(PREFS_DB_PASSWORD, tfDBPassword.getText());
            } else {
                prefs.put(PREFS_DB_PASSWORD, "");
            }
            prefs.putBoolean(PREFS_DB_STORE_PASSWORD, cbDBStorePassword.isSelected());
            prefs.put(PREFS_AS_CACHE_LOCATION, tfASCacheLocation.getText());
            prefs.put(PREFS_TESTBOX_LOCATION, tfTestboxLocation.getText());
            prefs.put(PREFS_WHOIS_SERVERS_FILENAME, tfWhoisServersFilename.getText());

            prefs.put(PREFS_MAX_LOG_LINES, "" + ((Number) spLogLines.getValue()).intValue());
            prefs.putBoolean(PREFS_LOG_TO_FILE, cbLogToFile.isSelected());
            prefs.put(PREFS_LOG_FILENAME, tfLogFilename.getText());
            prefs.putBoolean(PREFS_LOG_USE_ROTATE, cbLogUseRotation.isSelected());
            prefs.put(PREFS_LOG_ROTATE, ""  + ((Number) spLogRotation.getValue()).intValue());
            prefs.putBoolean(PREFS_LOG_APPEND, cbLogAppend.isSelected());
            prefs.put(PREFS_LOG_LIMIT, "" + ((Number) spLogLimit.getValue()).intValue());

            prefs.putBoolean(PREFS_XML_INCLUDE_TRACES, cbIncludeTraces.isSelected());
        } catch (AccessControlException e) {
        }
    }
}

class BrowseListener implements ActionListener {
    JTextField tfPath;
    JButton button;
    JDialog dialog;

    BrowseListener(JDialog dialog, JButton button, JTextField path) {
        this.dialog = dialog;
        this.tfPath = path;
        this.button = button;
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == button) {
            JFileChooser chooser = new JFileChooser();
            File f = new File(tfPath.getText());
            chooser.setSelectedFile(f);
            int returnVal = chooser.showOpenDialog(dialog);
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                tfPath.setText(chooser.getSelectedFile().getAbsolutePath());
            }
        }
    }
}
