/**
 * Title:	Traces
 * Copyright:	Copyright (c) Jan-Pascal van Best
 * Email:	    janpascal@vanbest.org
 * @author	    Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.traces;


import java.io.*;
import java.lang.*;
import java.lang.reflect.*;
import java.util.*;
import java.util.logging.*;
import javax.swing.*;
import javax.swing.tree.*;


public class ReflectTreeNode implements TreeNode {
    String name;
    Object obj;
    Class cl;
    ReflectTreeNode parent;
    Vector kids = null;
    private static Logger logger = Logger.getLogger("nl.tudelft.tbm.traces");
       
    public ReflectTreeNode(String name, Object obj, ReflectTreeNode parent) {
        this.name = name;
        this.obj = obj;
        this.parent = parent;
        cl = obj.getClass();
    }

    public Enumeration children() {
        if (kids==null) makeKids();
        return kids.elements();
    }

    public boolean getAllowsChildren() {
        return  (obj!=null && !cl.isPrimitive() && !(obj instanceof String));
    }

    public TreeNode getChildAt(int childIndex) {
        if (kids==null) makeKids();
        return (TreeNode) kids.get(childIndex);
    }

    public int getChildCount() {
        if (kids==null) makeKids();
        return kids.size();
    }

    public int getIndex(TreeNode node) {
        if (kids==null) makeKids();
        return kids.indexOf(node);
    }

    public TreeNode getParent() {
        return parent;
    }

    public boolean isLeaf() {
        boolean result = (obj==null || cl.isPrimitive() || obj instanceof String);
        if(!result) {
            if (kids==null) makeKids();
            result = (kids.size()==0);
        }
        return result;
    }

    public String toString() {
        return name;
    }


    private void makeKids() {
        kids = new Vector();
        Class superclass = cl;
        while(superclass!=null) {
//            logger.info(indent+"("+superclass.getName()+")");
            kids.add(new DefaultMutableTreeNode("("+superclass.getName()+")"));
            Field[] fields = superclass.getDeclaredFields();
            AccessibleObject.setAccessible(fields, true);
            for(int i=0; i<fields.length; i++) {
                Field field = fields[i];
                try {
                    Class fieldType = field.getType();
                    Object value = field.get(obj);
                    if (value==null || fieldType.isPrimitive() || value instanceof String) {
                        kids.add(new DefaultMutableTreeNode(field.getName()+" = "+value));
                    } else {
//                        logger.info(indent+"(field: "+field.getName()+")");
                        kids.add(new ReflectTreeNode(field.getName(), value, this));
                    }
                } catch (IllegalAccessException e) {
                    logger.info("!! Error accessing field "+field.getName());
                }
            }
            superclass = superclass.getSuperclass();
        }
    }
/*
    static Set knownObjects;
    static void showFields(Object obj) {
        knownObjects = new HashSet();
        showFieldsImpl(obj, "");
    }

    private static void showFieldsImpl(Object obj, String indent) {
        knownObjects.add(obj);
        Class cl = obj.getClass();
        while(cl!=null) {
            logger.info(indent+"("+cl.getName()+")");
            Field[] fields = cl.getDeclaredFields();
            AccessibleObject.setAccessible(fields, true);
            for(int i=0; i<fields.length; i++) {
                Field field = fields[i];
                try {
                    Class fieldType = field.getType();
                    Object value = field.get(obj);
                    if (value==null || fieldType.isPrimitive() || value instanceof String || knownObjects.contains(value)) {
                        logger.info(indent+field.getName()+" = "+value); // .toString()
                    } else {
                        logger.info(indent+"(field: "+field.getName()+")");
                        showFieldsImpl(value, indent + "    ");
                    }
                } catch (IllegalAccessException e) {
                    logger.info(indent+"!! Error accessing field "+field.getName());
                }
            }
            cl = cl.getSuperclass();
        }
    }
*/

}
/*
private Class FieldHolder {
    public String name;
    public Class type;
    public Object obj;

    public FieldHolder(String name, Class type, Object obj) {
        this.name = name;
        this.type = type;
        this.obj = obj;
    }
}
*/


