/**
 * Title:       Traces
 * Description: An analysis tool for traceroutes
 * Copyright:   Copyright (c) Jan-Pascal van Best
 * Email:	    janb@tbm.tudelft.nl
 * @author 	    Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.traces;


import java.util.*;
import java.util.logging.*;
import java.io.*;
import java.lang.*;
import java.net.*;


public class Route implements Serializable {
    private static Logger logger = Logger.getLogger("nl.tudelft.tbm.traces");
    public IPRange range;
    public String description;
    public int ASNumber;

    public Route(IPRange range, String description, int ASNumber) {
        this.range = range;
        this.description = description;
        this.ASNumber = ASNumber;
    }

    public Route(InetAddress ip) throws IOException, InterruptedException {
        boolean gotit = false;

        try {
            readRoute("whois.ripe.net", ip);
            gotit = (range != null);
        } catch (Exception e) {
        }

        if (!gotit) {
            logger.info("Looking at ARIN for " + ip.getHostAddress());
            readRoute("rr.arin.net", ip);
        }
    }
    
    public boolean containsIP(InetAddress ip) {
        return range == null ? false : range.contains(ip);
    }

    public String toString() {
        return range == null
            ? "?.?.?.?/? (" + description + ", origin: AS" + ASNumber + ")"
            : range.toString() + "(" + description + ", origin: AS" + ASNumber + ")";
    }

    int readRoute(String whois, InetAddress ip) throws IOException, InterruptedException {
        int number = 0;
        int result = -42;

        range = null;
        ASNumber = -42;
        description = "unknown";
        
        String[] cmdarray = new String[3];

        cmdarray[0] = "whois"; 
        cmdarray[1] = "-h" + whois; 
        cmdarray[2] = ip.getHostAddress(); 
        Process process = Runtime.getRuntime().exec(cmdarray);
        BufferedReader output = new BufferedReader(new InputStreamReader(process.getInputStream()));
        boolean finished = false;

        for (int i = 0; i < 600 && !finished; i++) {
            Thread.sleep(100);
            if (output.ready()) {
                while (output.ready()) {
                    String line = output.readLine();

                    //                    logger.info(line);
                    if (line != null && line.startsWith("route:")) {
                        range = new IPRange(line.substring(7).trim());
                    }
                    if (line != null && line.startsWith("descr:")) {
                        description = line.substring(6).trim();
                    }
                    if (line != null && line.startsWith("origin:")) {
                        ASNumber = Integer.parseInt(line.substring(7).trim().substring(2));
                    }
                }
            } else {
                //                System.out.print(".");
            }
            try {
                result = process.exitValue();
                finished = true;
            } catch (IllegalThreadStateException e) {
            }
        }
        return number;
    }
}

