/**
 * Title:       Traces
 * Description: An analysis tool for traceroutes
 * Copyright:   Copyright (c) Jan-Pascal van Best
 * Email:	    janb@tbm.tudelft.nl
 * @author 	    Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.traces;


import java.util.*;
import java.io.*;
import java.lang.*;
import java.net.*;
import javax.swing.*;


public class RouteHash implements Serializable {
    Hashtable routes;

    public RouteHash() {
        routes = new Hashtable();
    }

    public Route getRoute(InetAddress ip) throws IOException, InterruptedException {
        Route route = null;

        if (routes.containsKey(ip)) {
            route = (Route) routes.get(ip);
            //            logger.info("Route for ip "+ip.getHostAddress()+" cached: "+route.description);
        } else {
            boolean found = false;

            for (Enumeration e = routes.elements(); e.hasMoreElements() && !found;) {
                route = (Route) e.nextElement();
                found = route.containsIP(ip);
            }
            if (found) {
                routes.put(ip, route);
                //                logger.info("ip "+ip.getHostAddress()+" already in "+route.description);
            } else {        
                route = new Route(ip);
                logger.info("got route for ip " + ip.getHostAddress() + ": " + route.description);
                routes.put(ip, route);
            }
        }
        return route;
    }

    public String toString() {
        String s = "";

        for (Enumeration e = routes.elements(); e.hasMoreElements();) {
            Route route = (Route) e.nextElement();

            s = s + route.toString() + "\n";
        }
        return s;
    }
}

