/**
 * Title:	 Traces
 * Description:  An analysis tool for traceroutes
 * Copyright:	 Copyright (c) Jan-Pascal van Best
 * Email:	    janb@sepa.tudelft.nl
 * @author	    Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.traces;


import java.util.*;
import java.io.*;
import java.lang.*;
import java.net.*;
import nl.tudelft.tbm.graph.*;


public class Router extends Host implements Serializable {
    public Set interfaces; // contains InetAddresses

    public Router() {
        super();
        interfaces = new HashSet();
    }

    public Router(InetAddress ip, String hostName, Set interfaces) {
        super(ip, hostName);
        this.interfaces = interfaces;
        interfaces.add(ip);
    }

    public Router(InetAddress ip) {
        super(ip, null);
        this.interfaces = new HashSet();
        interfaces.add(ip);
    }

    public Router(String ipString, String hostName) {
        super(ipString, hostName);
        interfaces = new HashSet();
        addInterface(getIp());
    }

    public Router(Host host) {
        super(host);
        interfaces = new HashSet();
        addInterface(getIp());
    }

    public void addInterface(InetAddress interf) {
        interfaces.add(interf);
    }

    public void addInterfaces(Set interfaces) {
        interfaces.addAll(interfaces);
    }

    public void mergeHost(Host host) {
        addDates(host.getDates());
        addInterface(host.getIp());
    }

    public void mergeRouter(Router router) {
        addDates(router.getDates());
        addInterfaces(router.interfaces);
    }

    public boolean containsInterface(InetAddress interf) {
        return interfaces.contains(interf);
    }

    public List getPropertyNames() {
        String[] strings = {"interfaces"};
        List result = super.getPropertyNames();

        result.addAll(Arrays.asList(strings));
        return result;
    }

    public String getInterfacesString() {
        StringBuffer s = new StringBuffer();
        Iterator i = interfaces.iterator();

        if (i.hasNext()) s.append(((InetAddress) i.next()).getHostAddress());
        while (i.hasNext()) {
            s.append(" ");
            s.append(((InetAddress) i.next()).getHostAddress());
        }
        return s.toString();
    }

    public String getProperty(String name) {
        String result;

        if (name.equals("interfaces")) {
            result = getInterfacesString();
        } else
            result = super.getProperty(name);
        return result;
    }
}

