/**
 * Title:	Traces
 * Description: An analysis tool for traceroutes
 * Copyright:	Copyright (c) Jan-Pascal van Best
 * Email:	janb@tbm.tudelft.nl
 * Author:	Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.traces;


import java.io.*;
import java.lang.*;
import java.net.*;
import java.sql.*;
import java.util.*;
import java.util.logging.Logger;

public class TTMData {
    String recordsTable = "Records";
    String routesTable = "Routes";
    String delaysTable = "Delays";

    boolean useDNS = true;
    
    Connection C = null;
    private static Logger logger = Logger.getLogger("nl.tudelft.tbm.traces");

    public TTMData(String URL) throws Exception {
        // The newInstance() call is a work around for some
        // broken Java implementations			   
        Class.forName("org.gjt.mm.mysql.Driver").newInstance(); 		
        C = DriverManager.getConnection(URL);
    }

    public void setTables(String records, String routes, String delays) {
        recordsTable = records;
        routesTable = routes;
        delaysTable = delays;
    }

    public void close() throws SQLException {
        C.close();
    }

    public void setUseDNS(boolean useDNS) {
        this.useDNS = useDNS;
    }

    public java.util.Date getLastEntry() throws SQLException {
        Statement S = C.createStatement();
        String Q = "select max(tend) from " + recordsTable;
		logger.info(Q);
        ResultSet R = S.executeQuery(Q);
		logger.info("Getting result from resultset");
        int column = R.findColumn("max(tend)");
        java.util.Date date;
        if (R.first()) { // returns true if we're on a valid row
            date = new java.util.Date(R.getInt(column)*(long)1000);
        } else {
            date = new java.util.Date(0);
            logger.info("Unable to read date from database");
        }
		logger.info("Closing connection");
        R.close();
        S.close();
        return date;
    }
    
    public static String resultSetToString(ResultSet R) throws SQLException {
        String s = "";
	
        ResultSetMetaData M = R.getMetaData();
        int numberOfColumns = M.getColumnCount();

        for (int i = 1; i <= numberOfColumns; i++) {
            s = s + "\"" + M.getColumnName(i) + "\"";
            if (i != numberOfColumns) s = s + ",";
        }
        s = s + "\n";
        while (R.next()) {
            for (int i = 1; i <= numberOfColumns; i++) {
                String value = R.getString(i);

                s = s + "\"" + value + "\"";
                if (i != numberOfColumns) s = s + ",";
            }
            s = s + "\n";
        }
        return s;
    }
/*
    public Trace getTrace(int src, int dst, java.util.Date when) {
        Trace trace = null;

        logger.info("Reading trace from " + src + " to " + dst + " at " + when);
        try {
            Statement S = C.createStatement();

            // Should make ony query with JOIN or something like that,
            // to also look in the Routes table
            // This would look like
            // "select len,route from Records,Routes where Records.routeid=Routes.id and ....."
            //	      String Q =  "select routeid from " + recordsTable +
            String Q = "select routeid,src,dst from " + recordsTable +
                " where tstart<=" + when.getTime() / 1000 + " and tend>=" + when.getTime() / 1000 +
                " and src=" + src + " and dst=" + dst;

            logger.info("Executing first query:");
            logger.info(Q);

            ResultSet R = S.executeQuery(Q);

            int column = R.findColumn("routeid");

            // We're only interested in one trace here
            if (R.first()) { // returns true if we're on a valid row
                int routeid = R.getInt(column);

                trace = getTraceOfRoute(src, dst, routeid);
                if (trace != null) {
                    trace.date = when;
                    logger.info("Trace read.");
                } else {
                    logger.info("Trace not found.");
                }
            }
        } catch (SQLException E) {
            logger.info("SQLException: " + E.getMessage());
            logger.info("SQLState:	   " + E.getSQLState());
            logger.info("VendorError:  " + E.getErrorCode());
            E.printStackTrace();
        }
        return trace;
    }
*/
    String datesQueryString(Set dates, int interval) {
        StringBuffer Q = new StringBuffer();
        boolean first = true;

        for (Iterator i = dates.iterator(); i.hasNext();) {
            java.util.Date when = (java.util.Date) i.next();

            if (first)
                Q.append("    (");
            else
                Q.append("\n  or ");
            Q.append("(tstart<=" + ((when.getTime() / 1000) + (interval/2)) + " and tend>=" + (when.getTime()/1000 - (interval/2)) + ")");
            first = false;
        }
        Q.append(")");
        return Q.toString();
    }


    String numbersQueryString(String column, Set numbers) {
        StringBuffer Q = new StringBuffer();
        boolean first = true;

        for (Iterator i = numbers.iterator(); i.hasNext();) {
            int box = ((Integer) i.next()).intValue();

            if (first)
                Q.append("(");
            else
                Q.append(" or ");
            Q.append("(" + column + "=" + box + ")");
            first = false;
        }
        Q.append(")");
        return Q.toString();
    }
/*
    public Vector getPartialTracesSnapshot(Set from, Set to, Set dates) {
        Vector traces = new Vector();

        logger.info("Reading partial snapshot with testbox sources in " + from.toString() + " and destination " + to.toString() + " at " + dates.toString());
        try {
            Statement S = C.createStatement();

            String Q = "select tstart,tend,src,dst,routeid from " + recordsTable +
                " where \n" + datesQueryString(dates) +
                "\nand " + numbersQueryString("src", from) + "\nand " + numbersQueryString("dst", to);

            logger.info("Executing first query:");
            logger.info(Q);
            ResultSet R = S.executeQuery(Q);

            logger.info("Finished first query");

            int column = R.findColumn("routeid");
            int srcColumn = R.findColumn("src");
            int dstColumn = R.findColumn("dst");
            int tstartColumn = R.findColumn("tstart");
            int tendColumn = R.findColumn("tend");

            R.beforeFirst();
            while (R.next()) {
                int routeid = R.getInt(column);
                Trace trace = getTraceOfRoute(R.getInt(srcColumn), R.getInt(dstColumn), routeid);

                for (Iterator i = dates.iterator(); i.hasNext();) {
                    java.util.Date when = (java.util.Date) i.next();

                    if (R.getLong(tstartColumn) <= when.getTime() / 1000 && R.getLong(tendColumn) >= when.getTime() / 1000) {
                        Trace newbie = new Trace(trace);

                        newbie.setDate(when);
                        traces.add(newbie);
                    }
                }
            }
        } catch (SQLException E) {
            logger.info("SQLException: " + E.getMessage());
            logger.info("SQLState:	   " + E.getSQLState());
            logger.info("VendorError:  " + E.getErrorCode());
            E.printStackTrace();
        }
        return traces;
    }
*/

    public Vector getPartialTracesSnapshot(Set from, Set to, Set dates, int interval) {
        Vector traces = new Vector();
		if (from.isEmpty() || to.isEmpty() || dates.isEmpty()) {
			logger.warning("Empty query!");
			return traces;
		}
        Statement S = null;
        ResultSet R = null;

        logger.info("Reading partial snapshot with testbox sources in " + from.toString() + " and destination " + to.toString() + " at " + dates.toString());
        try {
            S = C.createStatement();

            String Q = "select tstart,tend,src,dst,len,route\n"+
                "from " + recordsTable + "," + routesTable + "\n" +
                "where " + datesQueryString(dates, interval) + "\n" +
                "and " + numbersQueryString("src", from) + "\n" + 
                "and " + numbersQueryString("dst", to) + "\n" +
                "and ("+recordsTable+".routeid = "+routesTable+".id)\n";

            logger.info("Executing query:");
            logger.info(Q);
            R = S.executeQuery(Q);

            logger.info("Finished query, processing..");

            int srcColumn = R.findColumn("src");
            int dstColumn = R.findColumn("dst");
            int tstartColumn = R.findColumn("tstart");
            int tendColumn = R.findColumn("tend");
            int lenColumn = R.findColumn("len");
            int routeColumn = R.findColumn("route");

            R.beforeFirst();
            while (R.next()) {
                Vector hops = routeStringToHops(R.getInt(lenColumn), R.getString(routeColumn));
                Host fromHost = TestBox.getTestBox(R.getInt(srcColumn));
                Host toHost = TestBox.getTestBox(R.getInt(dstColumn));
                hops.insertElementAt(fromHost, 0);
                Trace trace = new Trace(fromHost, toHost, Trace.MAX_HOPS_UNKNOWN, Trace.PACKET_SIZE_UNKNOWN, hops, Trace.TIME_UNKNOWN);

                long tstart = R.getLong(tstartColumn);
                long tend = R.getLong(tendColumn);

                for (Iterator i = dates.iterator(); i.hasNext();) {
                    java.util.Date when = (java.util.Date) i.next();

                    if (tstart <= (when.getTime()/1000 + (interval/2)) && tend >= (when.getTime()/1000 - (interval/2))) {
                        Trace newbie = new Trace(trace);
                        newbie.setDate(when);
                        traces.add(newbie);
                    }
                }
            }
        } catch (SQLException E) {
            logger.info("SQLException: " + E.getMessage());
            logger.info("SQLState:	   " + E.getSQLState());
            logger.info("VendorError:  " + E.getErrorCode());
            E.printStackTrace();
        } finally {
            if (R != null) try { R.close(); } catch (SQLException e2) {}
            if (S != null) try { S.close(); } catch (SQLException e3) {}
        }
        logger.info("Finished");
        return traces;
    }

/*
    public Vector getTracesSnapshot(java.util.Date when) {
        Vector traces = new Vector();

        logger.info("Reading snapshot at " + when);
        try {
            Statement S = C.createStatement();

            String Q = "select src,dst,routeid from " + recordsTable +
                " where tstart<=" + when.getTime() / 1000 + " and tend>=" + when.getTime() / 1000;

            logger.info("Executing first query (this could take a while):");
            logger.info(Q);
            ResultSet R = S.executeQuery(Q);

            logger.info("Finished first query");

            int column = R.findColumn("routeid");
            int srcColumn = R.findColumn("src");
            int dstColumn = R.findColumn("dst");

            R.beforeFirst();
            logger.info("Getting traces");
            int count = 0;

            while (R.next()) {
                int routeid = R.getInt(column);
                Trace trace = getTraceOfRoute(R.getInt(srcColumn), R.getInt(dstColumn), routeid);

                trace.date = when;
                traces.add(trace);
                logger.infoChar(".");
                count++;
                if (count % 80 == 0) logger.info("");
            }
        } catch (SQLException E) {
            logger.info("SQLException: " + E.getMessage());
            logger.info("SQLState:	   " + E.getSQLState());
            logger.info("VendorError:  " + E.getErrorCode());
            E.printStackTrace();
        }
        return traces;
    }
*/

/*
    public Vector getPartialTracesSnapshot(int first, int last, java.util.Date when) {
        Vector traces = new Vector();

        logger.info("Reading partial snapshot with testbox numbers between " + first + " and " + last + " at " + when);
        try {
            Statement S = C.createStatement();

            String Q = "select src,dst,routeid from " + recordsTable +
                " where tstart<=" + when.getTime() / 1000 + " and tend>=" + when.getTime() / 1000 +
                " and src>=" + first + " and src<=" + last + " and dst>=" + first + " and dst<=" + last;

            logger.info("Executing first query:");
            logger.info(Q);
            ResultSet R = S.executeQuery(Q);

            logger.info("Finished first query");

            int column = R.findColumn("routeid");
            int srcColumn = R.findColumn("src");
            int dstColumn = R.findColumn("dst");

            R.beforeFirst();
            while (R.next()) {
                int routeid = R.getInt(column);
                Trace trace = getTraceOfRoute(R.getInt(srcColumn), R.getInt(dstColumn), routeid);

                trace.date = when;
                traces.add(trace);
            }
        } catch (SQLException E) {
            logger.info("SQLException: " + E.getMessage());
            logger.info("SQLState:	   " + E.getSQLState());
            logger.info("VendorError:  " + E.getErrorCode());
            E.printStackTrace();
        }
        return traces;
    }
*/

/*
    Trace getTraceOfRoute(int src, int dst, int routeid) {
        Trace trace = null;

        logger.info("Get trace for route " + routeid + "...");

        try {
            Statement S = C.createStatement();
            String Q = "select len,route from " + routesTable +
                " where id=" + routeid;
            ResultSet R = S.executeQuery(Q);

            //	      logger.info(resultSetToString(R));

            R.absolute(1); // id is primary key, so there's exactly one route here

            int lenColumn = R.findColumn("len");
            int routeColumn = R.findColumn("route");
            Vector hops = routeStringToHops(R.getInt(lenColumn), R.getString(routeColumn));
            Host from = TestBox.getTestBox(src);
            Host to = TestBox.getTestBox(dst);

            hops.insertElementAt(from, 0);
            trace = new Trace(from, to, Trace.MAX_HOPS_UNKNOWN, Trace.PACKET_SIZE_UNKNOWN, hops, Trace.TIME_UNKNOWN);
            logger.info(" Got it");
        } catch (SQLException E) {
            logger.info("SQLException: " + E.getMessage());
            logger.info("SQLState:	   " + E.getSQLState());
            logger.info("VendorError:  " + E.getErrorCode());
            E.printStackTrace();
        }
        return trace;
    }
*/
    public Vector routeStringToHops(int len, String route) {
        Vector hops = new Vector();
        StringTokenizer st = new StringTokenizer(route);

        if (len != st.countTokens()) logger.info("route warning: len unequal to number of addresses in route");
        while (st.hasMoreTokens()) {
            long ipInt = 0;

            try {
                ipInt = Long.parseLong(st.nextToken());
            } catch (Exception e) {
                e.printStackTrace();
            }
            String ipString = ((ipInt & 255 * 256 * 256 * 256) >> 24) + "." + ((ipInt & 255 * 256 * 256) >> 16) + "." + ((ipInt & 255 * 256) >> 8) + "." + (ipInt & 255);
            Hop hop;
            if (useDNS) {
                hop = new Hop(ipString, null, Hop.MS_UNKNOWN);
            } else {
                hop = new Hop(ipString, ipString, Hop.MS_UNKNOWN);
            }
            hops.add(hop);
        }
        return hops;
    }
/*
    public Vector getTrace(int src, int dst, Set dates) {
        Vector traces = new Vector();

        for (Iterator i = dates.iterator(); i.hasNext();) {
            java.util.Date when = (java.util.Date) i.next();
            Trace trace = getTrace(src, dst, when);

            if (trace != null) traces.add(trace);
        }
        return traces;
    }
*/

/* Bakcup for maintenance 2001/12/11 
    public Vector getTracesSnapshot(Set dates) {
        // select * from Records
        // where src=1 and dst=9 and (
        //  (tstart<=936208800 and tend>=936208800) or
        //  (tstart<=936381600 and tend>=936381600) )
        Vector traces = new Vector();

        try {
            Statement S = C.createStatement();

            StringBuffer Q = new StringBuffer();

            Q.append("select src,dst,tstart,tend,routeid from " + recordsTable + " where ( ");

            boolean first = true;

            for (Iterator i = dates.iterator(); i.hasNext();) {
                java.util.Date when = (java.util.Date) i.next();

                if (first)
                    Q.append("\n (");
                else
                    Q.append("\n or (");
                Q.append("tstart<=" + when.getTime() / 1000 + " and tend>=" + when.getTime() / 1000 + ")");
                first = false;
            }
            Q.append(" )");

            logger.info("Executing main query (this could take a while):");
            logger.info(Q.toString());
            ResultSet R = S.executeQuery(Q.toString());

            logger.info("Finished main query");

            int column = R.findColumn("routeid");
            int srcColumn = R.findColumn("src");
            int dstColumn = R.findColumn("dst");
            int tstartColumn = R.findColumn("tstart");
            int tendColumn = R.findColumn("tend");

            R.beforeFirst();
            while (R.next()) {
                int routeid = R.getInt(column);
                Trace trace = getTraceOfRoute(R.getInt(srcColumn), R.getInt(dstColumn), routeid);

                for (Iterator i = dates.iterator(); i.hasNext();) {
                    java.util.Date when = (java.util.Date) i.next();

                    if (R.getLong(tstartColumn) <= when.getTime() / 1000 && R.getLong(tendColumn) >= when.getTime() / 1000) {
                        Trace newbie = new Trace(trace);

                        newbie.setDate(when);
                        traces.add(newbie);
                    }
                }
                //		  trace.date = when;
                //		  traces.add(trace);
            }
        } catch (SQLException E) {
            logger.info("SQLException: " + E.getMessage());
            logger.info("SQLState:	   " + E.getSQLState());
            logger.info("VendorError:  " + E.getErrorCode());
            E.printStackTrace();
        }

        logger.info("Got all traces");
        return traces;
    }

*/
    public Vector getTracesSnapshot(Set dates, int interval) {
        // select * from Records
        // where src=1 and dst=9 and (
        //  (tstart<=936208800 and tend>=936208800) or
        //  (tstart<=936381600 and tend>=936381600) )
        Vector traces = new Vector();
		if (dates.isEmpty()) {
			logger.warning("No dates selected!");
			return traces;
		}
        Statement S = null;
        ResultSet R = null;

        try {
            S = C.createStatement();

            String Q = "select src,dst,tstart,tend,len,route\n" +
                "    from " + recordsTable + "," + routesTable + "\n" +
                "    where " + datesQueryString(dates, interval)+ "\n" +
                "        and ("+recordsTable+".routeid = "+routesTable+".id)\n";

/*
            boolean first = true;
            for (Iterator i = dates.iterator(); i.hasNext();) {
                java.util.Date when = (java.util.Date) i.next();

                if (first)
                    Q.append("\n        (");
                else
                    Q.append("\n     or (");
                Q.append("tstart<=" + when.getTime() / 1000 + " and tend>=" + when.getTime() / 1000 + ")");
                first = false;
            }
            Q.append("\n    )\n");
            Q.append("     and ("+recordsTable+".routeid = "+routesTable+".id)\n");
*/

            logger.info("Executing query (this could take a while):");
            logger.info(Q);
            R = S.executeQuery(Q);

            logger.info("Finished query, processing...");

            int srcColumn = R.findColumn("src");
            int dstColumn = R.findColumn("dst");
            int tstartColumn = R.findColumn("tstart");
            int tendColumn = R.findColumn("tend");
            int lenColumn = R.findColumn("len");
            int routeColumn = R.findColumn("route");

            R.beforeFirst();
            while (R.next()) {
                Trace trace = null;

                logger.info("reading trace..");
                int len = R.getInt(lenColumn);
                String route = R.getString(routeColumn);
                int src = R.getInt(srcColumn);
                int dst = R.getInt(dstColumn);
                long tstart = R.getLong(tstartColumn);
                long tend = R.getLong(tendColumn);

                logger.info("making trace..");
                Vector hops = routeStringToHops(len, route);
                Host from = TestBox.getTestBox(src);
                Host to = TestBox.getTestBox(dst);
                hops.insertElementAt(from, 0);
                trace = new Trace(from, to, Trace.MAX_HOPS_UNKNOWN, Trace.PACKET_SIZE_UNKNOWN, hops, Trace.TIME_UNKNOWN);

                logger.info("inserting...");
                for (Iterator i = dates.iterator(); i.hasNext();) {
                    java.util.Date when = (java.util.Date) i.next();
                    if (tstart <= (when.getTime()/1000 + (interval/2)) && tend >= (when.getTime()/1000 - (interval/2))) {
                        Trace newbie = new Trace(trace);
                        newbie.setDate(when);
                        traces.add(newbie);
                    }
                }
                logger.info("Finished trace");
                //		  trace.date = when;
                //		  traces.add(trace);
            }
        } catch (SQLException E) {
            logger.info("SQLException: " + E.getMessage());
            logger.info("SQLState:	   " + E.getSQLState());
            logger.info("VendorError:  " + E.getErrorCode());
            E.printStackTrace();
        } finally {
            if (R != null) try { R.close(); } catch (SQLException e2) {}
            if (S != null) try { S.close(); } catch (SQLException e3) {}
        }

        logger.info("Got all traces");
        return traces;
    }
/*
    public Vector getPartialTracesSnapshot(int first, int last, Set dates) {
        Vector traces = new Vector();

        for (Iterator i = dates.iterator(); i.hasNext();) {
            java.util.Date when = (java.util.Date) i.next();

            traces.addAll(getPartialTracesSnapshot(first, last, when));
        }
        return traces;
    }
*/    
    public Trace getDelayTrace(int src, int dst, java.util.Date when) {
        Trace trace = null;
        Statement S = null;
        ResultSet R = null;

        logger.info("Reading trace from " + src + " to " + dst + " at " + when);
        try {
            S = C.createStatement();
            String Q = "select len,route,packetdelay,packetsize,arrivaltime\n" +
                "from " + delaysTable + " D," + routesTable + " R\n" +
                "where src=" + src + " and dst=" + dst + " and D.routeid=R.id\n" +
                "order by abs(arrivaltime-" + when.getTime() / 1000 + ") ASC\n" +
                "limit 1";

            logger.info(Q);
            R = S.executeQuery(Q);

            //	      logger.info(resultSetToString(R));

            // There is just one row closest to this time
            if (R.first()) { // returns true if we're on a valid row
                logger.info("Getting query result");
                int lenColumn = R.findColumn("len");
                int routeColumn = R.findColumn("route");
                int delayColumn = R.findColumn("packetdelay");
                int sizeColumn = R.findColumn("packetsize");
                int dateColumn = R.findColumn("arrivaltime");
                double arrivaltime = R.getDouble(dateColumn);
                double offset = java.lang.Math.abs(arrivaltime - (((double) when.getTime()) / 1000));

                logger.info("Arrivaltime, when, ofset: " + arrivaltime + ", " + when.getTime() / 1000 + ", " + offset);
                if (offset > 600) { // POLICY 600 second == 5 minutes.
                    logger.info("No trace found at time " + when.toString());
                } else {
                    Vector hops = routeStringToHops(R.getInt(lenColumn), R.getString(routeColumn));
                    Host from = TestBox.getTestBox(src);
                    Host to = TestBox.getTestBox(dst);

                    hops.insertElementAt(from, 0);
                    trace = new Trace(from, to, Trace.MAX_HOPS_UNKNOWN, R.getInt(sizeColumn), hops, R.getDouble(delayColumn));
                }

                if (trace != null) {
                    trace.date = new java.util.Date((long) (R.getDouble(dateColumn) * 1000));
                    logger.info("Trace read.");
                } else {
                    logger.info("Trace not found.");
                }
            }
        } catch (SQLException E) {
            logger.info("SQLException: " + E.getMessage());
            logger.info("SQLState:	   " + E.getSQLState());
            logger.info("VendorError:  " + E.getErrorCode());
            E.printStackTrace();
        } finally {
            if (R != null) try { R.close(); } catch (SQLException e2) {}
            if (S != null) try { S.close(); } catch (SQLException e3) {}
        }
        return trace;	     
    }

    public Vector getDelaySnapshot(java.util.Date when) {
        Vector traces = new Vector();
        Statement S = null;
        ResultSet R = null;

        logger.info("Reading traces including delay data at " + when);
        try {
            S = C.createStatement();
            String Q = "select src,dst,min(abs(arrivaltime-" + when.getTime() / 1000 + ")) as offset\n" +
                "from " + delaysTable + "\n" +
                "group by src,dst\n";
            //	      logger.info(Q);
            R = S.executeQuery(Q);
            //	      logger.info(resultSetToString(R));


            R.beforeFirst();
            while (R.next()) { // returns true if we're on a valid row
                Trace trace = null;
                int src = R.getInt(1);
                int dst = R.getInt(2);
                String offset = R.getString(3);
                Statement S2 = C.createStatement();
                ResultSet R2 = null;

                //		  logger.info("src,dst,offset: "+src+", "+dst+", "+offset);
                if (Double.parseDouble(offset) > 600) { // POLICY 600 second == 5 minutes.
                    logger.info("No trace found at time " + when.toString() + " from " + src + " to " + dst);
                } else {
                    String Q2 = "select len,route,packetdelay,packetsize,arrivaltime\n" +
                        "from " + delaysTable + " D," + routesTable + " R\n" +
                        "where D.routeid=R.id and src=" + src + " and dst=" + dst + " and abs(abs(arrivaltime-" + when.getTime() / 1000 + ")-" + offset + ")<1\n" +
                        "order by abs(abs(arrivaltime-" + when.getTime() / 1000 + ")-" + offset + ") ASC\n";
                    //		      logger.info(Q2);
                    R2 = S2.executeQuery(Q2);
                    //		      logger.info(resultSetToString(R2));
                    R2.beforeFirst();
                    if (R2.next()) {
                        Vector hops = routeStringToHops(R2.getInt(1), R2.getString(2));
                        Host from = TestBox.getTestBox(src);
                        Host to = TestBox.getTestBox(dst);

                        hops.insertElementAt(from, 0);
                        trace = new Trace(from, to, Trace.MAX_HOPS_UNKNOWN, R2.getInt(4), hops, R2.getDouble(3));
                    }
                }
                if (trace != null) {
                    trace.date = new java.util.Date((long) (R2.getDouble(5) * 1000));
                    logger.info("Trace read: " + trace);
                    traces.add(trace);
                } else {
                    logger.info("Trace not found.");
                }
                if (R2!=null) R2.close();
                S2.close();
            }
        } catch (SQLException E) {
            logger.info("SQLException: " + E.getMessage());
            logger.info("SQLState:	   " + E.getSQLState());
            logger.info("VendorError:  " + E.getErrorCode());
            E.printStackTrace();
        } finally {
            if (R != null) try { R.close(); } catch (SQLException e2) {}
            if (S != null) try { S.close(); } catch (SQLException e3) {}
        }
        return traces;
    }


    public Vector getPartialDelaySnapshot(Set froms, Set tos, java.util.Date when) {
        Vector traces = new Vector();
        Statement S = null;
        ResultSet R = null;

        logger.info("Reading traces including delay data at " + when);
        try {
            S = C.createStatement();
            String Q = "select src,dst,min(abs(arrivaltime-" + when.getTime() / 1000 + ")) as offset\n" +
                "from " + delaysTable + "\n" +
                "where " + numbersQueryString("src", froms) + "\n" + 
                "and " + numbersQueryString("dst", tos) + "\n" +
                "group by src,dst\n";
            //	      logger.info(Q);
            R = S.executeQuery(Q);
            //	      logger.info(resultSetToString(R));


            R.beforeFirst();
            while (R.next()) { // returns true if we're on a valid row
                Trace trace = null;
                int src = R.getInt(1);
                int dst = R.getInt(2);
                String offset = R.getString(3);
                Statement S2 = C.createStatement();
                ResultSet R2 = null;

                //		  logger.info("src,dst,offset: "+src+", "+dst+", "+offset);
                if (Double.parseDouble(offset) > 600) { // POLICY 600 second == 5 minutes.
                    logger.info("No trace found at time " + when.toString() + " from " + src + " to " + dst);
                } else {
                    String Q2 = "select len,route,packetdelay,packetsize,arrivaltime\n" +
                        "from " + delaysTable + " D," + routesTable + " R\n" +
                        "where D.routeid=R.id and src=" + src + " and dst=" + dst + " and abs(abs(arrivaltime-" + when.getTime() / 1000 + ")-" + offset + ")<1\n" +
                        "order by abs(abs(arrivaltime-" + when.getTime() / 1000 + ")-" + offset + ") ASC\n";
                    //		      logger.info(Q2);
                    R2 = S2.executeQuery(Q2);
                    //		      logger.info(resultSetToString(R2));
                    R2.beforeFirst();
                    if (R2.next()) {
                        Vector hops = routeStringToHops(R2.getInt(1), R2.getString(2));
                        Host from = TestBox.getTestBox(src);
                        Host to = TestBox.getTestBox(dst);

                        hops.insertElementAt(from, 0);
                        trace = new Trace(from, to, Trace.MAX_HOPS_UNKNOWN, R2.getInt(4), hops, R2.getDouble(3));
                    }
                    R2.close();
                }
                S2.close();
            
                if (trace != null) {
                    trace.date = new java.util.Date((long) (R2.getDouble(5) * 1000));
                    logger.info("Trace read: " + trace);
                    traces.add(trace);
                } else {
                    logger.info("Trace not found.");
                }
            }
        } catch (SQLException E) {
            logger.info("SQLException: " + E.getMessage());
            logger.info("SQLState:	   " + E.getSQLState());
            logger.info("VendorError:  " + E.getErrorCode());
            E.printStackTrace();
        } finally {
            if (R != null) try { R.close(); } catch (SQLException e2) {}
            if (S != null) try { S.close(); } catch (SQLException e3) {}
        }
        return traces;
    }
}

