/**
 * Title:	Traces
 * Description: An analysis tool for traceroutes
 * Copyright:	Copyright (c) Jan-Pascal van Best
 * Email:	    janb@tbm.tudelft.nl
 * @author	    Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.traces;


import java.awt.*;
import java.io.*;
import java.lang.*;
import java.lang.reflect.*;
import java.net.*;
import java.sql.*;
import java.text.*;
import java.util.*;
import java.util.logging.*;
import javax.swing.*;
import java.awt.print.*;
import javax.print.*;
import javax.print.attribute.*;
import javax.print.attribute.standard.*;

import nl.tudelft.tbm.graph.*;

// import hep.io.root.*;
// import hep.io.root.interfaces.*;

import Jama.*;


public class Test {
    static final String AS_DATA_FILE = "E:\\temp\\AS-cache.xml";
    static final String ROUTES_DATA_FILE = "E:\\temp\\Route-cache.dat";
    static final String DATABASE_URL = "jdbc:mysql://130.161.216.187/TTM?user=janpascal&password=??????";
    private static Logger logger = Logger.getLogger("nl.tudelft.tbm.traces");

    /*
     double[][] vals = { {1.,0.,0.,1.,0.,0.},
     {1.,0.,0.,0.,0.,1.},
     {0.,1.,0.,0.,1.,0.},
     {0.,1.,1.,0.,0.,0.},
     {0.,0.,1.,0.,0.,1.},
     {0.,0.,0.,1.,1.,0.}};
     double[][] Bvals = { {10.}, {15.}, {15.}, {10.}, {15.}, {15.} };

     Matrix A = new Matrix(vals);
     Matrix B = new Matrix(Bvals);
     */

    public Test() {
        super();
    }

    static void test1() throws IOException, UnknownHostException, InterruptedException {
        ASHash hash;

        if (new File(AS_DATA_FILE).canRead()) {
            try {
                hash = ASHash.readASHash(AS_DATA_FILE);
                System.out.println("Read hash");
            } catch (Exception e) {
                System.out.println("Failed reading AS data");
                e.printStackTrace();
                hash = new ASHash();
            }
        } else {
            hash = new ASHash();
        }

        AutSystem AS;

        AS = hash.getAS(InetAddress.getByName("212.136.172.14"));
        System.out.println("212.136.172.14: " + AS.containsIP(InetAddress.getByName("212.136.172.14")));
        //	  System.out.println("212.136.160.93: "+hash.getAS(InetAddress.getByName("212.136.160.93")));
        //	  System.out.println("212.136.176.42: "+hash.getAS(InetAddress.getByName("212.136.176.42")));

        //got AS for ip 212.45.37.33: Internet Exchange Europe B.V.
        //got AS for ip 212.45.33.133: Internet Exchange Europe B.V.
        //got AS for ip 212.45.32.1: Internet Exchange Europe B.V.
        //got AS for ip 212.45.32.2: Internet Exchange Europe B.V.

        //got AS for ip 207.45.196.69:
        //got AS for ip 207.45.222.210:
        //got AS for ip 207.45.222.214:
        //got AS for ip 157.130.22.109:
    }

    static void test3() throws IOException, UnknownHostException, InterruptedException {

        /*
         RouteHash hash;
         if (new File(ROUTES_DATA_FILE).canRead()) {
         try {
         hash = Traces.readRouteHash();
         System.out.println("Read hash");
         } catch (Exception e) {
         System.out.println("Failed reading Route data");
         e.printStackTrace();
         hash = new RouteHash();
         }
         } else {
         hash = new RouteHash();
         }

         Route route;

         route = hash.getRoute(InetAddress.getByName("212.136.172.14"));
         System.out.println("212.136.172.14: "+route.containsIP(InetAddress.getByName("212.136.172.14")));

         route = hash.getRoute(InetAddress.getByName("64.28.67.150"));
         System.out.println("64.28.67.150: "+route.containsIP(InetAddress.getByName("64.28.67.150")));
         */
        //	  System.out.println("212.136.160.93: "+hash.getAS(InetAddress.getByName("212.136.160.93")));
        //	  System.out.println("212.136.176.42: "+hash.getAS(InetAddress.getByName("212.136.176.42")));

        //got AS for ip 212.45.37.33: Internet Exchange Europe B.V.
        //got AS for ip 212.45.33.133: Internet Exchange Europe B.V.
        //got AS for ip 212.45.32.1: Internet Exchange Europe B.V.
        //got AS for ip 212.45.32.2: Internet Exchange Europe B.V.

        //got AS for ip 207.45.196.69:
        //got AS for ip 207.45.222.210:
        //got AS for ip 207.45.222.214:
        //got AS for ip 157.130.22.109:
    }

    static void test2() throws IOException, UnknownHostException, InterruptedException {
        AutSystem AS = new AutSystem(1103);

        System.out.println(AS.toString());
        System.out.println("TBM in Surf: " + AS.containsIP(InetAddress.getByName("130.161.216.243")));
        System.out.println("Leiden:	 " + AS.containsIP(InetAddress.getByName("132.229.132.11")));
        System.out.println("Slashdot:	 " + AS.containsIP(InetAddress.getByName("64.28.67.150")));

        IPRange range = new IPRange("194.53.224.0/19");

        System.out.println("In1:    " + range.contains(InetAddress.getByName("194.53.224.142")));
        System.out.println("In2:    " + range.contains(InetAddress.getByName("194.53.240.156")));
        System.out.println("Out1:    " + range.contains(InetAddress.getByName("194.53.208.156")));
    }

    static void test4() throws Exception {
        InetAddress ip = InetAddress.getByName("130.161.216.255");

        System.out.println(ip);
        byte[] addrBytes = ip.getAddress();

        System.out.println(addrBytes[0] + "." + addrBytes[1] + "." + addrBytes[2] + "." + addrBytes[3]);
        System.out.println(addrBytes[0] & 255);
        System.out.println(((long) (addrBytes[0] & 255)) << 24);
        System.out.println(Long.toString(((long) (addrBytes[0] & 255)) << 24, 16));
        long addr = (((long) (addrBytes[0] & 255)) << 24) + (((long) (addrBytes[1] & 255)) << 16) + (((long) (addrBytes[2] & 255)) << 8) + ((long) (addrBytes[3] & 255));
        long net = addr;

        System.out.println(((net & (255 << 24)) >> 24) + "." + ((net & (255 << 16)) >> 16) + "." + ((net & (255 << 8)) >> 8) + "." + (net & 255));
        System.out.println(Long.toString(addr, 16));	    
        System.out.println(Long.toString(net, 16));	   
        System.out.println(Long.toString(~net, 16));	    
        System.out.println(Long.toString(1 << (32 - 32), 16));	  
        System.out.println(Long.toString((~net) | (1 << (32 - 32)), 16));	     
        System.out.println(Long.toString(~((~net) | (1 << (32 - 32))), 16));	
        net = ~(~net | (1 << (32 - 32)));
        System.out.println(Long.toString(net, 16));	   
        System.out.println(((net & 255 * 256 * 256 * 256) >> 24) + "." + ((net & 255 * 256 * 256) >> 16) + "." + ((net & 255 * 256) >> 8) + "." + (net & 255));
        for (int prefix = 32; prefix >= 0; prefix--) {
            System.out.println(prefix + ": " + ((net & (255 << 24)) >> 24) + "." + ((net & (255 << 16)) >> 16) + "." + ((net & (255 << 8)) >> 8) + "." + (net & 255));
            net = ~(~net | (1 << (32 - prefix)));
        }

    }

    static void test5() throws Exception {
        Host dummy = new Host("192.168.1.0", null);
        Host host1 = new Host("192.168.1.1", null);
        Host host2 = new Host("192.168.1.2", null);
        Host host3 = new Host("192.168.1.3", null);
        Host host4 = new Host("192.168.1.4", null);
        Host host5 = new Host("192.168.1.5", null);
        Host host6 = new Host("192.168.1.6", null);
        Host host7 = new Host("192.168.1.7", null);
        Host host8 = new Host("192.168.1.8", null);

        DelayGraph g = new DelayGraph();

        EndToEnd trace = new EndToEnd();

        trace.addHop(host1);
        trace.addHop(host5);
        trace.addHop(host3);
        trace.setTime(41);
        g.addEndToEnd(trace);
	
        trace = new EndToEnd();
        trace.addHop(host1);
        trace.addHop(host5);
        trace.addHop(host4);
        trace.setTime(42.1);
        g.addEndToEnd(trace);

        trace = new EndToEnd();
        trace.addHop(host1);
        trace.addHop(host5);
        trace.addHop(host8);
        trace.setTime(52.2);
        g.addEndToEnd(trace);

        trace = new EndToEnd();
        trace.addHop(host2);
        trace.addHop(host5);
        trace.addHop(host3);
        trace.setTime(40.9);
        g.addEndToEnd(trace);

        trace = new EndToEnd();
        trace.addHop(host2);
        trace.addHop(host5);
        trace.addHop(host4);
        trace.setTime(40);
        g.addEndToEnd(trace);

        trace = new EndToEnd();
        trace.addHop(host2);
        trace.addHop(host5);
        trace.addHop(host8);
        trace.setTime(50);
        g.addEndToEnd(trace);

        trace = new EndToEnd();
        trace.addHop(host7);
        trace.addHop(host5);
        trace.addHop(host3);
        trace.setTime(36.1);
        g.addEndToEnd(trace);

        trace = new EndToEnd();
        trace.addHop(host7);
        trace.addHop(host5);
        trace.addHop(host4);
        trace.setTime(35);
        g.addEndToEnd(trace);

        trace = new EndToEnd();
        trace.addHop(host7);
        trace.addHop(host5);
        trace.addHop(host8);
        trace.setTime(45);
        g.addEndToEnd(trace);

        //	  g.addEdge(dummy, host1);

        g.exportXML("L:\\data\\tracing\\traces\\test\\test.xml");

        g.removeDullNodes();

        g.exportXML("L:\\data\\tracing\\traces\\test\\test-nodull.xml");

        //	  g.buildEquations();
    }
    
    static void test6() throws Exception {
        System.setProperty("line.separator", "\n");
        Vector traces = new Vector();

        // Read traces
        TTMData ttm = new TTMData(DATABASE_URL);

        //	      traces.add(ttm.getTrace(1,27, new java.util.Date(99,8,1,20,0))); // Sept 1, 1999, 20:00
        // traces = ttm.getPartialTracesSnapshot(1, 15, new java.util.Date(99, 8, 1, 20, 0));
        //	      traces = ttm.getPartialTracesSnapshot(20,30, new java.util.Date(99,8,1,20,0));

        System.out.println(((Trace) traces.elementAt(0)).toString());
        System.out.println(((Trace) traces.elementAt(1)).toString());
        System.out.println(((Trace) traces.elementAt(2)).toString());
        System.out.println(((Trace) traces.elementAt(3)).toString());

        // Convert to graph
        //	  DelayGraph graph = tracesToDelayGraph(traces);

        //	  graph.exportXML("L:\\data\\tracing\\traces\\test\\traces.xml");
        //	  graph.exportOtter("L:\\data\\tracing\\traces\\test\\traces.odf");
        //	  DelayGraph graph2 = graph.reduceUsingAutonomousSystems();
        //	  graph2.exportXML("L:\\data\\tracing\\traces\\test\\traces-AS.xml");
        //	  graph2.exportOtter("L:\\data\\tracing\\traces\\test\\traces-AS.odf");
    }

    static AutSystem testIP(ASHash hash, String ipAddress) throws IOException, UnknownHostException, InterruptedException {
        Log.logChar("Looking for " + ipAddress + "...");
        AutSystem AS = hash.getAS(InetAddress.getByName(ipAddress));

        Log.log(" AS" + AS.number + " (" + AS.name + ")");
        return AS;
    }

    static void test7() throws IOException, UnknownHostException, InterruptedException {
        ASHash hash;

        try {
            hash = ASHash.readASHash("E:\\Temp\\AS-cache.xml");
        } catch (Exception e) {
            Log.log("Error reading hash table, creating new");
            hash = new ASHash();
        }

        testIP(hash, "172.21.1.73");
        //	  testIP(hash, "192.108.148.10"); // Slovak...
        //	  testIP(hash, "192.108.148.20");
        testIP(hash, "192.121.49.1");
        testIP(hash, "192.121.49.2");
        testIP(hash, "192.124.58.12");
        testIP(hash, "192.124.58.9");
        testIP(hash, "192.205.37.26");
        testIP(hash, "192.36.145.1");
        testIP(hash, "192.67.199.19");
        testIP(hash, "192.67.199.30");
        testIP(hash, "192.67.199.42");
        testIP(hash, "192.67.199.7");
        testIP(hash, "192.67.199.9");
        //	  testIP(hash, "192.68.191.17");  // ESNET
        //	  testIP(hash, "192.68.191.18");
        //	  testIP(hash, "192.68.191.66");
        //	  testIP(hash, "192.68.191.69");
        //	  testIP(hash, "192.68.191.83");
        //	  testIP(hash, "192.68.191.85");
        testIP(hash, "193.0.15.129");
        testIP(hash, "193.188.137.61");
        //	  testIP(hash, "194.50.100.191");
        //	  testIP(hash, "194.50.100.90");
        testIP(hash, "194.68.129.101");
        testIP(hash, "194.68.129.207");
        testIP(hash, "194.68.129.221");
        testIP(hash, "194.68.129.241");
        testIP(hash, "194.71.10.1");
        testIP(hash, "194.71.10.3");
        testIP(hash, "194.71.10.4");
        //	  testIP(hash, "203.50.126.10");
        //	  testIP(hash, "203.50.126.17");
        //	  testIP(hash, "203.50.13.68");
        //	  testIP(hash, "204.137.135.250");
        //	  testIP(hash, "205.254.172.22");
        //	  testIP(hash, "206.40.205.73");
        testIP(hash, "208.30.198.34");
        testIP(hash, "208.30.204.17");
        testIP(hash, "208.30.204.18");
        testIP(hash, "208.30.208.10");
        testIP(hash, "208.30.208.9");
        //	  testIP(hash, "209.101.92.129");
        //	  testIP(hash, "63.144.128.221");
        //	  testIP(hash, "63.144.128.222");
        //	  testIP(hash, "63.144.131.65");
        //	  testIP(hash, "63.144.131.66");
        testIP(hash, "65.36.0.105");
        testIP(hash, "65.36.0.198");
        testIP(hash, "65.36.0.54");
        testIP(hash, "65.36.0.58");

        try {
            hash.writeASHash(hash, "E:\\Temp\\AS-cache.xml");
        } catch (Exception e) {
            Log.log("Error writing hash table");
        }

    }

    static void test8() {
        Host host = new Host("255.255.255.255", null);

        System.out.println(host);
        System.out.println(IPRange.isBogusIPAddress(host.ip));

        byte[] bytes = host.ip.getAddress();

        System.out.println(bytes[0] + "." + bytes[1] + "." + bytes[2] + "." + bytes[3]);

        java.util.Date date = new java.util.Date(((long) 995155037) * ((long) 1000));

        System.out.println(date.toString());
    }

    static void test9() throws Exception {

        Host dummy = new Host("130.161.216.243", null);
        Host host1 = new Host("166.49.117.38", null);
        Host host2 = new Host("206.157.77.217", null);
        Host host3 = new Host("206.157.77.170", null);
        Host host4 = new Host("206.157.77.109", null);
        Host host5 = new Host("206.157.77.105", null);
        Host host6 = new Host("166.48.11.253", null);
        Host host7 = new Host("204.70.143.52", null);
        Host host8 = new Host("204.70.67.86", null);
        Host host9 = new Host("212.63.192.50", null);
        Host host10 = new Host("212.63.192.49", null);
        Host host11 = new Host("204.70.9.132", null);
        Host host12 = new Host("166.49.67.74", null);
        Host host13 = new Host("213.254.0.1", null);

        DelayGraph g = new DelayGraph();

        System.out.print(".");

        EndToEnd trace = new EndToEnd();

        trace.addHop(host1);
        trace.addHop(host5);
        trace.addHop(host3);
        trace.setTime(41);
        g.addEndToEnd(trace);

        trace = new EndToEnd();
        trace.addHop(host1);
        trace.addHop(host5);
        trace.addHop(host4);
        trace.setTime(42.1);
        g.addEndToEnd(trace);

        trace = new EndToEnd();
        trace.addHop(host1);
        trace.addHop(host5);
        trace.addHop(host8);
        trace.setTime(52.2);
        g.addEndToEnd(trace);

        trace = new EndToEnd();
        trace.addHop(host2);
        trace.addHop(host5);
        trace.addHop(host3);
        trace.setTime(40.9);
        g.addEndToEnd(trace);

        trace = new EndToEnd();
        trace.addHop(host2);
        trace.addHop(host5);
        trace.addHop(host4);
        trace.setTime(40);
        g.addEndToEnd(trace);

        trace = new EndToEnd();
        trace.addHop(host2);
        trace.addHop(host5);
        trace.addHop(host8);
        trace.setTime(50);
        g.addEndToEnd(trace);

        trace = new EndToEnd();
        trace.addHop(host7);
        trace.addHop(host5);
        trace.addHop(host3);
        trace.setTime(36.1);
        g.addEndToEnd(trace);

        trace = new EndToEnd();
        trace.addHop(host7);
        trace.addHop(host5);
        trace.addHop(host4);
        trace.setTime(35);
        g.addEndToEnd(trace);

        trace = new EndToEnd();
        trace.addHop(host7);
        trace.addHop(host5);
        trace.addHop(host8);
        trace.setTime(45);
        g.addEndToEnd(trace);

        trace = new EndToEnd();
        trace.addHop(host8);
        trace.addHop(host9);
        trace.addHop(host10);
        trace.addHop(host11);
        trace.addHop(host12);
        trace.setTime(45);
        g.addEndToEnd(trace);

        trace = new EndToEnd();
        trace.addHop(host8);
        trace.addHop(host9);
        trace.addHop(host13);
        trace.addHop(host11);
        trace.addHop(host12);
        trace.setTime(45);
        g.addEndToEnd(trace);

        //	  g.addEdge(dummy, host1);

        //	  System.out.println(g);

        //	  g.exportXML("L:\\data\\tracing\\traces\\test\\test.xml");

        System.out.print(".");

        g.removeDullNodes();

        //	  g.exportXML("L:\\data\\tracing\\traces\\test\\test-nodull.xml");
        /*
         System.out.print(".");
         DelayGraph g2=g.reduceUsingAutonomousSystems(AS_DATA_FILE);
         System.out.println(g2);
         System.out.print(".");
         g2.exportXML("L:\\data\\tracing\\traces\\test\\test-AS.xml");
         System.out.print(".");
         */
        g.buildEquations();
    }

    static void test10() throws Exception {
        Whois.test();
    }

    static void test11() throws Exception {
        AutSystem AS;

        System.out.println(IPRange.isBogusIPAddress(InetAddress.getByName("172.21.1.3")));
        System.out.println(IPRange.isBogusIPAddress(InetAddress.getByName("172.15.0.0")));
        System.out.println(IPRange.isBogusIPAddress(InetAddress.getByName("172.16.0.0")));
        System.out.println(IPRange.isBogusIPAddress(InetAddress.getByName("172.17.0.0")));
        System.out.println(IPRange.isBogusIPAddress(InetAddress.getByName("172.31.0.0")));
        System.out.println(IPRange.isBogusIPAddress(InetAddress.getByName("172.32.0.0")));
        System.out.println(IPRange.isBogusIPAddress(InetAddress.getByName("172.48.0.0")));
        Whois inet2 = new Whois("whois.internet2.edu", 43, false);
        int number = inet2.getNumberFromIP(InetAddress.getByName("198.32.8.1"));

        System.out.println("AS number found: " + number);

        /*
         AS = new AutSystem(InetAddress.getByName("198.32.8.1"));
         System.out.println(AS.toString());

         AS = new AutSystem(InetAddress.getByName("192.68.191.17"));
         System.out.println(AS.toLongString());

         AS = new AutSystem(InetAddress.getByName("192.67.199.7"));
         System.out.println(AS.toLongString());
         */
    }

    static void test12() throws Exception {
        Host box1 = TestBox.getTestBox(1);

        System.out.println(box1.getName() + " " + box1.getLabel() + " " + box1.getProperty("testboxnumber"));

        Host box = TestBox.getTestBox(28);

        System.out.println(box.getName() + " " + box.getLabel() + " " + box.getProperty("testboxnumber"));
    }

    static void test13() throws Exception {
        Log.showLog();
        ASHash hash;

        try {
            hash = ASHash.readASHash("E:\\Temp\\AS.dat");
        } catch (Exception e) {
            hash = new ASHash();
        }
        //	  PrintWriter out = new PrintWriter(new FileWriter("e:\\temp\\AS-cache.xml"));
        //	  hash.writeXML(out, "");
        //	  out.flush();
        //	  out.close();

        //	  System.out.println("Consolidating");
        //	  hash.consolidate();
        /*
         InetAddress net = InetAddress.getByName("193.0.15.130");
         byte[] netBytes = net.getAddress();
         Long longNet = new Long((((long)(netBytes[0] & 255)) << 24) + (((long)(netBytes[1] & 255)) << 16) + (((long)(netBytes[2] & 255)) << 8) + ((long)(netBytes[3] & 255)));
         hash.autsystems[32].remove(longNet);


         net = InetAddress.getByName("192.65.192.17");
         netBytes = net.getAddress();
         longNet = new Long((((long)(netBytes[0] & 255)) << 24) + (((long)(netBytes[1] & 255)) << 16) + (((long)(netBytes[2] & 255)) << 8) + ((long)(netBytes[3] & 255)));
         hash.autsystems[32].remove(longNet);

         AutSystem Tele2AS = new AutSystem(8873);
         hash.addAS(Tele2AS);
         net = InetAddress.getByName("212.63.192.49");
         netBytes = net.getAddress();
         longNet = new Long((((long)(netBytes[0] & 255)) << 24) + (((long)(netBytes[1] & 255)) << 16) + (((long)(netBytes[2] & 255)) << 8) + ((long)(netBytes[3] & 255)));
         hash.autsystems[32].remove(longNet);
         hash.autsystems[32].put(longNet, Tele2AS);

         testIP(hash, "193.0.15.130");
         testIP(hash, "192.65.192.17");
         */
        //	  hash.print();
        hash.printSuspiciousEntries();

        //	  try {
        //	      hash.writeASHash(hash, "E:\\Temp\\AS.dat");
        //	  } catch (Exception e) {
        //	  }

        while (true) {
        }
    }

    static void test14() throws Exception {
        ASHash hash;

        try {
            hash = ASHash.readASHash("E:\\Temp\\AS.dat");
        } catch (Exception e) {
            System.out.println("Error reading hash table, creating new");
            hash = new ASHash();
        }

        System.out.println("Removing AS0 from hash table");
        hash.removeAS0();

        try {
            hash.writeASHash(hash, "E:\\Temp\\AS.dat");
        } catch (Exception e) {
            System.out.println("Error writing hash table");
        }
    }

    //     static void test15() throws Exception {
    //	   RootFileReader root=new RootFileReader("E:\\src\\root\\small.root");
    //	   System.out.println("1.");
    // //	     TKey key = (TKey) root.getKey("tree");
    // //	     System.out.println("1a.");
    // //	     System.out.println("key name: "+key.getName());
    // //	     System.out.println("key title: "+key.getTitle());
    //	   TTree tree = (TTree) root.get("tree");
    //	   System.out.println("tree name: "+tree.getName());
    //	   System.out.println("tree title: "+tree.getTitle());
    //	   System.out.println("2.");
    // //	     Object obj = key.getObject();
    //	   System.out.println("3.");
    // /*
    //	   TBranch branch = tree.getBranch("PacketId");
    //	   System.out.println("3a.");
    //	   System.out.println("# entries in brach PacketId: "+branch.getNEntries());
    //
    //	   TObjArray branches = tree.getBranches();
    //	   System.out.println("4.");
    //	   for(int i=branches.getLowerBound(); i<=branches.getUpperBound(); i++) {
    //	      System.out.println("branch #: "+i);
    //	       Object obj = branches.getElementAt(i);
    //	       System.out.println(obj.toString());
    //	   }
    // */
    // //	     Object obj = key.getObject();
    // //	     System.out.println(obj.toString());
    //     }

    static String resultSetToString(ResultSet R) throws SQLException {
        String s = "";

        ResultSetMetaData M = R.getMetaData();
        int numberOfColumns = M.getColumnCount();

        for (int i = 1; i <= numberOfColumns; i++) {
            s = s + "\"" + M.getColumnName(i) + "\"";
            if (i != numberOfColumns) s = s + ",";
        }
        s = s + "\n";
        while (R.next()) {
            for (int i = 1; i <= numberOfColumns; i++) {
                String value = R.getString(i);

                s = s + "\"" + value + "\"";
                if (i != numberOfColumns) s = s + ",";
            }
            s = s + "\n";
        }
        return s;
    }

    static void test16(String fileName) throws Exception {
        String URL = "jdbc:mysql://130.161.216.187/TTM?user=janpascal&password=????";

        //	  String FILENAME = "e:\\src\\root\\20010625\\tt02.ripe.net.20010625.txt";

        System.out.println("Adding delay data from " + fileName + " to MySQL database");
        Connection C = null;

        Class.forName("org.gjt.mm.mysql.Driver").newInstance();
        C = DriverManager.getConnection(URL);
        Statement S = C.createStatement();

        BufferedReader reader = new BufferedReader(new FileReader(fileName));

        reader.readLine(); // Skip header line
        int count = 0;

        while (reader.ready()) {
            String Q = "INSERT INTO Delays (packetid,src,srcport,dst,dstport,packetsize,arrivaltime,packetdelay,srcclock,dstclock,hops,routeid,srcntp,dstntp) VALUES ";

            for (int i = 0; i < 500 && reader.ready(); i++) {
                StringTokenizer tok = new StringTokenizer(reader.readLine());

                tok.nextToken(); // skip "i"-column
                if (i == 0)
                    Q = Q + "(";
                else
                    Q = Q + ",(";
                while (tok.hasMoreTokens()) {
                    Q = Q + tok.nextToken();
                    if (tok.hasMoreTokens()) Q = Q + ",";
                }
                Q = Q + ")";
            }
            //	      ResultSet R = S.executeQuery(Q);
            System.out.println(Q);
            count++;
            System.out.print(".");
            if (count % 50 == 0) System.out.println((500 * count) + "");
        }
        System.out.println();
        reader.close();
        C.close();
    }

    static void test17() throws Exception {
        for (int i = 32; i < 60; i++) {
            String boxname;

            if (i < 10)
                boxname = "tt0" + i + ".ripe.net";
            else
                boxname = "tt" + i + ".ripe.net";
		
            test16("e:\\src\\root\\20010625\\" + boxname + ".20010625.txt");
        }
    }

    static void test18() throws Exception {
        String URL = "jdbc:mysql://130.161.216.187/TTM?user=janpascal&password=????";
        TTMData ttm = new TTMData(URL);
        //	  java.util.Date date = new java.util.Date(((long)995500799)*1000);
        java.util.Date date = new java.util.Date(101, 5, 25, 13, 00);

        System.out.println(date.toString());
        //	  Trace trace = ttm.getDelayTrace(1, 23, date);
        //	  Vector traces = ttm.getTracesSnapshot(date);
        Vector traces = ttm.getDelaySnapshot(date);

        System.out.println(traces.toString());
        ttm.close();
    }

    static void test19() throws Exception {
        String URL = "jdbc:mysql://130.161.216.187/TTM?user=janpascal&password=????";
        //	  String FILENAME = "e:\\src\\root\\20010625\\tt02.ripe.net.20010625.txt";

        Connection C = null;

        Class.forName("org.gjt.mm.mysql.Driver").newInstance(); 		
        C = DriverManager.getConnection(URL);
        Statement S = C.createStatement();

        String Q;
        ResultSet R;

        /*
         Q =  "SELECT * \n" +
         "FROM Delays \n"+
         "WHERE src=1 and dst=3 and abs(arrivaltime-993440000) in \n"+
         "	 ( select min(abs(arrivaltime-993440000)) \n"+
         "	   from Delays \n"+
         "	   where src=1 and dst=3 \n"+
         "	   group by src,dst )\n";
         System.out.println(Q);
         R = S.executeQuery(Q);
         System.out.println(resultSetToString(R));
         */
        Q = "select min(abs(arrivaltime-993440000))\n" +
                "from Delays\n" +
                "where src=1 and dst=3\n" +
                "group by src,dst\n";
        System.out.println(Q);
        R = S.executeQuery(Q);
        System.out.println(resultSetToString(R));

        double offset = R.getDouble(1);

        System.out.println("Offset: " + offset);

        Q = "select *\n" +
                "from Delays\n" +
                "where src=1 and dst=3 and abs(abs(arrivaltime-993440000)-" + offset + ")<0.01\n";
        System.out.println(Q);
        R = S.executeQuery(Q);
        System.out.println(resultSetToString(R));

        C.close();
    }

    static Host[] hosts;

    static EndToEnd fill2(int first, int second, double delay) {
        EndToEnd trace = new EndToEnd();

        trace.addHop(hosts[first]);
        trace.addHop(hosts[second]);
        trace.setTime(delay);
        return trace;
    }
    
    static EndToEnd fill3(int first, int second, int third, double delay) {
        EndToEnd trace = new EndToEnd();

        trace.addHop(hosts[first]);
        trace.addHop(hosts[second]);
        trace.addHop(hosts[third]);
        trace.setTime(delay);
        return trace;
    }
    
    static EndToEnd fill4(int first, int second, int third, int fourth, double delay) {
        EndToEnd trace = new EndToEnd();

        trace.addHop(hosts[first]);
        trace.addHop(hosts[second]);
        trace.addHop(hosts[third]);
        trace.addHop(hosts[fourth]);
        trace.setTime(delay);
        return trace;
    }
    
    static EndToEnd fill5(int first, int second, int third, int fourth, int fifth, double delay) {
        EndToEnd trace = new EndToEnd();

        trace.addHop(hosts[first]);
        trace.addHop(hosts[second]);
        trace.addHop(hosts[third]);
        trace.addHop(hosts[fourth]);
        trace.addHop(hosts[fifth]);
        trace.setTime(delay);
        return trace;
    }
    
    static void test20() throws Exception {
        Log.redirectOut();
        hosts = new Host[15];
        for (int i = 0; i < 15; i++) {
            hosts[i] = new Host("130.161.1." + i, "130.161.1."+i);
        }

        DelayGraph g = new DelayGraph();

        g.addEndToEnd(fill2(1,3,4));
        g.addEndToEnd(fill2(2,3,4));
        g.addEndToEnd(fill2(5,6,4));
        g.addEndToEnd(fill2(5,7,4));

        g.addEndToEnd(fill5(1,3,4,5,6,19));
        g.addEndToEnd(fill5(1,3,4,5,7,19));
        g.addEndToEnd(fill5(2,3,4,5,6,19));
        g.addEndToEnd(fill5(2,3,4,5,7,19));

        //	  System.out.println(g);

        //	  g.exportXML("L:\\data\\tracing\\traces\\test\\test.xml");

        System.out.print(".");
/*
        Matrix A = g.buildCoefficientMatrix();
        A.print(2,0);
        LUDecomposition LU = new LUDecomposition(A);
        Matrix U = LU.getU();
        U.print(2,0);

        int rank = A.rank();
        int nulldim = A.getColumnDimension()-rank;
        System.out.println("Null dimension: "+nulldim);
        System.out.println("Pivot : "+LU.getPivot().toString());
        System.out.println("DoublePivot : "+LU.getDoublePivot().toString());
        
        // Unbound variables are the variables that do not have an
        // associated pivot in the U matrix
        int[] unbound = new int[nulldim];

        int k = 0;
        int i = 0;
        double[][] a = A.getArray();
        double[][] u = U.getArray();
        int j = 0;
        while (j<U.getColumnDimension()) {
            if (u[i][j]==0.) {
                // No pivot here
                System.out.println("Found not pivot, so unbound variable: "+j);
                unbound[k] = j;
                k++;
                i++;
                j++;
            } else {
                i++;
                j++;
            }
        }

        System.out.print("unbound: ");
        for(i=0; i<nulldim; i++) { System.out.print(unbound[i]+" "); }
        System.out.println();

        // Now create base of null space
        Matrix N = new Matrix(U.getRowDimension(), U.getColumnDimension());
        System.out.println("Empty N: ");
        N.print(2,0);
        j = U.getRowDimension()-1;
        i = j;
        double[][] n = N.getArray();
        while (j>=0) {
            if (U.get(i,j)==0.) {
                // No pivot here
                System.out.println("Found not pivot, so unbound variable: "+j);
                N.set(i,j,1);
                i--;
                j--;
            } else {
                System.out.println("Row for variable "+j);
                Matrix row = new Matrix(1, U.getColumnDimension());
                for (k=j+1; k<U.getColumnDimension(); k++) {
                    System.out.println("k = "+k);
                    Matrix test = N.getMatrix(k,k, 0, N.getColumnDimension()-1);
                    test.print(2,0);
                    row.plusEquals(N.getMatrix(k,k, 0, N.getColumnDimension()-1).times(U.get(j,k)));
                }
                System.out.println("Row: ");
                row.times(1/U.get(i,j)).print(2,0);
                N.setMatrix(j,j,0,N.getColumnDimension()-1, row.timesEquals(1/U.get(i,j)));
                i--;
                j--;
            }
        }
        System.out.println("Resulting N: ");
        N.print(2,0);

*/
    }

    static void test21() {
        double[][] a = {
                {1, 2, 3, 4, 5},
                {9, 5, 7, 6, 3},
                {6, 2, 7, 2, 8},
                {9, 3, 7, 3, 2},
                {5, 8, 3, 1, 0},
                {9, 3, 5, 7, 2},
                {6, 2, 6, 3, 5},
                {7, 3, 2, 8, 3},
                {8, 9, 12, 5, 8}
            };
        double[][] b = {
                {14},
                {31},
                {24},
                {21},
                {17},
                {21},
                {12},
                {23},
                {42}
            };
        Matrix A = new Matrix(a);
        Matrix B = new Matrix(b);

        Matrix x;
        Matrix result;
        double deviation;
/*
        x = NNLS.NNLS(A, B);
        System.out.println("NNLS solution:");
        x.print(1, 5);
        result = A.times(x);
        System.out.println("A.x =");
        result.print(1, 5);
        deviation = result.minus(B).normF();
        System.out.println("sqrt(sum(sqr(Ax-b))) = " + deviation);
	
        x = A.qr().solve(B);
        System.out.println("QRD solution:");
        x.print(1, 5);
        result = A.times(x);
        System.out.println("A.x =");
        result.print(1, 5);
        deviation = result.minus(B).normF();
        System.out.println("sqrt(sum(sqr(Ax-b))) = " + deviation);
*/
        /*
         x = A.lu().solve(B);
         System.out.println("Exact solution:");
         x.print(1, 5);
         result = A.times(x);
         System.out.println("A.x =");
         result.print(1, 5);
         deviation = result.minus(B).normF();
         System.out.println("sqrt(sum(sqr(Ax-b))) = " + deviation);
         */
    }

    static void test22() throws Exception {
        ASHash hash;

        try {
            hash = ASHash.readASHash("E:\\Temp\\AS.dat");
        } catch (Exception e) {
            System.out.println("Error reading hash table, creating new");
            e.printStackTrace();
            hash = new ASHash();
        }

        AutSystem AS = hash.getAS("198.32.8.1");

        System.out.println(AS.toLongString());

        try {
            hash.writeASHash(hash, "E:\\Temp\\AS.dat");
        } catch (Exception e) {
            System.out.println("Error writing hash table");
        }
    }

    static void test23() throws Exception {
        ASHash hash;

        try {
            hash = ASHash.readXML("E:\\Temp\\AS-cache.xml");
        } catch (Exception e) {
            e.printStackTrace();
            hash = new ASHash();
        }
        Log.log("AS data read");
        hash.printSuspiciousEntries();
        while (true) {
        }
    }

    static DelayGraph tracesToDelayGraph(Vector traces) {
        DelayGraph graph = new DelayGraph();

        graph.addNodes(TestBox.getTestBoxSet());
        for (Enumeration e1 = traces.elements(); e1.hasMoreElements();) {
            Trace trace = (Trace) e1.nextElement();

            graph.addEndToEnd(trace);
        }
        return graph;
    }

    static void test24(String password) throws Exception {
        //	  System.out.print("Enter password: ");
        //	  BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
        //	  String password = in.readLine();
        if (password == null || password.equals("")) System.out.println("Usage: make test TEST=\"24 <password>\"");
        TTMData ttm = new TTMData("jdbc:mysql://130.161.216.187/TTM?user=janpascal&password=" + password);

        ttm.setTables("Backup_Records", "Backup_Routes", "Delays");

        Vector traces = new Vector();
        Trace trace;

//        trace = ttm.getTrace(1, 9, new java.util.Date(99, 8, 1, 20, 0));
//        if (trace != null) traces.add(trace);
//        trace = ttm.getTrace(1, 9, new java.util.Date(99, 9, 1, 20, 0));
//        if (trace != null) traces.add(trace);

            /*
             trace = ttm.getTrace(1,9,new java.util.Date(99,10,1,20,0));
             if (trace!=null) traces.add(trace);
             trace = ttm.getTrace(1,9,new java.util.Date(99,11,1,20,0));
             if (trace!=null) traces.add(trace);
             */
        DelayGraph graph = tracesToDelayGraph(traces);

        graph.removeUnconnectedNodes();

        JWindow f = new JWindow(); // Frame("Testing GraphComponent");
        GraphComponent c = new GraphComponent(graph);

        f.getContentPane().add(c);
        f.setSize(600, 500);
        //	  f.addActionListener(c);
        f.show();
        c.setShowLabels(NodeComponent.SHOW_LABEL_NAME);

        GraphComponent view = c;

        DocFlavor flavor = DocFlavor.SERVICE_FORMATTED.PRINTABLE;
        String psMimeType = DocFlavor.BYTE_ARRAY.POSTSCRIPT.getMimeType(); // "application/postscript";
        FileOutputStream fos;
        StreamPrintService psPrinter;
        StreamPrintServiceFactory[]spsFactories = StreamPrintServiceFactory.lookupStreamPrintServiceFactories(flavor, psMimeType);

        Log.log("Got " + spsFactories.length + " factories");
        if (spsFactories.length > 0) {
            try {
                Log.log("Opening file E:\\temp\\out.ps");
                fos = new FileOutputStream("E:\\temp\\out.ps");
                psPrinter = spsFactories[0].getPrintService(fos);
            } catch (FileNotFoundException e) {
                Log.log(e);
                return;
            }
            DocPrintJob job = psPrinter.createPrintJob();
            PrintRequestAttributeSet aset = new HashPrintRequestAttributeSet();

            Log.log("Creating job");
            Doc doc = new SimpleDoc(view, flavor, null);

            try {
                Log.log("Printing job");
                job.print(doc, aset);
            } catch (PrintException e) {
                Log.log(e);
                return;
            }
            try {
                Log.log("Closing file");
                fos.close();
            } catch (IOException e) {
                Log.log(e);
                return;
            }
        }

        //	  g.buildEquations();
    }

    static void test25(String password) throws Exception {
        //	  System.out.print("Enter password: ");
        //	  BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
        //	  String password = in.readLine();
        if (password == null || password.equals("")) System.out.println("Usage: make test TEST=\"24 <password>\"");
        TTMData ttm = new TTMData("jdbc:mysql://130.161.216.187/TTM?user=janpascal&password=" + password);

        Vector traces = new Vector();
        Trace trace;

//        trace = ttm.getTrace(1, 9, new java.util.Date(99, 8, 1, 20, 0));
//        if (trace != null) traces.add(trace);
//        trace = ttm.getTrace(1, 9, new java.util.Date(99, 9, 1, 20, 0));
//        if (trace != null) traces.add(trace);

            /*
             trace = ttm.getTrace(1,9,new java.util.Date(99,10,1,20,0));
             if (trace!=null) traces.add(trace);
             trace = ttm.getTrace(1,9,new java.util.Date(99,11,1,20,0));
             if (trace!=null) traces.add(trace);
             */
        DelayGraph graph = tracesToDelayGraph(traces);

        graph.removeUnconnectedNodes();

        //	  XMLSerializer.write(graph, "C:\\TEMP\\test1.xml");
        //	  g.buildEquations();
    }

    static void test26() throws Exception {
        String cacheLocation = "E:\\Temp\\AS-cache.xml";
        ASHash hash;

        if (new File(cacheLocation).canRead()) {
            try {
                hash = ASHash.readASHash(cacheLocation);
                //		  Log.log("Read hash: " + hash.toString());
            } catch (Exception e) {
                Log.log("Failed reading AS data, starting with empty cache");
                //		  e.printStackTrace();
                hash = new ASHash(true);
            }
        } else {
            Log.log("No AS data, starting with empty cache");
            hash = new ASHash(true);
        }
        DelayGraph graph = DelayGraph.readXML(hash, "e:\\temp\\traces-oneday-ASname.xml");

        Frame f = new Frame("Testing GraphComponent");
        GraphComponent c = new GraphComponent(graph);

        c.setShowLabels(NodeComponent.SHOW_LABEL_NAME);
        f.add(c);
        f.setSize(600, 500);
        //	  f.addActionListener(c);
        f.show();
        c.setZoomFixed(true);

        while (Log.log.isVisible()) {
        }

        //	  graph.exportXML("e:\\temp\\test1.xml");
    }

    static void test27() throws Exception {
        DatagramSocket socket = new DatagramSocket();

        byte[] dataOut = new String("Hello, echo server").getBytes();
        byte[] dataIn = new byte[1000];

        DatagramPacket out = new DatagramPacket(dataOut, 18, InetAddress.getByName("130.161.1.50"), 7);
        DatagramPacket in = new DatagramPacket(dataIn, 18);

        Log.log("Sending packet");
        socket.send(out);
        Log.log("Receiving packet");
        socket.receive(in);

        Log.log("Got packet: " + in.toString());
        Log.log("Got String: " + new String(dataIn));
        Log.log("Got String: " + new String(in.getData()));
        Log.log("Finished");
    }

    static void test28() throws Exception {
        Graph graph = new Graph();

        Node n1 = new Host("192.168.1.1", null);
        Node n2 = new Host("192.168.1.2", null);
        Node n3 = new Host("192.168.1.3", null);
        Node n4 = new Host("192.168.1.4", null);

        graph.addEdge(new Edge(n1, n2));
        graph.addEdge(new Edge(n2, n3));
        graph.addEdge(new Edge(n3, n4));
        graph.addEdge(new Edge(n4, n1));
        graph.addEdge(new Edge(n1, n3));
        graph.addEdge(new Edge(n2, n4));

        Frame f = new Frame("Testing GraphComponent");
        GraphComponent c = new GraphComponent(graph);

        c.setShowLabels(NodeComponent.SHOW_LABEL_NAME);
        f.add(c);
        f.setSize(600, 500);
        f.show();
        c.setZoomFixed(true);

        while (Log.log.isVisible()) {
        }

        graph.exportXML("e:\\temp\\test1.xml");
    }

    static void test29() throws Exception {
        Log.redirectOut();
        AnnealLayout.test();
    }

    static void test30() throws Exception {
//        Log.redirectOut();
        ASHash hash = ASHash.readASHash("e:\\temp\\as-cache.xml");
        Graph graph = DelayGraph.readXML(hash, "e:\\temp\\traces\\traces-12days01-trace-AS.xml");
        Frame f = new Frame("Testing annealing");
        GraphComponent c = new GraphComponent(graph);
        c.setShowLabels(NodeComponent.SHOW_LABEL_NAME);
        f.add(c);
        f.setSize(1024, 900);
        f.show();
        c.setZoomFixed(true);

        AnnealLayout annealLayout = new AnnealLayout(graph, c);
        annealLayout.setGamma(0.85);
        annealLayout.setStartT(10);
        annealLayout.setEndT(7);
        annealLayout.setNodeRepulsion(1);
        annealLayout.setBorderRepulsion(1);
        annealLayout.setEdgeCrossCutoffT(15);
        annealLayout.setEdgeStretch(0.00000001);
        annealLayout.setEdgeCrossing(10);
        annealLayout.setNodeEdgeRepulsion(0.1);

        PartialLayout layout = new PartialLayout(graph, c, annealLayout, 0.825, PartialLayout.GET_LOW_DEGREE_NODES, true);
        layout.doLayout();

        while (layout.isLayouting()) {
            Thread.sleep(100);
        }

    }

    static void test31() throws Exception {
//        Log.redirectOut();
        final int NUM_NODES=100;
        final int NUM_EDGES=100;
        Node[] nodes = new Node[NUM_NODES];
        Random random = new Random();

        DelayGraph graph = new DelayGraph();

        for (int i=0; i<NUM_NODES; i++) {
            nodes[i] = new AbstractNode("N"+i, "N"+i); 
            nodes[i].setPosition(1024*random.nextDouble(), 900*random.nextDouble());
            graph.addNode(nodes[i]);
        }

        for (int i=0; i<NUM_EDGES; i++) {
            Node n1 = nodes[random.nextInt(NUM_NODES)];
            Node n2;
            do {
                n2 = nodes[random.nextInt(NUM_NODES)];
            } while (n1 == n2);
            graph.addEdge(n1,n2);
        }

        Frame f = new Frame("Testing annealing");
        GraphComponent c = new GraphComponent(graph);
        c.setShowLabels(NodeComponent.SHOW_LABEL_NAME);
        f.add(c);
        f.setSize(1024, 900);
        f.show();
        c.setZoomFixed(true);

        AnnealLayout annealLayout = new AnnealLayout(graph, c);
        annealLayout.setGamma(0.85);
        annealLayout.setStartT(10);
        annealLayout.setEndT(0.00000001);
        annealLayout.setNodeRepulsion(1);
        annealLayout.setBorderRepulsion(1);
        annealLayout.setEdgeCrossCutoffT(0.000001);
        annealLayout.setEdgeStretch(0.00000001);
        annealLayout.setEdgeCrossing(10);
        annealLayout.setNodeEdgeRepulsion(0.1);

        PartialLayout layout = new PartialLayout(graph, c, annealLayout, 0.825, PartialLayout.GET_LOW_DEGREE_NODES, true);
        layout.doLayout();

        while (layout.isLayouting()) {
            Thread.sleep(300);
        }

    }

    static void test32() {
        double[][] a = {
                {1,0,0,1,1,1,0,0},
                {1,0,0,1,1,0,1,0},
                {1,0,0,1,1,0,0,1},
                {0,1,0,1,1,1,0,0},
                {0,1,0,1,1,0,1,0},
                {0,1,0,1,1,0,0,1},
                {0,0,1,1,1,1,0,0},
                {0,0,1,1,1,0,1,0},
                {0,0,1,1,1,0,0,1}
            };
        double[][] a2 = {
                {1,0,0,1,0,0},
                {1,0,0,0,1,0},
                {1,0,0,0,0,1},
                {0,1,0,1,0,0},
                {0,1,0,0,1,0},
                {0,1,0,0,0,1},
                {0,0,1,1,0,0},
                {0,0,1,0,1,0},
                {0,0,1,0,0,1},
            };
        double[][] a3 = {
                {1,0,0,0,0,0},
                {0,1,0,0,0,0},
                {0,0,0,0,1,0},
                {0,0,0,0,0,1},
                {1,0,1,1,1,0},
                {0,1,1,1,1,0},
                {1,0,1,1,0,1},
                {0,1,1,1,0,1},
            };
        double[][] t = {
                {1,0,0,0,0,0},
                {0,1,0,0,0,0},
                {0,0,1,1,0,0},
                {0,0,0,0,1,0},
                {0,0,0,0,0,1},
            };
        double[][] ab = {
                {1,0,0,0,0,0},
                {0,1,0,0,0,0},
                {0,0,0,0,1,0},
                {0,0,0,0,0,1},
                {1,0,1,1,1,0},
                {0,1,1,1,1,0},
                {1,0,1,1,0,1},
                {0,1,1,1,0,1},
            };
        double[][] b = {
                {14},
                {31},
                {24},
                {21},
                {17},
                {21},
                {12},
                {23},
                {42}
            };
        Log.redirectOut();
		Log.setColumns(150);
        Matrix A = new Matrix(a3);
        Matrix B = new Matrix(b);
        Matrix T = new Matrix(t);

        testMatrix(new Matrix(a3));

/*
        System.out.println("T: ");
        T.print(2,0);

        Matrix TT = T.transpose();
        System.out.println("T': ");
        TT.print(2,0);

        Matrix ATT = A.times(TT);
        System.out.println("A.T': ");
        ATT.print(2,0);
        System.out.println("rank(A.T'): "+ATT.rank());
*/
    }

    static void test33() throws Exception {
        Log.redirectOut();
		Log.setColumns(150);
		
        ASHash hash = ASHash.readASHash("e:\\temp\\as-cache.xml");
//        DelayGraph g = DelayGraph.readXML(hash, "c:\\Temp\\traces-20011201-15.00-with-traces-AS.xml");
//        DelayGraph g = DelayGraph.readXML(hash, "c:\\Temp\\traces-19990901-1-12-AS-with-traces.xml");
//        DelayGraph g = DelayGraph.readXML(hash, "e:\\Temp\\traces\\test-990901-1-12-with-traces-AS.xml");
        DelayGraph g = DelayGraph.readXML(hash, "e:\\Temp\\traces\\delays-010625-20.00-AS.xml");
        nl.tudelft.tbm.matrix.SimpleMatrix A = g.buildCoefficientMatrix();
//        nl.tudelft.tbm.matrix.Test.testMatrix(A, );

//      nl.tudelft.tbm.matrix.VectorSpace S = new nl.tudelft.tbm.matrix.VectorSpace(A);
//        System.out.println("S: ");
//        S.A.print(2,0);

//System.out.println("Calculating nice basis...");

//nl.tudelft.tbm.matrix.SimpleMatrix niceBasis = S.makeNiceBasisFromRows(A);
//		System.out.println("Nice basis: ");
//		niceBasis.print(2,0);
//        nl.tudelft.tbm.matrix.SimpleMatrix B = A.times(niceBasis.transpose());

    }

    static void test35(String password) throws Exception {
        Log.redirectOut();
		Log.setColumns(150);
        if (password == null || password.equals("")) System.out.println("Usage: make test TEST=\"35 <password>\"");
        String URL = "jdbc:mysql://130.161.216.187/TTM?user=janpascal&password="+password;
        TTMData ttm = new TTMData(URL);
        java.util.Date date = new java.util.Date(101, 5, 25, 13, 00);


        System.out.println(date.toString());
        Vector traces = ttm.getDelaySnapshot(date);

        System.out.println(traces.toString());
        ttm.close();
        
        DelayGraph graph = tracesToDelayGraph(traces);
        ASHash hash = ASHash.readASHash("e:\\temp\\as-cache.xml");
        DelayGraph ASGraph = graph.reduceUsingAutonomousSystems(hash, DelayGraph.AS_FILTER_ALL, false);
        nl.tudelft.tbm.matrix.SimpleMatrix A = ASGraph.buildCoefficientMatrix();
//        nl.tudelft.tbm.matrix.Test.testMatrix(A);
    }

    static public void testMatrix(Matrix A) {
        System.out.println("A: ");
        A.print(2,0);

        LUDecomposition LU = new LUDecomposition(A);
        Matrix U = LU.getU();
        System.out.println("U: ");
        U.print(2,0);

        int rank = A.rank();
        System.out.println("rank(A): "+A.rank());
        int nulldim = A.getColumnDimension()-rank;
        System.out.println("Null dimension: "+nulldim);

        System.out.print("Pivot :");
        int[] pivots = LU.getPivot();
        for(int i=0; i<pivots.length; i++) {System.out.print(" "+pivots[i]);}
        System.out.println();
        
        int[] unbound = new int[nulldim];
        int l = nulldim-1;

		Log.log("Calculating basis of null space...");
        // Now create basis of null space
        Matrix N = new Matrix(U.getRowDimension(), U.getColumnDimension());

        for (int j=U.getRowDimension()-1; j>=0; j--) {
            double pivot = U.get(j,j);
            if (pivot==0. || pivot==-0.) {
                // No pivot here
                System.out.println("Found no pivot, so unbound variable: "+j);
                unbound[l] = j;
                l--;
                N.set(j,j,1);
            } else {
                Matrix row = new Matrix(1, U.getColumnDimension());
                for (int k=j+1; k<U.getColumnDimension(); k++) {
                    row.plusEquals(N.getMatrix(k,k, 0, N.getColumnDimension()-1).times(U.get(j,k)));
                }
                N.setMatrix(j,j,0,N.getColumnDimension()-1, row.timesEquals(-1/pivot));
            }
        }
        System.out.println("Resulting N: ");
//        N.print(2,0);

		Log.log("Calculating U2...");
        int[] rows = new int[rank];
        {
            l=0;
            int j=0;
            for(int i=0; i<U.getRowDimension(); i++) {
                if (l<nulldim && unbound[l] == i) {
                    l++;
                } else {
                    rows[j] = i;
                    j++;
                }
            }
        }

        Matrix U2 = U.getMatrix(rows, 0, U.getRowDimension()-1);
        System.out.println("U2: ");
        U2.print(2,0);

		Log.log("Calculating U2'...");
        Matrix U2T = U2.transpose();
        System.out.println("U2': ");
  //      U2T.print(2,0);

		Log.log("Calculating A.U2'...");
        Matrix AU2T = A.times(U2T);
  //      System.out.println("A.U2': ");
//        AU2T.print(2,0);
        System.out.println("rank(A.U2'): "+AU2T.rank());

		Log.log("Calculating nice basis...");

		Matrix niceBasis = makeNiceBasis(U2);
		Log.log("Nice basis: ");
		niceBasis.print(2,0);

		Log.log("rank of basis: "+niceBasis.rank());
    }

	/**
	 * In: basis contains basis vectors of subspace in rows
	 * vector is one row in the matrix 
	 */

	public static boolean subSpaceContainsVector(Matrix basis, Matrix vector) {
		Matrix basisPlus = new Matrix(basis.getRowDimension()+1, basis.getColumnDimension());
		basisPlus.setMatrix(0,basis.getRowDimension()-1, 0, basis.getColumnDimension()-1, basis);
		basisPlus.setMatrix(basis.getRowDimension(), basis.getRowDimension(), 0, basis.getColumnDimension()-1, vector);
//		System.out.println("rank: "+basisPlus.rank()+", was: "+ basis.getRowDimension());
		return basisPlus.rank() == basis.getRowDimension();
	}

    /** In: basis contains basis vectors of subspace in rows
	 *  vector is one row in the matrix
	 * this method does not check that the vector is not in the
	 * subspace spanned by the basis!
	 */
	public static Matrix addVectorToBasis(Matrix basis, Matrix vector) {
		Matrix result = new Matrix(basis.getRowDimension()+1, basis.getColumnDimension());
		result.setMatrix(0,basis.getRowDimension()-1, 0, basis.getColumnDimension()-1, basis);
		result.setMatrix(basis.getRowDimension(), basis.getRowDimension(), 0, basis.getColumnDimension()-1, vector);
		return result;
	}

	public static Matrix makeNiceBasis(Matrix basis) {
		int cols = basis.getColumnDimension();
		int rank = basis.rank();
		Matrix result = new Matrix(0, basis.getColumnDimension());

		for(int i=1; i<=cols; i++) {
			// Generate all vectors with i ones, rest zeros
			// First make a vector with the first i coefficients one, rest zero
			Matrix v = new Matrix(1, cols);
			for (int j=0; j<i; j++) {
				v.set(0,j,1);
			}

			Log.log("Trying vectors with "+i+" ones");

			int count = 0;
			do {
//				Log.log("Trying vector: ");
//				v.print(1,0);
				if (subSpaceContainsVector(basis, v) && !subSpaceContainsVector(result, v)) {
					Log.log("added vector to basis:");
					v.print(1,0);
					result = addVectorToBasis(result, v);
				}
				if(count%25 == 0) {
					v.print(1,0);
				}
				count++;
			} while (nextBasisTry(v));
			if(result.rank()>=rank) {
				Log.log("Reached rank "+result.rank());
				break;
			}
		}

		return result;
    }

	public static boolean nextBasisTry(Matrix v) {
		int numberOfOnesHad = 0;
		int cols = v.getColumnDimension();
		boolean finished = false;
		boolean noMore = false;
		int lastOne = cols-1;
		while (lastOne>0 && v.get(0,lastOne)<1) lastOne--;
		if (lastOne!=cols-1) {
			v.set(0,lastOne, 0);
			v.set(0, lastOne+1, 1);
		} else {
			do {
				v.set(0, lastOne, 0);
				numberOfOnesHad++;
				int previousOne = lastOne-1;
				while (previousOne>=0 && v.get(0,previousOne)<1) previousOne--;
				if (previousOne<0) {
					return false;
				}
				if (previousOne!=lastOne-1) {
					v.set(0, previousOne, 0);
					for(int j=0; j<numberOfOnesHad+1; j++) {
						v.set(0, previousOne+1+j, 1);
					}
					finished = true;
				} else {
					lastOne = previousOne;
				}
			} while (!finished);
		}
		return !noMore;
	}

    static void test34() throws Exception {
        Log.redirectOut();
		Log.setColumns(150);

		Log.log("1 one: ");
		Matrix v = new Matrix(1, 8);
		v.set(0,0, 1);

		do {
			v.print(1,0);
		} while (nextBasisTry(v));

		Log.log("2 ones: ");
		v = new Matrix(1, 8);
		v.set(0,0, 1);
		v.set(0,1, 1);
		
		do {
			v.print(1,0);
		} while (nextBasisTry(v));

		Log.log("3 ones: ");
		v = new Matrix(1, 8);
		v.set(0,0, 1);
		v.set(0,1, 1);
		v.set(0,2, 1);
		
		do {
			v.print(1,0);
		} while (nextBasisTry(v));

		Log.log("7 ones: ");
		v = new Matrix(1, 8);
		v.set(0,0, 1);
		v.set(0,1, 1);
		v.set(0,2, 1);
		v.set(0,3, 1);
		v.set(0,4, 1);
		v.set(0,5, 1);
		v.set(0,6, 1);
		
		do {
			v.print(1,0);
		} while (nextBasisTry(v));

		Log.log("8 ones: ");
		v = new Matrix(1, 8);
		v.set(0,0, 1);
		v.set(0,1, 1);
		v.set(0,2, 1);
		v.set(0,3, 1);
		v.set(0,4, 1);
		v.set(0,5, 1);
		v.set(0,6, 1);
		v.set(0,7, 1);
		
		do {
			v.print(1,0);
		} while (nextBasisTry(v));
	}

    static Set knownObjects;
    static void showFields(Object obj) {
        knownObjects = new HashSet();
        showFieldsImpl(obj, "");
    }

    private static void showFieldsImpl(Object obj, String indent) {
        knownObjects.add(obj);
        Class cl = obj.getClass();
        while(cl!=null) {
            logger.info(indent+"("+cl.getName()+")");
            Field[] fields = cl.getDeclaredFields();
            AccessibleObject.setAccessible(fields, true);
            for(int i=0; i<fields.length; i++) {
                Field field = fields[i];
                try {
                    Class fieldType = field.getType();
                    Object value = field.get(obj);
                    if (value==null || fieldType.isPrimitive() || value instanceof String || knownObjects.contains(value)) {
                        logger.info(indent+field.getName()+" = "+value); // .toString()
                    } else {
                        logger.info(indent+"(field: "+field.getName()+")");
                        showFieldsImpl(value, indent + "    ");
                    }
                } catch (IllegalAccessException e) {
                    logger.info(indent+"!! Error accessing field "+field.getName());
                }
            }
            cl = cl.getSuperclass();
        }
    }

    void test36() throws Exception {
        knownObjects = new HashSet();
        ASHash hash = ASHash.readASHash("e:\\temp\\as-cache.xml");
        Graph graph = DelayGraph.readXML(hash, "e:\\temp\\traces\\traces-12days01-trace-AS.xml");
        Frame f = new Frame("Testing annealing");
        GraphComponent c = new GraphComponent(graph);
        c.setShowLabels(NodeComponent.SHOW_LABEL_NAME);
        f.add(c);
        f.setSize(1024, 900);
        f.show();
        c.setZoomFixed(true);
        logger.info("---- component: ----");
        showFields(c);
        logger.info("---- graph: ----");
        showFields(graph);
    }

	void test37() throws Exception {
		URL url = new URL("http://www.tbm.tudelft.nl/webstaf/janb/software/traces/version");
		BufferedReader reader = new BufferedReader(new InputStreamReader(url.openStream()));
		boolean finished = false;
		while (!finished) {
			String s = reader.readLine();
			if (s==null) {
				finished = true;
				break;
			}
			logger.info("Read: "+s);
			if (s.startsWith("#")) {
				logger.info("-- Skipping comment");
			} else {
				StringTokenizer tok = new StringTokenizer(s);
				if (tok.countTokens()>=3) {
					String property = tok.nextToken();
					if (!"=".equals(tok.nextToken())) {
						logger.info("-- Syntax error");
						break;
					}
					String value = tok.nextToken();
					if (tok.hasMoreTokens() && !tok.nextToken().startsWith("#")) {
						logger.info("-- Syntax error");
					}
					logger.info("Interpreted: "+property+" = "+value);
				}
			}
		}
	}

	void test38() throws Exception {
		URL url = ClassLoader.getSystemResource("nl/tudelft/tbm/traces/README.html");
		logger.info(url.toString());
		InputStream is = ClassLoader.getSystemResourceAsStream("nl/tudelft/tbm/traces/README.html");
		BufferedReader reader = new BufferedReader(new InputStreamReader(is));
		boolean finished = false;
		while (!finished) {
			String s = reader.readLine();
			if (s==null) {
				finished = true;
				break;
			}
			logger.info("Read: "+s);
		}
		reader.close();
	}


    void initLogging() {
        Logger rootLogger = Logger.getLogger("");

        // Remove default handlers
        Handler[] old = rootLogger.getHandlers();
        for(int i=0; i<old.length; i++) {
            rootLogger.removeHandler(old[i]);
        }
        // Log all messages
        rootLogger.setLevel(Level.ALL);

        // Add Swing handler
        Handler handler = new SwingHandler();
        handler.setFormatter(new TrivialFormatter());
        rootLogger.addHandler(handler);
    }


    public void test(String[] args) {
        initLogging();
        Log.showLog();
        String arg1 = "";

        try {
            if (args.length >= 2) arg1 = args[1];
            String test = args[0];
            int num;

            if (test == null) num = 15;
            else {
                try {
                    num = Integer.parseInt(test);
                } catch (NumberFormatException e) {
                    num = 2;
                }
            }
            logger.info("Doing test #" + num);
            Method method = this.getClass().getDeclaredMethod("test"+num, null);
            if (method!=null) {
                method.invoke(this, null);
            } else {
                Class[] argtypes = new Class[1];
                argtypes[0] = arg1.getClass();
                method = this.getClass().getDeclaredMethod("test"+num, argtypes);
                Object[] arglist  = new Object[1];
                arglist[0] = arg1;
                method.invoke(this, arglist);
            }
        } catch (Throwable t) {
            t.printStackTrace();
        }
        Log.showLog();
        Log.log("Finished; close log window to exit");
        while (Log.log.isVisible()) {}
        System.exit(0);
    }

    public static void main(String[] args) {
        Test test = new Test();
        test.test(args);
    }
}

