/**
 * Title:	 Traces
 * Description:  An analysis tool for traceroutes
 * Copyright:	 Copyright (c) Jan-Pascal van Best
 * Email:	     janb@tbm.tudelft.nl
 * @author	     Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.traces;


import java.io.*;
import java.lang.*;
import java.util.*;
import java.util.logging.*;


public class TestBox extends Host {
    private static Logger logger = Logger.getLogger("nl.tudelft.tbm.traces");
    private static List testBoxen = null;
    private static Map hash = null;

    public static String dataFilename = null;

    static String[] testBoxData = {
            "tt01.ripe.net 193.0.0.4 52.3728 4.8878 RIPE NCC,Amsterdam,NL",
            "tt02.ripe.net 193.0.0.2 52.3726 4.8880 RIPE NCC,Amsterdam,NL",
            "tt03.ripe.net 193.0.0.28 52.3728 4.8878 RIPE NCC,Amsterdam,NL",
            "tt04.ripe.net 193.0.0.6 52.3728 4.8878 RIPE NCC,Amsterdam,NL",
            "tt05.ripe.net 193.0.0.24 52.3728 4.8878 RIPE NCC,Amsterdam,NL(weg)",
            "tt07.ripe.net 192.36.143.148 59.3176 18.0231 Stupi,Stockholm,SE",
            "tt08.ripe.net 193.10.252.21 59.3491 18.0691 NORDUnet,Stockholm,SE",
            "tt09.ripe.net 195.67.159.40 59.3306 17.9630 Telia Network,Stockholm,SE",
            "tt10.ripe.net 195.92.167.7 53.8023 -1.5343 Planet Online,Leeds,UK",
            "tt11.ripe.net 192.109.251.32 48.1505 11.6200 ECRC Network Services,Munich,DE",
            "tt12.ripe.net 193.149.44.25 48.1851 11.6015 Spacenet,Munich,DE",
            "tt13.ripe.net 192.87.106.111 52.3561 4.9531 Surfnet,Amsterdam,NL",
            "tt14.ripe.net 164.128.138.66 47.3900 8.5151 Swisscom,Ittingen,CH",
            "tt15.ripe.net 194.179.3.155 40.4036 -3.6953 Telefonica Data,Madrid,ES",
            "tt16.ripe.net 195.249.96.237 56.1081 10.1405 Tele Denmark Datanet,Viby,DK",
            "tt17.ripe.net 193.43.2.18 44.4861 11.2601 CINECA,Bologna,IT",
            "tt19.ripe.net 192.115.187.100 32.6306 35.0675 Golan Consulting,Haifa,IS",
            //"# tt19 heeft nu 212.116.184.53 als ip",
            "tt20.ripe.net 195.178.64.60 50.1015 14.3916 CESNET,Prague,CZ",
            "tt21.ripe.net 194.122.227.118 49.0126 8.4236 Xlink,Karlsruhe,DE",
            "tt22.ripe.net 194.160.23.2 48.1515 17.1131 SANET,Bratislava,SK",
            "tt23.ripe.net 209.211.237.18 41.1160 -73.7096 Advanced,Armonk,US",
            //"# tt23 had in 07-2000 als ip 207.24.7.8: truukje:",
            //"tt23.ripe.old 207.24.7.8 41.1160 -73.7096 Advanced,Armonk,US",
            "tt24.ripe.net 212.70.195.122 37.9765 23.7340 TEE,Athens,GR",
            "tt25.ripe.net 194.125.76.6 53.3343 -6.2573 Ireland Online,Dublin,IE",
            "tt26.ripe.net 195.66.248.74 51.4968 -0.0188 Linx,London,UK",
            "tt27.ripe.net 38.159.59.4 30.3195 -97.7006 MIDS,Austin,US",
            "tt28.ripe.net 134.79.196.38 37.4175 -122.2038 SLAC,Stanford,US",
            "tt30.ripe.net 212.56.224.2 49.6936 6.3288 Astra.Net,Luxembourg,LU",
            "tt31.ripe.net 192.65.185.39 46.2326 6.0463 CERN,Geneva,CH",
            "tt32.ripe.net 130.192.68.200 45.0400 7.6500 CSP,Torina,IT",
            "tt33.ripe.net 193.194.137.254 52.0205 8.5368 Dev.Consulting,Teuto,Bielefeld,DE",
            "tt34.ripe.net 193.166.4.105 60.1838 24.8173 FUNET,Espoo,FI",
            "tt35.ripe.net 193.1.198.10 53.3381 -6.2403 HEAnet,Dublin,IE",
            "tt36.ripe.net 129.88.9.1 45.1936 5.7700 IMAG,Grenoble,FR",
            "tt37.ripe.net 195.112.71.35 47.3813 8.5168 Internet Acces,Zurich,CH",
            "tt38.ripe.net 213.254.0.6 45.0728 7.6820 ITGate Network,Torino,IT",
            "tt39.ripe.net 62.157.150.94 51.3541 8.2510 MEGLA Gmbh,Meschede,DE",
            "tt40.ripe.net 212.5.128.15 42.6558 23.3708 Mobikom,Sofia,BU",
            "tt41.ripe.net 195.185.187.44 50.0323 8.7051 Nacamar,Dreieich,DE",
            "tt42.ripe.net 194.177.210.214 37.9856 23.7335 NTUA,Athens,GR",
            "tt43.ripe.net 193.158.32.29 48.1896 11.6505 Prosieben Info Serv,Munich,DE",
            "tt44.ripe.net 205.168.101.105 39.6825 -104.9691 Qwest,Denver,US",
            "tt45.ripe.net 193.216.127.6 59.9243 10.8106 Tele2,Norway,NO",
            "tt46.ripe.net 129.250.33.231 39.5713 -104.8808 Verio,Denver,US",
            "tt47.ripe.net 130.217.248.132 -37.7888 175.3180 New Zealand",
            "tt48.ripe.net 212.3.192.39 32.0593 34.8486 Surfree Ltd,IL",
            "tt49.ripe.net 157.159.11.30 48.6246 02.4430 INT,Evry,FR",
            "tt52.ripe.net 213.136.0.252 52.3065 05.6250 Business Internet Trends,NL"};

    double latitude = 0;
    double longitude = 0;
    String description = null;
    int number;

    public TestBox() {
        super();
        setDeletable(false);
    }

    public TestBox(String ipString, String hostName, int number, double latitude, double longitude, String description) {
        set(ipString, hostName);
        this.number = number;
        this.latitude = latitude;
        this.longitude = longitude;
        this.description = description;
        setDeletable(false);
    }

    public TestBox(int version, String line) throws NumberFormatException, NoSuchElementException {
        switch (version) {
        case 1:
            int prev, pos;

            pos = line.indexOf(" ");
            String hostName = line.substring(0, pos);

            prev = pos;
            pos = line.indexOf(" ", pos + 1);
            String ipString = line.substring(prev + 1, pos);

            set(ipString, hostName);
            number = Integer.parseInt(hostName.substring(2, 4));

            prev = pos;
            pos = line.indexOf(" ", pos + 1);
            String latString = line.substring(prev + 1, pos);

            prev = pos;
            pos = line.indexOf(" ", pos + 1);
            String longString = line.substring(prev + 1, pos);

            description = line.substring(pos + 1);

            latitude = Double.parseDouble(latString);
            longitude = Double.parseDouble(longString);
            break;

        case 2:
            StringTokenizer tok = new StringTokenizer(line);

            number = Integer.parseInt(tok.nextToken());
            hostName = tok.nextToken();
            ipString = tok.nextToken();
            set(ipString, hostName);
        }
        setDeletable(false);
    }

    public String toString() {
        return hostName;
    }

    public String toLongString() {
        return "Box #" + number + ": " + ip.getHostAddress() + " (" + hostName + ") @ " + latitude + "," + longitude + " \"" + description + "\"";
    }

    public String getName() {
        return ip.getHostAddress();
    }

    public String getLabel() {
        return hostName;
    }

    public List getPropertyNames() {
        String[] strings = {"testboxnumber", "latitude", "longitude", "description"};
        List result = super.getPropertyNames();

        result.addAll(Arrays.asList(strings));
        return result;
    }
    
    public String getProperty(String name) {
        String result;

        if (name.equals("testboxnumber")) {
            result = Integer.toString(number);
        } else if (name.equals("latitude")) {
            result = Double.toString(latitude);
        } else if (name.equals("longitude")) {
            result = Double.toString(longitude);
        } else if (name.equals("description")) {
            result = description;
        } else
            result = super.getProperty(name);
        return result;
    }

    public static Set getTestBoxSet() {
        if (testBoxen == null) makeTestBoxen();
        return new HashSet(testBoxen);
    }

    public static SortedSet getTestBoxNumbers() {
        if (testBoxen == null) makeTestBoxen();
        SortedSet result = new TreeSet();

        for (Iterator i = testBoxen.iterator(); i.hasNext();) {
            TestBox box = (TestBox) i.next();

            result.add(new Integer(box.number));
        }

        return result;
    }

    public static TestBox getTestBox(int number) {
        if (testBoxen == null) makeTestBoxen();
        TestBox box = (TestBox) hash.get(new Integer(number));

        if (box == null) {
            if (number < 10)
                box = new TestBox("192.168.1." + number, "tt0" + number + ".ripe.net", number, 0, 0, "Unknown testbox #" + number);
            else
                box = new TestBox("192.168.1." + number, "tt" + number + ".ripe.net", number, 0, 0, "Unknown testbox #" + number);
        }
        return box;
    }

    public static void clearCache() {
        testBoxen = null;
        hash = null;
    }

    protected static void makeTestBoxen() {
        logger.info("Making test boxes from table");
        testBoxen = new Vector();
        hash = new HashMap();
        for (int i = 0; i < testBoxData.length; i++) {
            try {
                TestBox box = new TestBox(1, testBoxData[i]);

                testBoxen.add(box);
                hash.put(new Integer(box.number), box);
            } catch (Exception e) {
                logger.info("Error in testboxdata line " + i);
                e.printStackTrace();
            }
        }
        if (dataFilename != null) {
            loadCacheFromFile(dataFilename);
        }
    }

    protected static void loadCacheFromFile(String filename) {
        logger.info("Loading testbox data from " + filename);
        if (testBoxen == null) {
            testBoxen = new Vector();
            hash = new HashMap();
        }

        BufferedReader in;

        try {
            in = new BufferedReader(new FileReader(filename));
        } catch (FileNotFoundException e) {
            logger.info("Testbox data file " + filename + " not found");
            logger.throwing("TestBox", "loadCacheFromFile", e);
            return;
        }

        int i = 0;
        String s;

        do {
            try {
                s = in.readLine();
            } catch (IOException e) {
                logger.info("Error reading from " + filename);
	            logger.throwing("TestBox", "loadCacheFromFile", e);
                s = null;
            }
            i++;
            if (s != null) {
                //		  logger.info("Looking at line "+s);
                if (s.equals("") || s.charAt(0) == '#') continue;
                try {
                    TestBox box = new TestBox(2, s);

                    // contains() works on IP address
                    if (!testBoxen.contains(box)) {
                        testBoxen.add(box);
                        hash.put(new Integer(box.number), box);
                        //			  logger.info("Added testbox: "+box);
                    }
                } catch (Exception e) {
                    logger.info("Error in " + filename + " line " + i);
		            logger.throwing("TestBox", "loadCacheFromFile", e);
                }
            }
        }
        while (s != null);
        logger.info("Finished loading testbox data");
    }
}

