/**
 * Title:	Traces
 * Description: An analysis tool for traceroutes
 * Copyright:	Copyright (c) Jan-Pascal van Best
 * Email:	    janb@tbm.tudelft.nl
 * @author	    Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.traces;


import java.util.*;
import java.io.*;
import java.lang.*;
import java.net.*;
import java.text.*;
import javax.swing.*;
import Jama.*;
import nl.tudelft.tbm.graph.*;


public class Traces {
    //    static String AS_DATA_FILE = "L:\\data\\tracing\\traces\\test\\AS.dat"; Moved to:
    static final String AS_DATA_FILE = "D:\\Java\\traces\\AS.dat";
    static final String ROUTES_DATA_FILE = "L:\\data\\tracing\\traces\\test\\Routes.dat";
    static final String DATABASE_URL = "jdbc:mysql://130.161.216.187/TTM?user=janpascal&password=??????";

    public static void readTracesFromDir(Vector traces, String dirName, int traceFormat) throws IOException {
        File dir = new File(dirName);

        if (!dir.isDirectory()) {
            System.out.println(dirName + " is not a directory!");
        } else {
            File[] files = dir.listFiles();

            System.out.println("Aantal files: " + files.length);
            for (int i = 0; i < files.length; i++) {
                // System.out.println("Lezen van trace uit"+files[i].getName()+", format "+traceFormat);
                BufferedReader reader = new BufferedReader(new FileReader(files[i]));
                Trace trace = new Trace(reader, traceFormat);

                traces.addElement(trace);
            }
        }
    }

    public static void readTraces(Vector traces, String directory) throws Exception {
        FileDialog dialog = new FileDialog("Choose trace directories", directory);

        int returnVal = dialog.createDialog();

        if (returnVal == 0) { 
            System.out.println("OK");
            String[] files = dialog.getFiles();

            System.out.println("Aantal directories: " + files.length);
            BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

            for (int i = 0; i < files.length; i++) {
                System.out.println("Lezen van files uit " + files[i]);
                //		  System.out.println("Format is NT, Linux?");
                //		  String response = in.readLine();
                //		  System.out.println(response + ","+response.substring(0,1).toLowerCase()+","+(response.substring(0,1).toLowerCase().equals("n")));
                //		  int traceFormat = (response.substring(0,1).toLowerCase().equals("n")) ? Trace.TRACE_FORMAT_WINNT : Trace.TRACE_FORMAT_LINUX;
                //		  System.out.println(traceFormat);
                readTracesFromDir(traces, files[i], dialog.traceFormat);
            }
        }
        System.out.println("Traces gelezen");
    }

    public static String getDomain(String host) {
        String domain = "unknown";
        int pos1 = host.lastIndexOf(".");
        int pos2 = host.lastIndexOf(".", pos1 - 1);

        if (pos2 < 0) pos2 = -1;
        if (pos1 > 0) {
            try {
                Integer.parseInt(host.substring(pos1 + 1, host.length()));
                if (!host.equals("0.0.0.0")) domain = host; // Only executed if it's an IP address
            } catch (NumberFormatException e) {
                domain = host.substring(pos2 + 1, pos1);
            }
        }
        return domain.toLowerCase();
    }

    public static Graph tracesToGraph(Vector traces) {
        Graph g = new Graph();

        for (int i = 0; i < traces.size(); i++) {
            Trace trace = (Trace) traces.elementAt(i);
            InetAddress prev = null; // ((Host)trace.hops.elementAt(0)).ip;

            for (int j = 0; j < trace.hops.size(); j++) {
                Host host = (Host) trace.hops.elementAt(j);
                InetAddress current = host.ip;

                if (prev != null && current != null && !IPRange.isBogusIPAddress(current) && !IPRange.isBogusIPAddress(prev) && !current.equals(prev)) {
                    g.addEdge(prev.getHostAddress(), current.getHostAddress());
                }
                prev = current;
            }
        }
        return g;
    }

    public static Graph tracesToDomainGraph(Vector traces) {
        Graph g = new Graph();

        for (int i = 0; i < traces.size(); i++) {
            Trace trace = (Trace) traces.elementAt(i);
            String prev = getDomain(((Hop) trace.hops.elementAt(0)).hostName);

            for (int j = 1; j < trace.hops.size(); j++) {
                Hop hop = (Hop) trace.hops.elementAt(j);
                String host = getDomain(hop.hostName);

                if (!host.equals("unknown") && !prev.equals("unknown") && !host.equals(prev)) {
                    g.addEdge(new String(prev), new String(host));
                    if (prev.equals("myweb") || prev.equals("wxs") || prev.equals("everyday"))
                        System.out.println("From " + prev + " to " + host);
                }
                prev = host;
            }
        }
        return g;
    }

    public static void writeRouteHash(RouteHash hash) throws IOException {
        System.out.println("Writing Route data to " + ROUTES_DATA_FILE);
        ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(ROUTES_DATA_FILE));

        out.writeObject(hash);
        out.close();
    }

    public static RouteHash readRouteHash() throws IOException, ClassNotFoundException {
        System.out.println("Reading Route data from " + ROUTES_DATA_FILE);
        ObjectInputStream in = new ObjectInputStream(new FileInputStream(ROUTES_DATA_FILE));
        RouteHash hash = (RouteHash) in.readObject();

        in.close();
        return hash;
    }

    public static Graph tracesToASGraph(Vector traces) throws IOException, ClassNotFoundException, InterruptedException {
        Graph g = new Graph();

        ASHash hash;

        if (new File(AS_DATA_FILE).canRead()) {
            try {
                hash = ASHash.readASHash(AS_DATA_FILE);
                //		  System.out.println("Read hash: " + hash.toString());
            } catch (Exception e) {
                System.out.println("Failed reading AS data");
                e.printStackTrace();
                hash = new ASHash();
            }
        } else {
            hash = new ASHash();
        }
        for (int i = 0; i < traces.size(); i++) {
            Trace trace = (Trace) traces.elementAt(i);
            AutSystem prev = null; // hash.getAS(trace.from.ip);

            for (int j = 0; j < trace.hops.size(); j++) {
                AutSystem current = null;
                Host hop = (Host) trace.hops.elementAt(j);

                if (!IPRange.isBogusIPAddress(hop.ip)) current = hash.getAS(hop.ip);
                    //		  System.out.println("AS"+current.number+": "+current.name);
                if (current != null && prev != null && prev.number > 0 && current.number > 0 && prev.number != current.number) {
                    //		      System.out.println("AS"+prev.number+" -> AS"+current.number);
                    g.addEdge("" + prev.number, "" + current.number);
                    //		      g.addEdge(new String("AS"+prev.number), new String("AS"+current.number));
                }
                prev = current;
            }
            System.out.print(".");
        }

        try {
            ASHash.writeASHash(hash, AS_DATA_FILE);
        } catch (Exception e) {
            System.out.println("Failed writing AS data");
            e.printStackTrace();
        }
	
        return g;
    }

    public static Graph tracesToRouteGraph(Vector traces) throws IOException, ClassNotFoundException, InterruptedException {
        Graph g = new Graph();

        RouteHash hash;

        if (new File(ROUTES_DATA_FILE).canRead()) {
            try {
                hash = readRouteHash();
                //		  System.out.println("Read hash: " + hash.toString());
            } catch (Exception e) {
                System.out.println("Failed reading Routes data");
                e.printStackTrace();
                hash = new RouteHash();
            }
        } else {
            hash = new RouteHash();
        }
        for (int i = 0; i < traces.size(); i++) {
            Trace trace = (Trace) traces.elementAt(i);
            Route prev = null; // hash.getRoute(trace.from.ip);

            for (int j = 0; j < trace.hops.size(); j++) {
                Hop hop = (Hop) trace.hops.elementAt(j);
                Route current = hash.getRoute(hop.ip);

                if (current != null && prev != null && !IPRange.isBogusIPAddress(hop.ip) && prev.ASNumber > 0 && current.ASNumber > 0 && !prev.description.equals(current.description)) {
                    g.addEdge(new String(prev.description), new String(current.description));
                }
                prev = current;
            }
        }

        try {
            writeRouteHash(hash);
        } catch (Exception e) {
            System.out.println("Failed writing Routes data");
            e.printStackTrace();
        }
	
        return g;
    }

    /*
     public static Graph tracesToDelayGraph_very_old(Vector traces) throws IOException, Exception {
     Graph g = new Graph();
     Vector links = new Vector(); // contains Pairs of InetAddresses
     
     // Set up the set of equations for calculating the hop delays
     // First find all unique links and put them in order in links
     System.out.println("Identify unique links");
     for (Enumeration e1 = traces.elements() ; e1.hasMoreElements();) {
     Trace trace=(Trace) e1.nextElement();
     InetAddress prev=trace.from.getIp();

     for (Enumeration e2 = trace.hops.elements() ; e2.hasMoreElements();) {
     Hop hop = (Hop)e2.nextElement();
     InetAddress current = hop.host.getIp();

     if (prev != null && current != null) {
     Pair link = new Pair(prev, current);
     int i=links.indexOf(link);
     if (i < 0) {
     links.add(link);
     i = links.size()-1;
     }
     }
     }
     }

     int numLinks = links.size();
     System.out.println("Building "+numLinks+" equations");
     Matrix matrix = new Matrix(numLinks, numLinks+1);
     matrix.clear();

     int i=0;
     for (Enumeration e0 = links.elements(); e0.hasMoreElements(); i++) {
     System.out.print(i+" ");
     Pair linki=(Pair) e0.nextElement();

     for (Enumeration e1 = traces.elements() ; e1.hasMoreElements();) {
     Trace trace=(Trace) e1.nextElement();
     InetAddress first=trace.from.getIp();
     InetAddress prev=first;
     boolean found = false;

     for (Enumeration e2 = trace.hops.elements() ; e2.hasMoreElements() && !found;) {
     Hop hop = (Hop)e2.nextElement();
     InetAddress current = hop.host.getIp();

     if (prev != null && current != null) {
     Pair linkj = new Pair(prev, current);
     //			  int j=links.indexOf(linkj);
     if (linki.equals(linkj)) found = true;
     }
     }

     if (found) { // we've found a trace that contains linki
     prev = first;
     for (Enumeration e2 = trace.hops.elements() ; e2.hasMoreElements(); ) {
     Hop hop = (Hop)e2.nextElement();
     InetAddress current = hop.host.getIp();

     if (prev != null && current != null) {
     Pair linkj = new Pair(prev, current);
     int j=links.indexOf(linkj);
     matrix.set(i,j,matrix.get(i,j) + 1);
     // FIXME: Is dit klaar of nog niet?
     }
     }
     matrix.set(i,numLinks, matrix.get(i,numLinks)+trace.time);
     }
     }
     }

     System.out.println("Finished building eqns");
     System.out.println(matrix);
     System.out.println("Solving equations, n="+numLinks);
     Matrix result = matrix.gaussJordan().subMatrix(0,numLinks-1, numLinks, numLinks);
     System.out.println(result);


     g.addEdge(new String(prev), new String(host));
     if (prev.equals("myweb") || prev.equals("wxs") || prev.equals("everyday"))
     System.out.println("From "+prev+" to "+host);
     }
     prev = host;
     }
     }

     return g;	 
     }
     */
    public static Graph tracesToDelayGraph_old(Vector traces) throws IOException, Exception {
        Graph g = new Graph();
        Vector links = new Vector(); // contains Pairs of InetAddresses

        // Set up the set of equations for calculating the hop delays
        // First find all unique links and put them in order in links
        System.out.println("Identify unique links");
        for (Enumeration e1 = traces.elements(); e1.hasMoreElements();) {
            Trace trace = (Trace) e1.nextElement();
            InetAddress prev = null; // trace.from.getIp();

            for (Enumeration e2 = trace.hops.elements(); e2.hasMoreElements();) {
                Hop hop = (Hop) e2.nextElement();
                InetAddress current = hop.getIp();

                if (prev != null && current != null) {
                    Pair link = new Pair(prev, current);
                    int i = links.indexOf(link);

                    if (i < 0) {
                        links.add(link);
                        g.addEdge(link);
                        i = links.size() - 1;
                    }
                }
            }
        }

        int numLinks = links.size();

        System.out.println("Building " + traces.size() + " equations");

        Matrix A = new Matrix(traces.size(), numLinks);
        Matrix B = new Matrix(traces.size(), 1);

        int i = 0;

        for (Enumeration e1 = traces.elements(); e1.hasMoreElements(); i++) {
            Trace trace = (Trace) e1.nextElement();

            B.set(i, 0, trace.time);

            InetAddress prev = null; // trace.from.getIp();

            for (Enumeration e2 = trace.hops.elements(); e2.hasMoreElements();) {
                Hop hop = (Hop) e2.nextElement();
                InetAddress current = hop.getIp();

                if (prev != null && current != null) {
                    Pair link = new Pair(prev, current);
                    int j = links.indexOf(link);

                    if (j < 0) {
                        throw new Exception("This link should have been identified already!");
                    }
		    
                    A.set(i, j, A.get(i, j) + 1);
                }
            }
        }

        System.out.println("Finished building eqns");
        A.print(5, 1);
        B.print(5, 1);
        System.out.println("Solving equations");
        Matrix x = A.qr().solve(B);

        x.print(5, 1);

        /*
         g.addEdge(new String(prev), new String(host));
         if (prev.equals("myweb") || prev.equals("wxs") || prev.equals("everyday"))
         System.out.println("From "+prev+" to "+host);
         }
         prev = host;
         }
         }
         */
        return g;	 
    }

    public static DelayGraph tracesToDelayGraph(Vector traces) throws IOException, Exception {
        DelayGraph g = new DelayGraph();

        g.addNodes(TestBox.getTestBoxSet());
        for (Enumeration e1 = traces.elements(); e1.hasMoreElements();) {
            Trace trace = (Trace) e1.nextElement();

            g.addEndToEnd(trace);
        }

        return g;
    }

    public static void main(String[] args) {
        try {
            System.setProperty("line.separator", "\n");
            Vector traces = new Vector();

            //	      // Read traces
            //	      readTracesFromDir(traces, "L:\\Data\\Tracing\\traces\\data\\test3", Trace.TRACE_FORMAT_LINUX);
            //	      readTraces(traces, "L:\\Data\\Tracing\\traces\\data");
            //	      Date date = new java.util.Date(99,8,1,20,0); // Sept 1, 1999, 20:00
            Date date = new java.util.Date(101, 3, 1, 15, 0); 

            TTMData ttm = new TTMData(DATABASE_URL);

            //	      traces.add(ttm.getTrace(1,27, new java.util.Date(99,8,1,20,0))); // Sept 1, 1999, 20:00

            //	      traces = ttm.getPartialTracesSnapshot(10,20, date);
            /*
             Trace trace = ttm.getTrace(1,27, date); 
             if (trace!=null) traces.add(trace);
             trace = ttm.getTrace(27,1, date); 
             if (trace!=null) traces.add(trace);
             trace = ttm.getTrace(1,15, date); 
             if (trace!=null) traces.add(trace);
             trace = ttm.getTrace(15,1, date); 
             if (trace!=null) traces.add(trace);
             trace = ttm.getTrace(27,15, date); 
             if (trace!=null) traces.add(trace);
             trace = ttm.getTrace(15,27, date); 
             if (trace!=null) traces.add(trace);
             */
            //	      for (int i=10; i<20; i++) {
            //		  Trace tr = ttm.getTrace(15,i, new java.util.Date(100,11,8,20,0));
            //		  if (tr!=null) traces.add(tr);
            //	      }
            //	      traces = ttm.getPartialTracesSnapshot(10,19, new java.util.Date(99,8,1,20,0));
            traces = ttm.getTracesSnapshot(date);

            // Convert to graph
            //	      Graph graphIP = tracesToGraph(traces);
            //	      Graph graph = tracesToASGraph(traces);
            //	      Graph graph = tracesToRouteGraph(traces);
            //	      Graph graph = tracesToDelayGraph(traces);
            DelayGraph graph = tracesToDelayGraph(traces);

            graph.removeUnconnectedNodes();

            System.out.println("Number of nodes in IP graph: " + graph.nodes.size() + "; number of edges: " + graph.edges.size());
            //	      System.out.println(graph.toString());

            // The following is also needed to fill in the AS fields in the Hosts.
            ASHash hash;

            if (new File(AS_DATA_FILE).canRead()) {
                try {
                    hash = ASHash.readASHash(AS_DATA_FILE);
                    //		      Log.log("Read hash: " + hash.toString());
                } catch (Exception e) {
                    Log.log("Failed reading AS data, starting with empty cache");
                    //		      e.printStackTrace();
                    hash = new ASHash(true);
                }
            } else {
                Log.log("No AS data, starting with empty cache");
                hash = new ASHash(true);
            }
            DelayGraph graph2 = graph.reduceUsingAutonomousSystems(hash);

            try {
                hash.consolidate();
                ASHash.writeASHash(hash, AS_DATA_FILE);
            } catch (Exception e) {
                Log.log("Failed writing AS data");
                e.printStackTrace();
            }
            System.out.println("Number of nodes in AS graph: " + graph2.nodes.size() + "; number of edges: " + graph2.edges.size());

            // Export to files
            String dateString = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT).format(date).replace(':', '.') + " CET";

            graph.addEdge(new Host("42.42.42.42", "DUMMY"), TestBox.getTestBox(1));
            String directory = "L:\\data\\tracing\\traces\\test\\";
            String basename = "traces"; //"small-10-20";  //
            String datetime = dateString; // "20000301-15.00"; // "19990901-20.00"; // 

            graph.exportXML(directory + basename + "-" + datetime + "-IP.xml");
            graph.exportOtter(directory + basename + "-" + datetime + "-IP.odf");
            graph.removeDullNodes();
            System.out.println("Number of nodes in IP graph without dull nodes: " + graph.nodes.size() + "; number of edges: " + graph.edges.size());
            graph.exportXML(directory + basename + "-" + datetime + "-IP-nodull.xml");
            graph.exportOtter(directory + basename + "-" + datetime + "-IP-nodull.odf");
            AutSystem.discrete = true;
            graph2.addEdge(new AutSystem("DUMMY", 42), graph2.nodes.iterator().next());
            graph2.exportXML(directory + basename + "-" + datetime + "-ASnumber.xml");
            graph2.exportOtter(directory + basename + "-" + datetime + "-ASnumber.odf");
            AutSystem.discrete = false;
            graph2.exportXML(directory + basename + "-" + datetime + "-ASname.xml");
            graph2.exportOtter(directory + basename + "-" + datetime + "-ASname.odf");
            graph2.removeDullNodes();
            System.out.println("Number of nodes in AS graph without dull nodes: " + graph2.nodes.size() + "; number of edges: " + graph2.edges.size());
            AutSystem.discrete = true;
            graph2.exportXML(directory + basename + "-" + datetime + "-ASnumber-nodull.xml");
            graph2.exportOtter(directory + basename + "-" + datetime + "-ASnumber-nodull.odf");
            AutSystem.discrete = false;
            graph2.exportXML(directory + basename + "-" + datetime + "-ASname-nodull.xml");
            graph2.exportOtter(directory + basename + "-" + datetime + "-ASname-nodull.odf");

            /*
             graph2.buildEquations();
             */
        } catch (Throwable t) {
            t.printStackTrace();
        }
        System.exit(0);
    }
}

